//----------------------------------------------------------------------------
//  Project Test_At3dx_Viewer
//  MicroMouse Productions
//  Copyright  2010-2017. All Rights Reserved.
//  Author: Wayne Hogue
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdlib.h>
#include <stdio.h>
#include <dos.h>

#include "Test_At3dx_LocationsApp.h"
#include "at3dx_my_functions.h"
#include "at3ddll.h"

#define COLOR_PANEL_LINE              0
#define COLOR_PANEL_VERTEX1           1
#define COLOR_PANEL_VERTEX2           2
#define COLOR_PANEL_OBSTRUCTION       3
#define COLOR_PANEL_OBSERVER_LINE     4
#define COLOR_PANEL_OBSERVER_VERTEX   5

#define DISPLAY_RUBBER_CLEAR_ALL       -1
#define DISPLAY_RUBBER_MOUSE_HOVER      0
#define DISPLAY_RUBBER_MOUSE_DOWN       1
#define DISPLAY_RUBBER_MOUSE_MOVE       2
#define DISPLAY_RUBBER_MOUSE_UP         3
#define DISPLAY_RUBBER_MOUSE_SWAP       4
#define DISPLAY_RUBBER_MOUSE_CLIP       5
#define DISPLAY_RUBBER_OBSERVER_ADD     6
#define DISPLAY_RUBBER_OBSERVER_CANCEL  7

//---------------------------------------------------------------------------

// To be able to send XYZ locations for where the mouse is clicked on a mesh
// the DLL has to have a message number to  use in the Windows OS PostMessage(...)
// this number must be >= WM_USER.  See Windows documentation for more
// information on the PostMessage(...) function
// Select a message number that does not conflict with any other message numbers
// that your application may be using
//
#define WM_MESH_POSITION    WM_USER + 7

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTest_At3dx_LocationsForm *Test_At3dx_LocationsForm;
//---------------------------------------------------------------------------

__fastcall TTest_At3dx_LocationsForm::TTest_At3dx_LocationsForm(TComponent* Owner)
  : TForm(Owner)
{
  sender_W8057_Flag = 0;
  shift_W8057_Flag  = 0;
  done_W8057_Flag   = 0;

  spare1 = 0;

  dllOpen    = 0;
  fileLoaded = 0;

  my_memset(inName, 0, sizeof(char) * MAX_PATH);

  inputFileType = 0;

  // Display textures
  //
  useTexturesFlag = 1;

  // default setting for camera cursor
  //
  alwaysShowCameraCursorFlag = 0;

  // if set, touch screen is available
  //
  touchScreenInputFlag = 0;

  // --

  rgbLineArray[0] = 1.0f;
  rgbLineArray[1] = 0.0f;
  rgbLineArray[2] = 0.0f;

  rgbPoint1Array[0] = 1.0f;
  rgbPoint1Array[1] = 0.0f;
  rgbPoint1Array[2] = 1.0f;

  rgbPoint2Array[0] = 0.0f;
  rgbPoint2Array[1] = 1.0f;
  rgbPoint2Array[2] = 1.0f;

  rgbObstructionArray[0] = 1.0f;
  rgbObstructionArray[1] = 1.0f;
  rgbObstructionArray[2] = 0.0f;

  rgbLineObserverArray[0] = 0.0f;
  rgbLineObserverArray[1] = 0.0f;
  rgbLineObserverArray[2] = 1.0f;

  rgbPointObserverArray[0] = 0.0f;
  rgbPointObserverArray[1] = 1.0f;
  rgbPointObserverArray[2] = 0.0f;

  lineWidth     = 2.0f;
  observerWidth = 2.0f;

  pointSize = 4.0f;
  pointObstructionSize = 4.0f;
  observerSize = 4.0f;

  depthTestFlag = 1;
  pointsFlag    = 1;

  obstructionDisplayFlag     = 1;
  observerLineFlag           = 1;
  observerPointFlag          = 1;
  haveObserverFlag           = 0;
  haveObstructionMessageFlag = 0;

  // --

  screenX0 = 0;
  screenX1 = 0;
  screenX2 = 0;
  screenXObstruction = 0;
  screenXObserver    = 0;

  screenY0 = 0;
  screenY1 = 0;
  screenY2 = 0;
  screenYObstruction = 0;
  screenYObserver    = 0;

  hitFlag0 = 0;
  hitFlag1 = 0;
  hitFlag2 = 0;
  hitFlagObstruction = 0;
  hitFlagObserver    = 0;

  my_memset(layerName0, 0, sizeof(char) * LEN_LAYER_NAME);
  my_memset(layerName1, 0, sizeof(char) * LEN_LAYER_NAME);
  my_memset(layerName2, 0, sizeof(char) * LEN_LAYER_NAME);
  my_memset(layerNameObstruction, 0, sizeof(char) * LEN_LAYER_NAME);
  my_memset(layerNameNoData, 0, sizeof(char) * LEN_LAYER_NAME);

  for (Int32 i=0; i<3; i++)
  {
    xyzArray0[i] = 0.0;
    xyzArray1[i] = 0.0;
    xyzArray2[i] = 0.0;
    xyzArrayObstruction[i] = 0.0;
    xyzArrayObserver[i]    = 0.0;

    xyzDeltaArray[i] = 0.0;
    xyzDeltaArrayObstruction[i] = 0.0;
    xyzDeltaArrayObserver[i] = 0.0;
    xyzDeltaArrayNoData[i] = 0.0;
  }

  rubberbandDistance  = 0.0;
  obstructionDistance = 0.0;
  nodataDistance  = 0.0;
  obstructionSkipDistance = 0.0;

  // --

  cameraXViewpoint = 0.0;
  cameraYViewpoint = 0.0;
  cameraZViewpoint = 0.0;

  cameraFieldOfView = 30.0;
  cameraStepFOV     = 1.0;

  cameraRotateSpeed = CAMERA_ROTATE_SPEED_2;

  cameraXAngle      = 0.0;
  cameraYAngle      = 0.0;
  cameraZAngle      = 0.0;
  cameraStepAngle   = 5.0;

  cameraXPan    = 0;
  cameraZPan    = 0;
  cameraStepPan = 1;

  reversePanDirectionFlag  = 0;

  reverseZoomDirectionFlag = 0;

  cameraZoomSpeed = CAMERA_ZOOM_SPEED_6;

  roamingZoomLock = ROAMING_LOCK_ZOOM_BOTH;

  zoomAutomaticFlag = 1;

  cameraTotal     = 0;
  activeCameraID  = 0;
  cameraID        = 0;
  getFromCameraID = 0;

  roaming_asDisplayed_Flag = 0;

  cameraInitialDistance    = 0.0;
  cameraDistance           = 0.0;

  xRotation_Lock_Flag = 0;
  yRotation_Lock_Flag = 0;
  zRotation_Lock_Flag = 0;

  xRotation_Constrain_Flag = 0;
  yRotation_Constrain_Flag = 0;
  zRotation_Constrain_Flag = 0;

  xRotation_Constrain_From = 0.0;
  xRotation_Constrain_To   = 0.0;

  yRotation_Constrain_From = 0.0;
  yRotation_Constrain_To   = 0.0;

  zRotation_Constrain_From = 0.0;
  zRotation_Constrain_To   = 0.0;

  for (Int32 i=0; i<3; i++)
  {
    roaming_Eye[i]    = 0.0;
    roaming_Center[i] = 0.0;
    roaming_Up[i]     = 0.0;
  }

  moveAheadBack_Step = 1.0;
  moveRightLeft_Step = 0.1;
  moveUpDown_Step    = 1.0;
  moveTurn_Step      = 5.0;

  // --

  Form_Setup();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Form_Setup(void)
{
#if defined(_WIN64)
  #ifdef _DEBUG
    const char* cptr = " (x64-Debug)";
  #else
    const char* cptr = " (x64)";
  #endif
#else
  #ifdef _DEBUG
    const char* cptr = " (x32-Debug)";
  #else
    const char* cptr = " (x32)";
  #endif
#endif

  Caption = Caption + cptr;

  Application->OnMessage = AppMessage;
  Application->OnIdle    = MyIdleHandler;

  // --

  // clear all location data on display
  //
  Display_Rubberband_Data(DISPLAY_RUBBER_CLEAR_ALL);

  // set intial file type to read
  //
  if (!inputFileType)
  {
    TypeObj->Checked = 1;
  }
  else
  {
    TypeWrl->Checked = 1;
  }

  // --

  Start_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::FormDestroy(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  // free all memory used by the DLL
  //
  if (dllOpen) Stop_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AppMessage(MSG &Msg, bool &Handled)
{
  if ( (Msg.message == WM_MESH_POSITION) &&
       (dllOpen) )
  {
    if (Msg.lParam == 1)
    {
       if (Msg.wParam == 0)
       {
         // event: move with no mouse buttons down
         //
         Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_HOVER);
       }
       else if (Msg.wParam == 1)
       {
         // event: left mouse button down
         //
         Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_DOWN);
       }
       else if (Msg.wParam == 2)
       {
         // event: move with left button down
         //
         Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_MOVE);
       }
       else if (Msg.wParam == 3)
       {
         // event: left button up
         //
         Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_UP);
       }
    }

    Handled = true;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MyIdleHandler(TObject *Sender, bool &Done)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;
  done_W8057_Flag   = (Done) ? 1 : 0;

  // --

  ZoomAutomatic->Down = zoomAutomaticFlag > 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read3dFileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Read_3D_File(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MultiReadClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Read_3D_File(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Start_DLL(void)
{
  if (dllOpen) return;

  dllOpen = At3d_Start();

  if (!dllOpen) return;

  // To detect mouse scroll wheel messages the DLL has to know the
  // handle to the main window of the application which
  // receives all the messages from the Windows Operating System
  //
  //At3d_SetAppHandle(Handle);

  // To be able to send XYZ locations for where the mouse is clicked on a mesh
  // the DLL has to know the handle to the main window of the application which
  // receives all the messages from the Windows Operating System
  // plus a message number to indicate the type of message being sent
  //
  At3d_Init_Mesh_Location(Handle, WM_MESH_POSITION);

  // Set parameters for rubberband line
  //
  Do_RGB_Panel(COLOR_PANEL_LINE);
  Do_RGB_Panel(COLOR_PANEL_VERTEX1);
  Do_RGB_Panel(COLOR_PANEL_VERTEX2);

  LineWidth->Text = lineWidth;
  PointSize->Text = pointSize;

  UseDepthTest->Checked = (depthTestFlag) ? 1 : 0;

  DisplayVertices->Checked = (pointsFlag) ? 1 : 0;

  At3d_Set_Mesh_Location_Parameters(&rgbLineArray[0], &rgbPoint1Array[0],
                                    &rgbPoint2Array[0], lineWidth, pointSize,
                                    depthTestFlag, pointsFlag);

  // Set parameters for obstuction point on rubberband line
  //
  Do_RGB_Panel(COLOR_PANEL_OBSTRUCTION);

  ObstructionPointSize->Text = pointObstructionSize;

  DisplayObstructionVertex->Checked = (obstructionDisplayFlag) ? 1 : 0;

  ObstructionSkipDistance->Text = obstructionSkipDistance;

  At3d_Set_Mesh_Obstruction_Parameters(rgbObstructionArray, pointObstructionSize,
                                       obstructionDisplayFlag, obstructionSkipDistance);

  // set parameters for observer on rubberband line
  //
  Do_RGB_Panel(COLOR_PANEL_OBSERVER_LINE);
  Do_RGB_Panel(COLOR_PANEL_OBSERVER_VERTEX);

  ObserverLineWidth->Text = observerWidth;
  ObserverPointSize->Text = observerSize;

  DisplayObserverLine->Checked   = (observerLineFlag) ? 1 : 0;
  DisplayObserverVertex->Checked = (observerPointFlag) ? 1 : 0;

  At3d_Set_Mesh_Location_Observer_Parameters( rgbLineObserverArray, rgbPointObserverArray,
                                              observerWidth, observerSize,
                                              observerLineFlag, observerPointFlag );

  // set deltas for obseerver on rubberband line
  //
  Observer_DeltaX->Text = xyzDeltaArrayObserver[0];
  Observer_DeltaY->Text = xyzDeltaArrayObserver[1];
  Observer_DeltaZ->Text = xyzDeltaArrayObserver[2];

  // Set size of DLL window
  //
  At3d_Size(600, 500);

  // Set position of DLL window
  //
  At3d_Position(16, 26);

  // Make DLL window a child of a window in the application
  //
  At3d_SetParent(ViewPanel->Handle);

  // If the DLL window does not have a parent
  // At3d_SetParent(0);
  // a button will appear on the Windows Taskbar
  // use this function to set the name that will appear
  //
  At3d_SetCaption("DLL (at3d)");

  // Use the OpenGL view window
  //
  At3d_WriteFlag(LIBFLAG_USE_OPENGL, 1);

  // Light both sides of the polygons
  //
  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, 1);

  // Cull back faces of polygons
  //
  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES,1);

  UseTextures->Checked = (useTexturesFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);

  // Display default texture if there is no texture file assigned
  //
  At3d_WriteFlag(LIBFLAG_OGL_USE_DEFAULT_TEXTURE, 1);

  // Let texture colors be affected by the lighting
  //
  At3d_WriteFlag(LIBFLAG_OGL_REPLACE, 0);

  // Do not blend polygon color with the texture colors
  //
  At3d_WriteFlag(LIBFLAG_OGL_WHITE_BASE, 1);

  AlwaysShowCameraCursor->Checked = (alwaysShowCameraCursorFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);

  TouchScreenInput->Checked = (touchScreenInputFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, touchScreenInputFlag);

  // Measuring distances (LIBFLAG_DISPLAY_MESH_LOCATION)
  // between locations on mesh is OFF by default
  //
  Locations_Off->Checked = 1;

  // --

  CameraXViewpoint->Text = cameraXViewpoint;
  CameraYViewpoint->Text = cameraYViewpoint;
  CameraZViewpoint->Text = cameraZViewpoint;

  CameraFieldOfView->Text = cameraFieldOfView;
  CameraStepFOV->Text     = cameraStepFOV;

  if (cameraRotateSpeed == CAMERA_ROTATE_SPEED_1)
  {
    CameraRotateSpeed->Text = "R1";
  }
  else
  {
    CameraRotateSpeed->Text = "R2";
  }

  CameraXAngle->Text = cameraXAngle;
  CameraYAngle->Text = cameraYAngle;
  CameraZAngle->Text = cameraZAngle;

  CameraStepAngle->Text = cameraStepAngle;

  CameraXPan->Text = cameraXPan;
  CameraZPan->Text = cameraZPan;

  CameraStepPan->Text = cameraStepPan;

  CameraInitialDistance->Text = cameraInitialDistance;
  CameraDistance->Text        = cameraDistance;

  if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_1)
  {
    CameraZoomSpeed->Text = "Z1";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_2)
  {
    CameraZoomSpeed->Text = "Z2";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_3)
  {
    CameraZoomSpeed->Text = "Z3";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_4)
  {
    CameraZoomSpeed->Text = "Z4";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_5)
  {
    CameraZoomSpeed->Text = "Z5";
  }
  else
  {
    CameraZoomSpeed->Text = "Z6";
  }

  At3d_WriteFlag(LIBFLAG_CAMERA_ZOOM_SPEED_AUTOMATIC, zoomAutomaticFlag);

  if (roamingZoomLock == ROAMING_LOCK_ZOOM_NONE)
  {
    Zoom_Lock_None->Checked = 1;
  }
  else if (roamingZoomLock == ROAMING_LOCK_ZOOM_IN)
  {
    Zoom_Lock_In->Checked = 1;
  }
  else if (roamingZoomLock == ROAMING_LOCK_ZOOM_OUT)
  {
    Zoom_Lock_Out->Checked = 1;
  }
  else //if (roamingZoomLock == ROAMING_LOCK_ZOOM_BOTH)
  {
    Zoom_Lock_Both->Checked = 1;
  }

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

  // create roaming cameras
  //
  cameraTotal = At3d_Add_Roaming_Cameras(3);
  Total_Cameras->Text = cameraTotal;
  if (cameraTotal > 1)
  {
    cameraID = 1;
  }
  CameraID->Text = cameraID;
  Get_From_CameraID->Text = getFromCameraID;

  Set_CameraID_List();
  Set_ActiveCameraID_List();

  Display_Roaming_Parameters();

  Move_AheadBack_Step->Text = moveAheadBack_Step;
  Move_RightLeft_Step->Text = moveRightLeft_Step;
  Move_UpDown_Step->Text    = moveUpDown_Step;
  Move_Turn_Step->Text      = moveTurn_Step;

  // --

  // By default, set viewpoint is set to use middle mouse button
  //
  SetViewpoint_Middle->Checked = 1;

  // By default, left button toggle is set to rotate
  //
  LeftToggle_Rotate->Checked = 1;

  // --

  // get axis rotation parameters
  //
  Get_Axis_XClick(NULL);
  Get_Axis_YClick(NULL);
  Get_Axis_ZClick(NULL);

  // --

  // Make DLL window visible
  //
  At3d_Show();

  // set tab sheet
  //
  PageControl1->ActivePage = LocationsSheet;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Stop_DLL(void)
{
  if (!dllOpen) return;

  // free all memory used by the DLL
	//
  At3d_Stop();
  dllOpen = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_3D_File(Int32 multiFlag)
{
  if (!dllOpen) return;

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
	char dir0[_MAX_DIR];
	char name0[_MAX_FNAME];
	char ext0[_MAX_EXT];

  if (inName[0] != 0)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr;
  const char* inputExtensionPtr;
  Int32 inputType;

  if (!inputFileType)
  {
    inputNamePtr      = "Wavefront";
    inputExtensionPtr = ".obj";
    inputType         = INTYPE_WAVEFRONT;
  }
  else
  {
    inputNamePtr      = "VRML";
    inputExtensionPtr = ".wrl";
    inputType         = INTYPE_VRML;
  }

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(0);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  if (multiFlag)
  {
    dlg->Options = dlg->Options << ofAllowMultiSelect;
  }

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

 	if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;
    Int32 errorFlag = 0;
    Int32 count = 0;
    char** filesList = NULL;

    if (multiFlag)
    {
      count = dlg->Files->Count;

      try
      {
        filesList = new char*[count];
      }
      catch(...)
      {
        filesList = NULL;
      }

      if (filesList)
      {
        my_memset(filesList, 0, sizeof(char**) * (UInt32)count);

        for (Int32 i=0; i<count; i++)
        {
          try
          {
            filesList[i] = new char[LEN_FILE_PATH];
          }
          catch(...)
          {
            filesList[i] = NULL;
            errorFlag++;
            break;
          }
        }

        if (!errorFlag)
        {
          for (Int32 i=0; i<count; i++)
          {
            my_copystring_to_ansi_from_vcl(&fileName, dlg->Files->Strings[i]);
            my_strcpy(filesList[i], MAX_PATH, fileName.c_str());
          }
        }
      }
    }
    else
    {
      my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
      my_strcpy(inName, MAX_PATH, fileName.c_str());
    }

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // display messages in statusbar
    // plus message for removing duplicate vertices, edges and polygons
    //
    At3d_Show_Statusbar();
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATES_MSG, 1);

    // read file
    //
    if (multiFlag)
    {
      if ( (filesList) && (!errorFlag) )
      {
        fileLoaded = At3d_Multi_Read(inputType, count, LEN_FILE_PATH,
                                      &filesList[0], 1.0);
      }
    }
    else
    {
      fileLoaded = At3d_Read(inputType, inName, 1.0);
    }

    // turn off statusbar
    //
    At3d_Hide_Statusbar();

    if (fileLoaded)
    {
      Get_All_ActiveCamera_Data();

      // Measuring distances (LIBFLAG_DISPLAY_MESH_LOCATION)
      // between locations on mesh is turned OFF when a file is cleared from
      // memory when a new file is read
      //
      // Had to add the PostMessage(...) functions for the radio buttons
      // to work correctly if Measure, Pause or Continue radio button
      // had been selected before the file was read
      //
      PostMessage(Locations_Off->Handle, WM_LBUTTONDOWN, 0, 0);
      PostMessage(Locations_Off->Handle, WM_LBUTTONUP, 0, 0);
      Locations_Off->Checked = 1;
    }

    if (filesList)
    {
      for (Int32 i=0; i<count; i++)
      {
        if (filesList[i])
        {
          delete[] filesList[i];
        }
      }

      delete[] filesList;
    }
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseTexturesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  useTexturesFlag = (UseTextures->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AlwaysShowCameraCursorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  alwaysShowCameraCursorFlag = (AlwaysShowCameraCursor->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_OffClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (Locations_Off->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_OFF);

    // clear all location data on display
    //
    Display_Rubberband_Data(DISPLAY_RUBBER_CLEAR_ALL);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_MeasureClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (Locations_Measure->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_MEASURE);

    UpdateScreenXYClick(NULL);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_PauseClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (Locations_Pause->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_PAUSE);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_ContinueClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (Locations_Continue->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_CONTINUE);

    UpdateScreenXYClick(NULL);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Locations0(void)
{
  if (!hitFlag0)
  {
    Location0_ScreenX->Caption = "";
    Location0_ScreenY->Caption = "";

    Location0_MeshX->Caption = "";
    Location0_MeshY->Caption = "";
    Location0_MeshZ->Caption = "";

    Location0_LayerName->Caption = "";
  }
  else
  {
    Location0_ScreenX->Caption = screenX0;
    Location0_ScreenY->Caption = screenY0;

    if (hitFlag0 == 2)
    {
      Location0_MeshX->Caption = xyzArray0[0];
      Location0_MeshY->Caption = xyzArray0[1];
      Location0_MeshZ->Caption = xyzArray0[2];

      Location0_LayerName->Caption = layerName0;
    }
    else
    {
      Location0_MeshX->Caption = "";
      Location0_MeshY->Caption = "";
      Location0_MeshZ->Caption = "";

      Location0_LayerName->Caption = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Locations0(void)
{
  Int32 result = At3d_Get_Mesh_Location(0, &hitFlag0, &xyzArray0[0], &screenX0,
                                        &screenY0, &xyzDeltaArrayNoData[0],
                                        &nodataDistance, layerName0);

  if (!result)
  {
    hitFlag0 = 0;
  }
  
  Display_Locations0();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Locations1(void)
{
  if (!hitFlag1)
  {
    Location1_ScreenX->Caption = "";
    Location1_ScreenY->Caption = "";

    Location1_MeshX->Caption = "";
    Location1_MeshY->Caption = "";
    Location1_MeshZ->Caption = "";

    Location1_LayerName->Caption = "";
  }
  else
  {
    Location1_ScreenX->Caption = screenX1;
    Location1_ScreenY->Caption = screenY1;

    if (hitFlag1 == 2)
    {
      Location1_MeshX->Caption = xyzArray1[0];
      Location1_MeshY->Caption = xyzArray1[1];
      Location1_MeshZ->Caption = xyzArray1[2];

      Location1_LayerName->Caption = layerName1;

      if (haveObserverFlag)
      {
        Get_LocationsObserver();
      }
    }
    else
    {
      Location1_MeshX->Caption = "";
      Location1_MeshY->Caption = "";
      Location1_MeshZ->Caption = "";

      Location1_LayerName->Caption = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Locations1(void)
{
  Int32 result = At3d_Get_Mesh_Location(1, &hitFlag1, &xyzArray1[0], &screenX1,
                                        &screenY1, &xyzDeltaArrayNoData[0],
                                        &nodataDistance, layerName1);

  if (!result)
  {
    hitFlag1 = 0;
  }

  Display_Locations1();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Locations2(void)
{
  if (!hitFlag2)
  {
    Location2_ScreenX->Caption = "";
    Location2_ScreenY->Caption = "";

    Location2_MeshX->Caption = "";
    Location2_MeshY->Caption = "";
    Location2_MeshZ->Caption = "";

    Location2_LayerName->Caption = "";

    Locations_DeltaX->Caption = "";
    Locations_DeltaY->Caption = "";
    Locations_DeltaZ->Caption = "";

    Locations_Distance->Caption = "";
  }
  else
  {
    Location2_ScreenX->Caption = screenX2;
    Location2_ScreenY->Caption = screenY2;

    if (hitFlag2 == 2)
    {
      Location2_MeshX->Caption = xyzArray2[0];
      Location2_MeshY->Caption = xyzArray2[1];
      Location2_MeshZ->Caption = xyzArray2[2];

      Locations_DeltaX->Caption = xyzDeltaArray[0];
      Locations_DeltaY->Caption = xyzDeltaArray[1];
      Locations_DeltaZ->Caption = xyzDeltaArray[2];

      Locations_Distance->Caption = rubberbandDistance;

      Location2_LayerName->Caption = layerName2;
    }
    else
    {
      Location2_MeshX->Caption = "";
      Location2_MeshY->Caption = "";
      Location2_MeshZ->Caption = "";

      Location2_LayerName->Caption = "";

      Locations_DeltaX->Caption = "";
      Locations_DeltaY->Caption = "";
      Locations_DeltaZ->Caption = "";

      Locations_Distance->Caption = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Locations2(void)
{
  Int32 result = At3d_Get_Mesh_Location(2, &hitFlag2, &xyzArray2[0], &screenX2,
                                        &screenY2, &xyzDeltaArray[0],
                                        &rubberbandDistance, layerName2);
  if (!result)
  {
    hitFlag2 = 0;
  }

  Display_Locations2();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_LocationsObstruction(void)
{
  if (hitFlagObstruction == 2)
  {
    haveObstructionMessageFlag = 0;

    Obstruction_ScreenX->Caption = screenXObstruction;
    Obstruction_ScreenY->Caption = screenYObstruction;

    Obstruction_MeshX->Caption = xyzArrayObstruction[0];
    Obstruction_MeshY->Caption = xyzArrayObstruction[1];
    Obstruction_MeshZ->Caption = xyzArrayObstruction[2];

    Obstruction_LayerName->Caption = layerNameObstruction;

    Obstruction_DeltaX->Caption = xyzDeltaArrayObstruction[0];
    Obstruction_DeltaY->Caption = xyzDeltaArrayObstruction[1];
    Obstruction_DeltaZ->Caption = xyzDeltaArrayObstruction[2];

    Obstruction_Distance->Caption = obstructionDistance;
  }
  else
  {
    if (haveObstructionMessageFlag)
    {
      haveObstructionMessageFlag = 0;
    }
    else
    {
      ObstructionMessage->Caption = "";
    }

    Obstruction_ScreenX->Caption = "";
    Obstruction_ScreenY->Caption = "";

    Obstruction_MeshX->Caption = "";
    Obstruction_MeshY->Caption = "";
    Obstruction_MeshZ->Caption = "";

    Obstruction_LayerName->Caption = "";

    Obstruction_DeltaX->Caption = "";
    Obstruction_DeltaY->Caption = "";
    Obstruction_DeltaZ->Caption = "";

    Obstruction_Distance->Caption = "";
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_LocationsObstruction(void)
{
  Int32 result = At3d_Get_Mesh_Location(3, &hitFlagObstruction, &xyzArrayObstruction[0],
                                        &screenXObstruction, &screenYObstruction,
                                        &xyzDeltaArrayObstruction[0],
                                        &obstructionDistance, layerNameObstruction);

  if ( (result) &&
       (hitFlagObstruction == 2) )
  {
    //
  }
  else
  {
    hitFlagObstruction = 0;
  }

  Display_LocationsObstruction();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_LocationsObserver(void)
{
  if (!hitFlagObserver)
  {
    Observer_ScreenX->Caption = "";
    Observer_ScreenY->Caption = "";

    Observer_MeshX->Caption = "";
    Observer_MeshY->Caption = "";
    Observer_MeshZ->Caption = "";
  }
  else
  {
    Observer_ScreenX->Caption = screenXObserver;
    Observer_ScreenY->Caption = screenYObserver;

    if (hitFlagObserver == 2)
    {
      Observer_MeshX->Caption = xyzArrayObserver[0];
      Observer_MeshY->Caption = xyzArrayObserver[1];
      Observer_MeshZ->Caption = xyzArrayObserver[2];
    }
    else
    {
      Observer_MeshX->Caption = "";
      Observer_MeshY->Caption = "";
      Observer_MeshZ->Caption = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_LocationsObserver(void)
{
  Int32 result = At3d_Get_Mesh_Location(4, &hitFlagObserver, &xyzArrayObserver[0],
                                        &screenXObserver, &screenYObserver,
                                        &xyzDeltaArrayNoData[0],
                                        &nodataDistance, layerNameNoData);

  if (!result)
  {
    hitFlagObserver = 0;
  }

  Display_LocationsObserver();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TouchScreenInputClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  touchScreenInputFlag = (TouchScreenInput->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, touchScreenInputFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewFrontClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_ViewFront();
  GetCameraAnglesClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewBackClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_ViewBack();
  GetCameraAnglesClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewRightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_ViewRight();
  GetCameraAnglesClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewLeftClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_ViewLeft();
  GetCameraAnglesClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewTopClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_ViewTop();
  GetCameraAnglesClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewBottomClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_ViewBottom();
  GetCameraAnglesClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraViewpointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_GetCameraViewPoint(&cameraXViewpoint,
                                         &cameraYViewpoint,
                                         &cameraZViewpoint);

  if (result)
  {
    CameraXViewpoint->Text = cameraXViewpoint;
    CameraYViewpoint->Text = cameraYViewpoint;
    CameraZViewpoint->Text = cameraZViewpoint;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraViewpointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    cameraXViewpoint = CameraXViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    cameraXViewpoint = 0.0;
  }

  try
  {
    cameraYViewpoint = CameraYViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    cameraYViewpoint = 0.0;
  }

  try
  {
    cameraZViewpoint = CameraZViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    cameraZViewpoint = 0.0;
  }

  At3d_SetCameraViewPoint(cameraXViewpoint,
                          cameraYViewpoint,
                          cameraZViewpoint);

  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraFieldOfViewClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_GetCameraFieldOfView(&cameraFieldOfView);

  if (result)
  {
    CameraFieldOfView->Text = cameraFieldOfView;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraFieldOfViewClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    cameraFieldOfView = CameraFieldOfView->Text.ToDouble();
  }
  catch(...)
  {
    cameraFieldOfView = 30.0;
  }

  Int32 result = At3d_SetCameraFieldOfView(0, cameraFieldOfView);

  if (result) GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraStepFOV(void)
{
  try
  {
    cameraStepFOV = CameraStepFOV->Text.ToDouble();
  }
  catch(...)
  {
    cameraStepFOV = 1.0;
  }

  CameraStepFOV->Text = cameraStepFOV;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraFOV1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepFOV();
  At3d_SetCameraFieldOfView(1, -cameraStepFOV);
  GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraFOV2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepFOV();
  At3d_SetCameraFieldOfView(1, cameraStepFOV);
  GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraRotateSpeed(void)
{
  Int32 result = At3d_GetCameraRotateSpeed(&cameraRotateSpeed);

  if (result)
  {
    if (cameraRotateSpeed == CAMERA_ROTATE_SPEED_1)
    {
      CameraRotateSpeed->Text = "R1";
    }
    else
    {
      CameraRotateSpeed->Text = "R2";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraRotateSpeed1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_SetCameraRotateSpeed(CAMERA_ROTATE_SPEED_1);

  if (result) GetCameraRotateSpeed();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraRotateSpeed2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_SetCameraRotateSpeed(CAMERA_ROTATE_SPEED_2);

  if (result) GetCameraRotateSpeed();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraAnglesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);

  CameraXAngle->Text = cameraXAngle;
  CameraYAngle->Text = cameraYAngle;
  CameraZAngle->Text = cameraZAngle;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraAnglesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    cameraXAngle = CameraXAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraXAngle = 0.0;
  }

  try
  {
    cameraYAngle = CameraYAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraYAngle = 0.0;
  }

  try
  {
    cameraZAngle = CameraZAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraZAngle = 0.0;
  }

  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);

  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraStepAngle(void)
{
  try
  {
    cameraStepAngle = CameraStepAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraStepAngle = 1.0;
  }

  CameraStepAngle->Text = cameraStepAngle;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXAngle1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, -cameraStepAngle, 0.0, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXAngle2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, cameraStepAngle, 0.0, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraYAngle1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, -cameraStepAngle, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraYAngle2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, cameraStepAngle, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZAngle1Click(TObject *Sender)
{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, 0.0, -cameraStepAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZAngle2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, 0.0, cameraStepAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleXClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

   At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
   cameraXAngle = 0.0;
   At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
   GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleYClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

   At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
   cameraYAngle = 0.0;
   At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
   GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleZClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

   At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
   cameraZAngle = 0.0;
   At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
   GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleXYZClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

   At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
   cameraXAngle = 0.0;
   cameraYAngle = 0.0;
   cameraZAngle = 0.0;
   At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
   GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraPanClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    cameraXPan = CameraXPan->Text.ToInt();
  }
  catch(...)
  {
    cameraXPan = 0;
  }

  try
  {
    cameraZPan = CameraZPan->Text.ToInt();
  }
  catch(...)
  {
    cameraZPan = 0;
  }

  At3d_PanCamera(0, cameraXPan, cameraZPan);

  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraStepPan(void)
{
  try
  {
    cameraStepPan = CameraStepPan->Text.ToInt();
  }
  catch(...)
  {
    cameraStepPan = 1;
  }

  CameraStepPan->Text = cameraStepPan;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXPan1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepPan();
  At3d_PanCamera(1, -cameraStepPan, 0);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXPan2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepPan();
  At3d_PanCamera(1, cameraStepPan, 0);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZPan1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepPan();
  At3d_PanCamera(1, 0, -cameraStepPan);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZPan2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  GetCameraStepPan();
  At3d_PanCamera(1, 0, cameraStepPan);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewCenterXClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_CenterX();
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewCenterZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_CenterZ();
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReversePanDirectionClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  reversePanDirectionFlag = (ReversePanDirection->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_PAN_DIRECTION, reversePanDirectionFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraInitialDistanceClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_GetCameraInitialDistance(&cameraInitialDistance);

  if (result)
  {
    CameraInitialDistance->Text = cameraInitialDistance;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraInitialDistanceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    cameraInitialDistance = CameraInitialDistance->Text.ToDouble();
  }
  catch(...)
  {
    cameraInitialDistance = 1.0;
  }

  Int32 result = At3d_SetCameraInitialDistance(cameraInitialDistance);

  if (result) GetCameraInitialDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraDistanceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_GetCameraDistance(&cameraDistance);

  if (result)
  {
    CameraDistance->Text = cameraDistance;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraDistanceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    cameraDistance = CameraDistance->Text.ToDouble();
  }
  catch(...)
  {
    cameraDistance = 1.0;
  }

  Int32 result = At3d_SetCameraDistance(cameraDistance);

  if (result) GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraZoomSpeedClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_GetCameraZoomSpeed(&cameraZoomSpeed);

  if (result)
  {
    if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_1)
    {
      CameraZoomSpeed->Text = "Z1";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_2)
    {
      CameraZoomSpeed->Text = "Z2";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_3)
    {
      CameraZoomSpeed->Text = "Z3";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_4)
    {
      CameraZoomSpeed->Text = "Z4";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_5)
    {
      CameraZoomSpeed->Text = "Z5";
    }
    else
    {
      CameraZoomSpeed->Text = "Z6";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_1);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_2);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed3Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_3);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed4Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_4);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed5Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_5);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed6Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_6);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomAutomaticClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  zoomAutomaticFlag = (zoomAutomaticFlag) ? 0 : 1;

  At3d_WriteFlag(LIBFLAG_CAMERA_ZOOM_SPEED_AUTOMATIC, zoomAutomaticFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomInClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_ZoomIn();
  GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomOutClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_ZoomOut();
  GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomResetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_ZoomReset();

  Get_All_ActiveCamera_Data();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReverseZoomDirectionClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  reverseZoomDirectionFlag = (ReverseZoomDirection->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_ZOOM_DIRECTION, reverseZoomDirectionFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_All_ActiveCamera_Data(void)
{
  GetCameraViewpointClick(NULL);

  GetCameraFieldOfViewClick(NULL);

  GetCameraAnglesClick(NULL);

  GetCameraInitialDistanceClick(NULL);

  GetCameraDistanceClick(NULL);

  GetCameraZoomSpeedClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SwapEndsClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (At3d_Swap_Ends_Mesh_Location() > 0)
  {
    UpdateObstructionClick(NULL);
    Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_SWAP);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_RGB_Panel(Int32 index)
{
  TPanel* panelPtr;
  Int32 red;
  Int32 green;
  Int32 blue;

  if (index == COLOR_PANEL_LINE)
  {
    panelPtr = Line_Panel;

    red   = my_change_float_to_256_int(rgbLineArray[0]);
    green = my_change_float_to_256_int(rgbLineArray[1]);
    blue  = my_change_float_to_256_int(rgbLineArray[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1)
  {
    panelPtr = Vertex1_Panel;

    red   = my_change_float_to_256_int(rgbPoint1Array[0]);
    green = my_change_float_to_256_int(rgbPoint1Array[1]);
    blue  = my_change_float_to_256_int(rgbPoint1Array[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2)
  {
    panelPtr = Vertex2_Panel;

    red   = my_change_float_to_256_int(rgbPoint2Array[0]);
    green = my_change_float_to_256_int(rgbPoint2Array[1]);
    blue  = my_change_float_to_256_int(rgbPoint2Array[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION)
  {
    panelPtr = Obstruction_Panel;

    red   = my_change_float_to_256_int(rgbObstructionArray[0]);
    green = my_change_float_to_256_int(rgbObstructionArray[1]);
    blue  = my_change_float_to_256_int(rgbObstructionArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE)
  {
    panelPtr = ObserverLine_Panel;

    red   = my_change_float_to_256_int(rgbLineObserverArray[0]);
    green = my_change_float_to_256_int(rgbLineObserverArray[1]);
    blue  = my_change_float_to_256_int(rgbLineObserverArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX)
  {
    panelPtr = ObserverVertex_Panel;

    red   = my_change_float_to_256_int(rgbPointObserverArray[0]);
    green = my_change_float_to_256_int(rgbPointObserverArray[1]);
    blue  = my_change_float_to_256_int(rgbPointObserverArray[2]);
  }
  else
  {
    return;
  }

  panelPtr->Color = (TColor)(red + (green << 8) + (blue << 16));
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_ColorDialog(Int32* inColorPtr, Int32* outColorPtr)
{
  if( (!inColorPtr) || (!outColorPtr) ) return;

  // --

  TColorDialog* ColorDlg;
  try
  {
    ColorDlg = new TColorDialog(0);
  }
  catch(...)
  {
    ColorDlg = NULL;
  }

  if (ColorDlg)
  {
    ColorDlg->Options = ColorDlg->Options << cdFullOpen;
    ColorDlg->Color = (TColor)((inColorPtr[0] + (inColorPtr[1] << 8) + (inColorPtr[2] << 16)));

    ColorDlg->Execute();

    outColorPtr[0] = (UInt8)(ColorDlg->Color & 0xFF);
    outColorPtr[1] = (UInt8)((ColorDlg->Color & 0xFF00) >> 8);
    outColorPtr[2] = (UInt8)((ColorDlg->Color & 0xFF0000) >> 16);

    delete ColorDlg;
  }
  else
  {
    for (Int32 i=0; i<3; i++)
    {
      outColorPtr[i] = inColorPtr[i];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Pick_Color(Int32 index)
{
  Int32 temp1[3];
  Int32 temp2[3];

  if (index == COLOR_PANEL_LINE)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineArray[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint1Array[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint1Array[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint1Array[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint2Array[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint2Array[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint2Array[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION)
  {
    temp1[0] = my_change_float_to_256_int(rgbObstructionArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbObstructionArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbObstructionArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineObserverArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineObserverArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineObserverArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX)
  {
    temp1[0] = my_change_float_to_256_int(rgbPointObserverArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbPointObserverArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbPointObserverArray[2]);
  }
  else
  {
    return;
  }

  // --

  Do_ColorDialog(temp1, temp2);

  // --

  float red   = my_change_256_int_to_float(temp2[0]);
  float green = my_change_256_int_to_float(temp2[1]);
  float blue  = my_change_256_int_to_float(temp2[2]);

  if (index == COLOR_PANEL_LINE)
  {
    rgbLineArray[0] = red;
    rgbLineArray[1] = green;
    rgbLineArray[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX1)
  {
    rgbPoint1Array[0] = red;
    rgbPoint1Array[1] = green;
    rgbPoint1Array[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX2)
  {
    rgbPoint2Array[0] = red;
    rgbPoint2Array[1] = green;
    rgbPoint2Array[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSTRUCTION)
  {
    rgbObstructionArray[0] = red;
    rgbObstructionArray[1] = green;
    rgbObstructionArray[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE)
  {
    rgbLineObserverArray[0] = red;
    rgbLineObserverArray[1] = green;
    rgbLineObserverArray[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX)
  {
    rgbPointObserverArray[0] = red;
    rgbPointObserverArray[1] = green;
    rgbPointObserverArray[2] = blue;
  }

  // --

  Do_RGB_Panel(index);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Line_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_LINE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex1_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex2_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseDepthTestClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  depthTestFlag = (UseDepthTest->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayVerticesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

 // --

   pointsFlag = (DisplayVertices->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetRubberbandParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    lineWidth = my_double_to_float(LineWidth->Text.ToDouble());
  }
  catch(...)
  {
    lineWidth = 1.0f;
  }

  try
  {
    pointSize = my_double_to_float(PointSize->Text.ToDouble());
  }
  catch(...)
  {
    pointSize = 1.0f;
  }

  // --

  At3d_Set_Mesh_Location_Parameters(&rgbLineArray[0], &rgbPoint1Array[0],
                                    &rgbPoint2Array[0], lineWidth, pointSize,
                                    depthTestFlag, pointsFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Obstruction_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSTRUCTION);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObstructionVertexClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  obstructionDisplayFlag = (DisplayObstructionVertex->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObstructionParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    pointObstructionSize = my_double_to_float(ObstructionPointSize->Text.ToDouble());
  }
  catch(...)
  {
    pointObstructionSize = 1.0f;
  }

  try
  {
    obstructionSkipDistance = ObstructionSkipDistance->Text.ToDouble();
  }
  catch(...)
  {
    obstructionSkipDistance = 0.0;
  }

  // --

  At3d_Set_Mesh_Obstruction_Parameters(rgbObstructionArray, pointObstructionSize,
                                       obstructionDisplayFlag, obstructionSkipDistance);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CheckForObstructionClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result = At3d_Get_Mesh_Obstruction_Point(&xyzArrayObstruction[0],
                                  &screenXObstruction, &screenYObstruction,
                                  &xyzDeltaArrayObstruction[0], &obstructionDistance,
                                  layerNameObstruction);

  if (result < 0)
  {
    hitFlagObstruction = 0;
    haveObstructionMessageFlag = 1;
    ObstructionMessage->Caption = "Error";
  }
  else if (!result)
  {
    hitFlagObstruction = 0;
    haveObstructionMessageFlag = 1;
    ObstructionMessage->Caption = "No obstruction point";
  }
  else
  {
    hitFlagObstruction = 2;
    haveObstructionMessageFlag = 1;
    ObstructionMessage->Caption = "Have obstruction point";
  }

  Display_LocationsObstruction();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ClipAtObstructionPointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (At3d_Clip_At_Mesh_Obstruction_Point() > 0)
  {
    Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_CLIP);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UpdateScreenXYClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 result;

  if (At3d_Update_Mesh_Location_ScreenXY() > 0)
  {
    result = At3d_Get_Mesh_Location(1, &hitFlag1, &xyzArray1[0], &screenX1,
                                    &screenY1, &xyzDeltaArrayNoData[0],
                                    &nodataDistance, layerName1);

    if ( (result) && (hitFlag1 == 2) )
    {
      Location1_ScreenX->Caption = screenX1;
      Location1_ScreenY->Caption = screenY1;
    }

    // --

    result = At3d_Get_Mesh_Location(2, &hitFlag2, &xyzArray2[0], &screenX2,
                                    &screenY2, &xyzDeltaArray[0],
                                    &rubberbandDistance, layerName2);

    if ( (result) && (hitFlag2 == 2) )
    {
      Location2_ScreenX->Caption = screenX2;
      Location2_ScreenY->Caption = screenY2;
    }

    // --

    result = At3d_Get_Mesh_Location(3, &hitFlagObstruction, &xyzArrayObstruction[0],
                                    &screenXObstruction, &screenYObstruction,
                                    &xyzDeltaArrayObstruction[0],
                                    &obstructionDistance, layerNameObstruction);

    if ( (result) && (hitFlagObstruction == 2) )
    {
      Obstruction_ScreenX->Caption = screenXObstruction;
      Obstruction_ScreenY->Caption = screenYObstruction;
    }

    // --

    result = At3d_Get_Mesh_Location(4, &hitFlagObserver, &xyzArrayObserver[0], &screenXObserver,
                                    &screenYObserver, &xyzDeltaArrayNoData[0],
                                    &nodataDistance, layerNameNoData);

    if ( (result) && (hitFlagObserver == 2) )
    {
      Observer_ScreenX->Caption = screenXObserver;
      Observer_ScreenY->Caption = screenYObserver;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UpdateObstructionClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_LocationsObstruction();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TypeObjClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  inputFileType = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TypeWrlClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  inputFileType = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint_MiddleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_SET_VIEWPOINT_MOUSE, SET_VIEWPOINT_MOUSE_MIDDLE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint_LeftClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_SET_VIEWPOINT_MOUSE, SET_VIEWPOINT_MOUSE_LEFT);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::LeftToggle_RotateClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_LEFT_MOUSE_ROTATE_PAN, LEFT_MOUSE_TOGGLE_ROTATE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::LeftToggle_PanClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_LEFT_MOUSE_ROTATE_PAN, LEFT_MOUSE_TOGGLE_PAN);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Lock_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  xRotation_Lock_Flag = (Axis_Lock_X->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Lock_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  yRotation_Lock_Flag = (Axis_Lock_Y->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Lock_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  zRotation_Lock_Flag = (Axis_Lock_Z->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Constrain_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  xRotation_Constrain_Flag = (Axis_Constrain_X->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Constrain_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  yRotation_Constrain_Flag = (Axis_Constrain_Y->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Constrain_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  zRotation_Constrain_Flag = (Axis_Constrain_Z->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_Get_Rotation_Lock(cameraID, 0, &xRotation_Lock_Flag, &xRotation_Constrain_Flag,
                         &xRotation_Constrain_From, &xRotation_Constrain_To);

  // --

  Axis_Lock_X->Checked = (xRotation_Lock_Flag) ? 1 : 0;

  Axis_Constrain_X->Checked = (xRotation_Constrain_Flag) ? 1 : 0;

  Constrain_From_X->Text = xRotation_Constrain_From;

  Constrain_To_X->Text = xRotation_Constrain_To;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    xRotation_Constrain_From = Constrain_From_X->Text.ToDouble();
  }
  catch(...)
  {
    xRotation_Constrain_From = -180.0;
  }

  try
  {
    xRotation_Constrain_To = Constrain_To_X->Text.ToDouble();
  }
  catch(...)
  {
    xRotation_Constrain_To = 180.0;
  }

  // --

  At3d_Set_Rotation_Lock(cameraID, 0, xRotation_Lock_Flag, xRotation_Constrain_Flag,
                         xRotation_Constrain_From, xRotation_Constrain_To);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_Get_Rotation_Lock(cameraID, 1, &yRotation_Lock_Flag, &yRotation_Constrain_Flag,
                         &yRotation_Constrain_From, &yRotation_Constrain_To);

  // --

  Axis_Lock_Y->Checked = (yRotation_Lock_Flag) ? 1 : 0;

  Axis_Constrain_Y->Checked = (yRotation_Constrain_Flag) ? 1 : 0;

  Constrain_From_Y->Text = yRotation_Constrain_From;

  Constrain_To_Y->Text = yRotation_Constrain_To;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    yRotation_Constrain_From = Constrain_From_Y->Text.ToDouble();
  }
  catch(...)
  {
    yRotation_Constrain_From = -180.0;
  }

  try
  {
    yRotation_Constrain_To = Constrain_To_Y->Text.ToDouble();
  }
  catch(...)
  {
    yRotation_Constrain_To = 180.0;
  }

  // --

  At3d_Set_Rotation_Lock(cameraID, 1, yRotation_Lock_Flag, yRotation_Constrain_Flag,
                         yRotation_Constrain_From, yRotation_Constrain_To);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_Get_Rotation_Lock(cameraID, 2, &zRotation_Lock_Flag, &zRotation_Constrain_Flag,
                         &zRotation_Constrain_From, &zRotation_Constrain_To);

  // --

  Axis_Lock_Z->Checked = (zRotation_Lock_Flag) ? 1 : 0;

  Axis_Constrain_Z->Checked = (zRotation_Constrain_Flag) ? 1 : 0;

  Constrain_From_Z->Text = zRotation_Constrain_From;

  Constrain_To_Z->Text = zRotation_Constrain_To;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_XYZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Axis_XClick(NULL);
  Get_Axis_YClick(NULL);
  Get_Axis_ZClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    zRotation_Constrain_From = Constrain_From_Z->Text.ToDouble();
  }
  catch(...)
  {
    zRotation_Constrain_From = -180.0;
  }

  try
  {
    zRotation_Constrain_To = Constrain_To_Z->Text.ToDouble();
  }
  catch(...)
  {
    zRotation_Constrain_To = 180.0;
  }

  // --

  At3d_Set_Rotation_Lock(cameraID, 2, zRotation_Lock_Flag, zRotation_Constrain_Flag,
                         zRotation_Constrain_From, zRotation_Constrain_To);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_XYZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Set_Axis_XClick(NULL);
  Set_Axis_YClick(NULL);
  Set_Axis_ZClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewFromV1ToV2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == 2) &&
       (hitFlag2 == 2) )
  {
    if ( (haveObserverFlag) && (hitFlagObserver == 2) )
    {
      roaming_Eye[0] = xyzArrayObserver[0];
      roaming_Eye[1] = xyzArrayObserver[1];
      roaming_Eye[2] = xyzArrayObserver[2];
    }
    else
    {
      roaming_Eye[0] = xyzArray1[0];
      roaming_Eye[1] = xyzArray1[1];
      roaming_Eye[2] = xyzArray1[2];
    }

    roaming_Center[0] = xyzArray2[0];
    roaming_Center[1] = xyzArray2[1];
    roaming_Center[2] = xyzArray2[2];

    roaming_Up[0] = 0.0;
    roaming_Up[1] = 1.0;
    roaming_Up[2] = 0.0;

    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewFromV2ToV1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == 2) &&
       (hitFlag2 == 2) )
  {
    roaming_Eye[0] = xyzArray2[0];
    roaming_Eye[1] = xyzArray2[1];
    roaming_Eye[2] = xyzArray2[2];

    if ( (haveObserverFlag) && (hitFlagObserver == 2) )
    {
      roaming_Center[0] = xyzArrayObserver[0];
      roaming_Center[1] = xyzArrayObserver[1];
      roaming_Center[2] = xyzArrayObserver[2];
    }
    else
    {
      roaming_Center[0] = xyzArray1[0];
      roaming_Center[1] = xyzArray1[1];
      roaming_Center[2] = xyzArray1[2];
    }

    roaming_Up[0] = 0.0;
    roaming_Up[1] = 1.0;
    roaming_Up[2] = 0.0;

    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverLine_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_LINE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverVertex_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_VERTEX);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverLineClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerLineFlag = (DisplayObserverLine->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverVertexClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerPointFlag = (DisplayObserverVertex->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObserverParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    observerWidth = my_double_to_float(ObserverLineWidth->Text.ToDouble());
  }
  catch(...)
  {
    observerWidth = 1.0f;
  }

  try
  {
    observerSize = my_double_to_float(ObserverPointSize->Text.ToDouble());
  }
  catch(...)
  {
    observerSize = 1.0f;
  }

  // --

  At3d_Set_Mesh_Location_Observer_Parameters( rgbLineObserverArray, rgbPointObserverArray,
                                              observerWidth, observerSize,
                                              observerLineFlag, observerPointFlag );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseObserverClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (UseObserver->Checked)
  {
    try
    {
      xyzDeltaArrayObserver[0] = Observer_DeltaX->Text.ToDouble();
    }
    catch(...)
    {
      xyzDeltaArrayObserver[0] = 0.0;
    }

    // --

    try
    {
      xyzDeltaArrayObserver[1] = Observer_DeltaY->Text.ToDouble();
    }
    catch(...)
    {
      xyzDeltaArrayObserver[1] = 0.0;
    }

    // --

    try
    {
      xyzDeltaArrayObserver[2] = Observer_DeltaZ->Text.ToDouble();
    }
    catch(...)
    {
      xyzDeltaArrayObserver[2] = 0.0;
    }

    // --

    if (At3d_Add_Mesh_Location_Observer(&xyzDeltaArrayObserver[0]) > 0)
    {
      haveObserverFlag = 1;
    }
    else
    {
      // error if return is -1
      // x,y and z are zero if return is zero
      //
      haveObserverFlag = 0;
    }

    Display_Rubberband_Data(DISPLAY_RUBBER_OBSERVER_ADD);
  }
  else
  {
    At3d_Cancel_Mesh_Location_Observer();

    haveObserverFlag = 0;

    Display_Rubberband_Data(DISPLAY_RUBBER_OBSERVER_CANCEL);
  }

  At3d_RenderOpenGLScene();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Rubberband_Data(Int32 item)
{
  if (item == DISPLAY_RUBBER_CLEAR_ALL)
  {
    hitFlag0 = 0;
    hitFlag1 = 0;
    hitFlag2 = 0;
    hitFlagObstruction = 0;
    hitFlagObserver = 0;

    Display_Locations0();
    Display_Locations1();
    Display_Locations2();
    Display_LocationsObstruction();
    Display_LocationsObserver();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_HOVER)
  {
    // event: move with no mouse buttons down
    //
    Get_Locations0();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_DOWN)
  {
    // event: left mouse button down
    //
#ifdef _DEBUG
    Get_Locations2();
    Get_LocationsObstruction();
    Get_LocationsObserver();
#else
    hitFlag2 = 0;
    Display_Locations2();

    hitFlagObstruction = 0;
    Display_LocationsObstruction();

    hitFlagObserver = 0;
    Display_LocationsObserver();
#endif

    Get_Locations1();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_MOVE)
  {
    // event: move with left button down
    //
    hitFlagObstruction = 0;
    Display_LocationsObstruction();

    Get_Locations2();
    Get_Locations0();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_UP)
  {
    // event: left button up
    //
    hitFlagObstruction = 0;
    Display_LocationsObstruction();

    Get_Locations2();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_SWAP)
  {
    Get_Locations1();
    Get_Locations2();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_CLIP)
  {
    hitFlagObstruction = 0;
    Display_LocationsObstruction();

    Get_Locations2();
  }
  else if (item == DISPLAY_RUBBER_OBSERVER_ADD)
  {
    if (haveObserverFlag)
    {
      hitFlagObstruction = 0;
      Display_LocationsObstruction();

      Get_LocationsObserver();
      Get_Locations2();
    }
    else
    {
      hitFlagObserver = 0;
      Display_LocationsObserver();
    }
  }
  else if (item == DISPLAY_RUBBER_OBSERVER_CANCEL)
  {
    hitFlagObserver = 0;
    Display_LocationsObserver();

    hitFlagObstruction = 0;
    Display_LocationsObstruction();

    Get_Locations2();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (hitFlag1 == 2)
  {
    At3d_SetCameraViewPoint(xyzArray1[0], xyzArray1[1], xyzArray1[2]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (hitFlag2 == 2)
  {
    At3d_SetCameraViewPoint(xyzArray2[0], xyzArray2[1], xyzArray2[2]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_NoneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roamingZoomLock = ROAMING_LOCK_ZOOM_NONE;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_InClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roamingZoomLock = ROAMING_LOCK_ZOOM_IN;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_OutClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roamingZoomLock = ROAMING_LOCK_ZOOM_OUT;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_BothClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roamingZoomLock = ROAMING_LOCK_ZOOM_BOTH;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateX1ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectX_90Minus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateX2ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectX_90Plus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateX3ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectX_180();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateY1ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectY_90Minus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateY2ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectY_90Plus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateY3ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectY_180();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateZ1ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectZ_90Minus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateZ2ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectZ_90Plus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateZ3ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectZ_180();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_AheadBack_Step(void)
{
  try
  {
    moveAheadBack_Step = fabs( Move_AheadBack_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveAheadBack_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_AheadClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Move_AheadBack_Step();
  At3d_Move_Roaming_AheadBack(moveAheadBack_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_BackClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Move_AheadBack_Step();
  At3d_Move_Roaming_AheadBack(-moveAheadBack_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_RightLeft_Step(void)
{
  try
  {
    moveRightLeft_Step = fabs( Move_RightLeft_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveRightLeft_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_RightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Move_RightLeft_Step();
  At3d_Move_Roaming_RightLeft(moveRightLeft_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_LeftClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Move_RightLeft_Step();
  At3d_Move_Roaming_RightLeft(-moveRightLeft_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_UpDown_Step(void)
{
  try
  {
    moveUpDown_Step = fabs( Move_UpDown_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveUpDown_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_UpClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Move_UpDown_Step();
  At3d_Move_Roaming_UpDown(moveUpDown_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_DownClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Move_UpDown_Step();
  At3d_Move_Roaming_UpDown(-moveUpDown_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_Turn_Step(void)
{
  try
  {
    moveTurn_Step = fabs( Move_Turn_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveTurn_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_Turn_RightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Move_Turn_Step();
  At3d_Move_Roaming_Turn(moveTurn_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_Turn_LeftClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Move_Turn_Step();
  At3d_Move_Roaming_Turn(-moveTurn_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Roaming_Parameters(void)
{
  Roaming_Eye_X->Text = roaming_Eye[0];
  Roaming_Eye_Y->Text = roaming_Eye[1];
  Roaming_Eye_Z->Text = roaming_Eye[2];

  Roaming_Center_X->Text = roaming_Center[0];
  Roaming_Center_Y->Text = roaming_Center[1];
  Roaming_Center_Z->Text = roaming_Center[2];

  Roaming_Up_X->Text = roaming_Up[0];
  Roaming_Up_Y->Text = roaming_Up[1];
  Roaming_Up_Z->Text = roaming_Up[2];
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Roaming_Parameters(void)
{
  try
  {
    roaming_Eye[0] = Roaming_Eye_X->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Eye[0] = 0.0;
  }

  try
  {
    roaming_Eye[1] = Roaming_Eye_Y->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Eye[1] = 0.0;
  }

  try
  {
    roaming_Eye[2] = Roaming_Eye_Z->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Eye[2] = 0.0;
  }

  // --

  try
  {
    roaming_Center[0] = Roaming_Center_X->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Center[0] = 0.0;
  }

  try
  {
    roaming_Center[1] = Roaming_Center_Y->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Center[1] = 0.0;
  }

  try
  {
    roaming_Center[2] = Roaming_Center_Z->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Center[2] = 0.0;
  }

  // --

  try
  {
    roaming_Up[0] = Roaming_Up_X->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Up[0] = 0.0;
  }

  try
  {
    roaming_Up[1] = Roaming_Up_Y->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Up[1] = 1.0;
  }

  try
  {
    roaming_Up[2] = Roaming_Up_Z->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Up[2] = 0.0;
  }

  // --

  // just displaying again to check for errors
  //
  Display_Roaming_Parameters();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (At3d_Get_Roaming_Data(cameraID, &roaming_Eye[0], &roaming_Center[0],
                            &roaming_Up[0], roaming_asDisplayed_Flag) > 0)
  {
    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_As_DisplayedClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roaming_asDisplayed_Flag = (Roaming_As_Displayed->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_Get_FromClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  try
  {
    getFromCameraID = Get_From_CameraID->Text.ToInt();
  }
  catch(...)
  {
    getFromCameraID = 0;
  }

  if (At3d_Get_Roaming_Data(getFromCameraID, &roaming_Eye[0], &roaming_Center[0],
                            &roaming_Up[0], roaming_asDisplayed_Flag) > 0)
  {
    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Roaming_Parameters();

  At3d_Set_Roaming_Data(cameraID, &roaming_Eye[0], &roaming_Center[0], &roaming_Up[0]);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CameraClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  cameraTotal = At3d_Add_Roaming_Cameras(1);
  Total_Cameras->Text = cameraTotal;

  Set_CameraID_List();
  Set_ActiveCameraID_List();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Free_Roaming_CamerasClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_Free_All_Roaming_Cameras();

  cameraTotal = At3d_Roaming_Camera_Count();
  Total_Cameras->Text = cameraTotal;

  Set_CameraID_List();
  Set_ActiveCameraID_List();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_CameraID_List(void)
{
  CameraID_List->Items->BeginUpdate();

  try
  {
    CameraID_List->Clear();

    for (Int32 i=0; i<cameraTotal; i++)
    {
      CameraID_List->Items->Add(i);
    }

    if (!cameraTotal)
    {
      cameraID = -1;
      CameraID_List->ItemIndex = -1;
    }
    else
    {
      if (cameraID >= cameraTotal) cameraID = 0;
      CameraID_List->ItemIndex = cameraID;
    }

    CameraID_List->Items->EndUpdate();
  }
  catch(...)
  {
    CameraID_List->Items->EndUpdate();
  }

  CameraID->Text = cameraID;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CameraID_ListClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  cameraID = CameraID_List->ItemIndex;

  CameraID->Text = cameraID;

  Get_Roaming_AxisClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_ActiveCameraID_List(void)
{
  Active_CameraID_List->Items->BeginUpdate();

  try
  {
    Active_CameraID_List->Clear();

    for (Int32 i=0; i<cameraTotal; i++)
    {
      Active_CameraID_List->Items->Add(i);
    }

    if (!cameraTotal)
    {
      cameraID = -1;
      Active_CameraID_List->ItemIndex = -1;
    }
    else
    {
      if (activeCameraID >= cameraTotal) activeCameraID = 0;
      Active_CameraID_List->ItemIndex = activeCameraID;
    }

    Active_CameraID_List->Items->EndUpdate();
  }
  catch(...)
  {
    Active_CameraID_List->Items->EndUpdate();
  }

  Active_CameraID->Text = activeCameraID;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Active_CameraID_ListClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  activeCameraID = Active_CameraID_List->ItemIndex;

  Active_CameraID->Text = activeCameraID;

  At3d_Switch_OGL_Camera(activeCameraID);

  Get_All_ActiveCamera_Data();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Roaming_AxisClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Roaming_GetClick(NULL);
  Get_Axis_XYZClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Roaming_AxisClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Roaming_SetClick(NULL);
  Set_Axis_XYZClick(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_Up_DefaultClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roaming_Up[0] = 0.0;
  roaming_Up[1] = 1.0;
  roaming_Up[2] = 0.0;

  Display_Roaming_Parameters();
}
//---------------------------------------------------------------------------

