//----------------------------------------------------------------------------
//  Project Test_At3dx_Viewer
//  MicroMouse Productions
//  Copyright  2010-2017. All Rights Reserved.
//  Author: Wayne Hogue
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdlib.h>
#include <stdio.h>

#include "Test_At3dx_ViewerApp.h"
#include "at3dx_my_functions.h"
#include "at3ddll.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTest_At3dx_ViewerForm *Test_At3dx_ViewerForm;
//---------------------------------------------------------------------------

__fastcall TTest_At3dx_ViewerForm::TTest_At3dx_ViewerForm(TComponent* Owner)
  : TForm(Owner)
{
  unused_W8057_Flag = 0;

  dllOpen    = 0;
  fileLoaded = 0;

  memset(inName, 0, sizeof(char) * MAX_PATH);

  // Display textures
  //
  useTexturesFlag = 1;

  // Reverse direction of pan
  //
  reversePanFlag = 1;

  // Reverse direction of zoom
  //
  reverseZoomFlag = 1;

  // default setting for camera cursor
  //
  alwaysShowCameraCursorFlag = 0;

  // if set, touch screen is available
  //
  touchScreenInputFlag = 0;

  // --

  Form_Setup();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::Form_Setup(void)
{
#if defined(_WIN64)
  #ifdef _DEBUG
    const char* cptr = " (x64-Debug)";
  #else
    const char* cptr = " (x64)";
  #endif
#else
  #ifdef _DEBUG
    const char* cptr = " (x32-Debug)";
  #else
    const char* cptr = " (x32)";
  #endif
#endif

  Caption = Caption + cptr;

  Start_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::FormDestroy(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  // free all memory used by the DLL
	//
  if (dllOpen) Stop_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::ReadObjFileClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Read_3D_File();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::Start_DLL(void)
{
  if (dllOpen) return;

  dllOpen = At3d_Start();

  if (!dllOpen) return;

  // To detect mouse scroll wheel messages the DLL has to know the
  // handle to the main window of the application which
  // receives all the messages from the Windows Operating System
  //
  At3d_SetAppHandle(Handle);

  // Set size of DLL window
  //
  At3d_Size(600, 500);

  // Set position of DLL window
  //
  At3d_Position(16, 26);

  // Make DLL window a child of a window in the application
  //
  At3d_SetParent(ViewPanel->Handle);

  // If the DLL window does not have a parent
  // At3d_SetParent(0);
  // a button will appear on the Windows Taskbar
  // use this function to set the name that will appear
  //
  At3d_SetCaption("DLL (at3d)");

  // Use the OpenGL view window
  //
  At3d_WriteFlag(LIBFLAG_USE_OPENGL, 1);

  // Light both sides of the polygons
  //
  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, 1);

  // Cull back faces of polygons
  //
  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES,1);

  UseTextures->Checked = (useTexturesFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);

  // Display default texture if there is no texture file assigned
  //
  At3d_WriteFlag(LIBFLAG_OGL_USE_DEFAULT_TEXTURE, 1);

  // Let texture colors be affected by the lighting
  //
  At3d_WriteFlag(LIBFLAG_OGL_REPLACE, 0);

  // Do not blend polygon color with the texture colors
  //
  At3d_WriteFlag(LIBFLAG_OGL_WHITE_BASE, 1);

  ReversePan->Checked = (reversePanFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_PAN_DIRECTION, reversePanFlag);

  ReverseZoom->Checked = (reverseZoomFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_ZOOM_DIRECTION, reverseZoomFlag);

  AlwaysShowCameraCursor->Checked = (alwaysShowCameraCursorFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);

  TouchScreenInput->Checked = (touchScreenInputFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, touchScreenInputFlag);

  // Make DLL window visible
  //
  At3d_Show();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::Stop_DLL(void)
{
  if (!dllOpen) return;

  // free all memory used by the DLL
	//
  At3d_Stop();
  dllOpen = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::Read_3D_File(void)
{
  if (!dllOpen) return;

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
	char dir0[_MAX_DIR];
	char name0[_MAX_FNAME];
	char ext0[_MAX_EXT];

  if (inName[0] != 0)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  AnsiString filter = "Wavefront";
  filter += "|*";
  filter += ".obj";
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(0);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

 	if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;
    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);

    my_strcpy(inName, MAX_PATH, fileName.c_str());

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // read file
    //
    fileLoaded = At3d_Read(INTYPE_WAVEFRONT, inName, 1.0);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::UseTexturesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!dllOpen) return;

  useTexturesFlag = (UseTextures->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::ReversePanClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!dllOpen) return;

  reversePanFlag = (ReversePan->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_REVERSE_PAN_DIRECTION, reversePanFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::ReverseZoomClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!dllOpen) return;

  reverseZoomFlag = (ReverseZoom->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_REVERSE_ZOOM_DIRECTION, reverseZoomFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::AlwaysShowCameraCursorClick(TObject *Sender)

{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!dllOpen) return;

  alwaysShowCameraCursorFlag = (AlwaysShowCameraCursor->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_ViewerForm::TouchScreenInputClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!dllOpen) return;

  touchScreenInputFlag = (TouchScreenInput->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, touchScreenInputFlag);
}
//---------------------------------------------------------------------------

