//----------------------------------------------------------------------------
//  Project Test_At3dx_Dll
//  MicroMouse Productions
//  Copyright  1999-2017. All Rights Reserved.
//  Author: Wayne Hogue
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <Clipbrd.hpp>

#include "Test_At3dx_DllApp.h"
#include "at3dx_my_functions.h"
#include "at3ddll.h"
#include "At3dData.h"

static CAt3dData* at3dData;
static CMyImagePanel* SnapshotPanel;

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTest_At3dx_DllForm *Test_At3dx_DllForm;
//---------------------------------------------------------------------------

__fastcall TTest_At3dx_DllForm::TTest_At3dx_DllForm(TComponent* Owner)
  : TForm(Owner)
{
  at3dData = NULL;
  SnapshotPanel = NULL;

  unused_W8057_Flag = 0;

  red   = 0.0f;
  green = 0.0f;
  blue  = 0.0f;

  dllParentHwnd = NULL;

  // --

  Form_Setup();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Form_Setup(void)
{
#if defined(_WIN64)
  #ifdef _DEBUG
    const char* cptr = " (x64-Debug)";
  #else
    const char* cptr = " (x64)";
  #endif
#else
  #ifdef _DEBUG
    const char* cptr = " (x32-Debug)";
  #else
    const char* cptr = " (x32)";
  #endif
#endif

  Caption = Caption + cptr;

  // Create a class that has the variables and functions
	// for working with the DLL
  //
  // Decide what to do if the class is not created
  //
  try
  {
	  at3dData = new CAt3dData;
  }
  catch (...)
  {
    at3dData = NULL;
  }

  if (at3dData) Start_DLL();

  PageControl1->ActivePage = ReadSheet;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::FormDestroy(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  // free all memory used by the DLL
	//
  Stop_DLL();

  // free class used for working with the DLL
	//
  if (at3dData) delete at3dData;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Start_DLL(void)
{
  if ((!at3dData) || (at3dData->dllOpen)) return;

  at3dData->dllOpen = At3d_Start();

  if (!at3dData->dllOpen) return;

  // To detect mouse scroll wheel messages the DLL has to know the
  // handle to the main window of the application which
  // receives all the messages from the Windows Operating System
  //
  At3d_SetAppHandle(Handle);

  // Set size of DLL window
  //
  At3d_Size(600, 500);

  // Set position of DLL window
  //
  At3d_Position(6, 6);

  // Make DLL window a child of a window in the application
  //
  dllParentHwnd = ViewPanel->Handle;
  At3d_SetParent(dllParentHwnd);

  // If the DLL window does not have a parent
  // At3d_SetParent(0);
  // a button will appear on the Windows Taskbar
  // use this function to set the name that will appear
  //
  At3d_SetCaption("DLL (at3d)");

  UseOpenGL->Checked = (at3dData->useOglFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_USE_OPENGL, at3dData->useOglFlag);

  DisplayOglDebugMsgs->Checked = (at3dData->oglDebugMsgFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_DISPLAY_OGL_DEBUG_MSGS, at3dData->oglDebugMsgFlag);

  PauseAtOglSnapshotMsg->Checked = (at3dData->pauseAtOglSnapshotMsgFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_DISPLAY_OGL_SNAPSHOT_DEBUG_MSGS, at3dData->pauseAtOglSnapshotMsgFlag);

  TwoSidedLighting->Checked = (at3dData->twoSidedLightingFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, at3dData->twoSidedLightingFlag);

  CullBackFaces->Checked = (at3dData->cullBackFacesFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES, at3dData->cullBackFacesFlag);

  DisplayTextures->Checked = (at3dData->displayTexturesFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, at3dData->displayTexturesFlag);

  DefaultTexture->Checked = (at3dData->defaultTextureFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_USE_DEFAULT_TEXTURE, at3dData->defaultTextureFlag);

  OglReplace->Checked = (at3dData->oglReplaceFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_REPLACE, at3dData->oglReplaceFlag);

  UseWhiteBase->Checked = (at3dData->useWhiteBaseFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_WHITE_BASE, at3dData->useWhiteBaseFlag);

  TextureSearch->Checked = (at3dData->textureSearchFlag) ? 1 : 0;

  TextureMatch->Checked = (at3dData->textureNameMatchFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_TEXTURENAME_MATCH, at3dData->textureNameMatchFlag);

  RelocateTextures->Checked = (at3dData->relocateTexturesFlag) ? 1 : 0;

  SaveTexturePath->Checked = (at3dData->saveTexturePathFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_SAVE_TEXTUREPATH, at3dData->saveTexturePathFlag);

  PartialTexturePath->Checked = (at3dData->partialTexturePathFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_PARTIAL_TEXTUREPATH, at3dData->partialTexturePathFlag);

  RemoveVertices->Checked = (at3dData->removeVerticesFlag) ? 1 : 0;
  RemoveFaces->Checked = (at3dData->removeFacesFlag) ? 1 : 0;
  ShowReadMessages->Checked = (at3dData->showReadMessagesFlag) ? 1 : 0;

  if (!at3dData->useDxfTextFlag)
  {
    DxfTextNo->Checked = 1;
  }
  else if (at3dData->useDxfTextFlag == 1)
  {
    DxfTextFlat->Checked = 1;
  }
  else
  {
    DxfTextExtruded->Checked = 1;
  }

  if (!at3dData->vrmlTextFlag)
  {
    VrmlTextNo->Checked = 1;
  }
  else if (at3dData->vrmlTextFlag == 1)
  {
    VrmlTextFlat->Checked = 1;
  }
  else
  {
    VrmlTextExtruded->Checked = 1;
  }

  DxfAllLayersOn->Checked = (at3dData->dxfAllLayersOnFlag) ? 1 : 0;

  OutputAvatar->Checked = (at3dData->outputAvatarFlag) ? 1 : 0;
  OutputHierarchy->Checked = (at3dData->outputHierarchyFlag) ? 1 : 0;
  OneLayerPerFile->Checked = (at3dData->oneLayerPerFileFlag) ? 1 : 0;
  MultiMaterial->Checked = (at3dData->multiMaterialFlag) ? 1 : 0;
  OutputQuads->Checked = (at3dData->quadFlag) ? 1 : 0;
  OutputUV->Checked = (at3dData->uvFlag) ? 1 : 0;
  OutputVNormals->Checked = (at3dData->vnormFlag) ? 1 : 0;
  OutputVColors->Checked = (at3dData->vcolorFlag) ? 1 : 0;
  SavePointClouds->Checked = (at3dData->savePointCloudFlag) ? 1 : 0;
  Save2DLines->Checked = (at3dData->save2DFlag) ? 1 : 0;
  ShowSaveMessages->Checked = (at3dData->showSaveMessagesFlag) ? 1 : 0;
  AutoDivideLayers->Checked = (at3dData->autoDivideLayersFlag) ? 1 : 0;
  NoIndent->Checked = (at3dData->noIndentFlag) ? 1 : 0;
  TextAsText->Checked = (at3dData->textAsTextFlag) ? 1 : 0;

  if (at3dData->outType3dmf == SAVE_3DMF_MESH)
  {
    Mesh3dmf->Checked = 1;
  }
  else if (at3dData->outType3dmf == SAVE_3DMF_TRIMESH)
  {
    TriMesh3dmf->Checked = 1;
  }
  else if (at3dData->outType3dmf == SAVE_3DMF_TRIANGLE)
  {
    Triangle3dmf->Checked = 1;
  }
  else if (at3dData->outType3dmf == SAVE_3DMF_QUAD)
  {
    Quad3dmf->Checked = 1;
  }

  if (at3dData->pixelFlag3dmf == PIXEL_3DMF_16)
  {
    TdmfPixel16->Checked = 1;
  }
  else if (at3dData->pixelFlag3dmf == PIXEL_3DMF_24)
  {
    TdmfPixel24->Checked = 1;
  }
  else if (at3dData->pixelFlag3dmf == PIXEL_3DMF_32)
  {
    TdmfPixel32->Checked = 1;
  }

  if (!at3dData->demSeaLevelFlag)
  {
    DemSeaLevel0->Checked = 1;
  }
  else if (at3dData->demSeaLevelFlag == 1)
  {
    DemSeaLevel1->Checked =1;
  }
  else
  {
    DemSeaLevel2->Checked = 1;
  }

  DemBelowFactor->Text = at3dData->demBelowFactor;
  DemAboveFactor->Text = at3dData->demAboveFactor;

  if (at3dData->transferFromTypeFlag == TRANSFERFROM_TYPE_BASIC)
  {
    TransferFromTypeFlag0->Checked = 1;
  }
  else if (at3dData->transferFromTypeFlag == TRANSFERFROM_TYPE_ARRAYS)
  {
    TransferFromTypeFlag1->Checked = 1;
  }
  else
  {
    TransferFromTypeFlag2->Checked = 1;
  }

  UseVertexFloatTransfer->Checked = (at3dData->vertexFloatTransferFlag) ? 1 : 0;

  UseAmbient1->Checked = (at3dData->useAmbient1Flag) ? 1 : 0;
  UseDiffuse1->Checked = (at3dData->useDiffuse1Flag) ? 1 : 0;
  UseReflection1->Checked = (at3dData->useReflection1Flag) ? 1 : 0;
  UseSpecular1->Checked = (at3dData->useSpecular1Flag) ? 1 : 0;
  UseTransparency1->Checked = (at3dData->useTransparency1Flag) ? 1 : 0;

  TextFontName->Text    = at3dData->textFaceName;
  TextEdit->Text        = at3dData->textText;
  TextUnicodeEdit->Text = at3dData->textUnicodeText;

  if (!at3dData->textLayersFlag)
  {
    TextOneLayer->Checked = 1;
  }
  else
  {
    TextMultipleLayers->Checked = 1;
  }

  TextOnCurve->Checked   = (at3dData->textOnCurveFlag) ? 1 : 0;
  TextOrigVerts->Checked = (at3dData->textOrigVertsFlag) ? 1 : 0;
  TextAllChars->Checked  = (at3dData->textAllCharsFlag) ? 1 : 0;

  TextReverseOutlines->Checked = (at3dData->textReverseOutlinesFlag) ? 1 : 0;

  TextReduceVertices->Checked = (at3dData->textReduceFlag) ? 1 : 0;
  TextJoin->Checked = (at3dData->textJoinFlag) ? 1 : 0;
  TextFace->Checked = (at3dData->textFaceFlag) ? 1 : 0;
  TextExtrude->Checked = (at3dData->textExtrudeFlag) ? 1 : 0;

  TextPoints->Text = at3dData->textPoints;
  TextHeight->Text = at3dData->textHeight;
  TextDepth->Text  = at3dData->textDepth;
  TextBorder->Text = at3dData->textBorder;

  TriQuadNormalAngle->Text = at3dData->triQuadNormalAngle;
  TriQuadEdgeAngle->Text   = at3dData->triQuadEdgeAngle;

  CameraXViewpoint->Text = at3dData->cameraXViewpoint;
  CameraYViewpoint->Text = at3dData->cameraYViewpoint;
  CameraZViewpoint->Text = at3dData->cameraZViewpoint;

  if (at3dData->cameraRotateSpeed == CAMERA_ROTATE_SPEED_1)
  {
    CameraRotateSpeed->Text = "R1";
  }
  else
  {
    CameraRotateSpeed->Text = "R2";
  }

  CameraXAngle->Text = at3dData->cameraXAngle;
  CameraYAngle->Text = at3dData->cameraYAngle;
  CameraZAngle->Text = at3dData->cameraZAngle;

  CameraStepAngle->Text = at3dData->cameraStepAngle;

  CameraInitialDistance->Text = at3dData->cameraInitialDistance;
  CameraDistance->Text        = at3dData->cameraDistance;

  if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_1)
  {
    CameraZoomSpeed->Text = "Z1";
  }
  else if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_2)
  {
    CameraZoomSpeed->Text = "Z2";
  }
  else if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_3)
  {
    CameraZoomSpeed->Text = "Z3";
  }
  else if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_4)
  {
    CameraZoomSpeed->Text = "Z4";
  }
  else if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_5)
  {
    CameraZoomSpeed->Text = "Z5";
  }
  else
  {
    CameraZoomSpeed->Text = "Z6";
  }

  ReverseZoomDirection->Checked = (at3dData->reverseZoomDirectionFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_ZOOM_DIRECTION, at3dData->reverseZoomDirectionFlag);

  ReversePanDirection->Checked = (at3dData->reversePanDirectionFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_PAN_DIRECTION, at3dData->reversePanDirectionFlag);

  CameraFieldOfView->Text = at3dData->cameraFieldOfView;
  CameraStepFOV->Text     = at3dData->cameraStepFOV;

  CameraXPan->Text = at3dData->cameraXPan;
  CameraZPan->Text = at3dData->cameraZPan;

  CameraStepPan->Text = at3dData->cameraStepPan;

  PartialTexturePath->Checked = (at3dData->originRotateFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, at3dData->originRotateFlag);

  PartialTexturePath->Checked = (at3dData->drawObjectAxesFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_DRAW_OBJECT_AXES, at3dData->drawObjectAxesFlag);

  BackgroundRed->Text   = at3dData->backgroundColor.red;
  BackgroundGreen->Text = at3dData->backgroundColor.green;
  BackgroundBlue->Text  = at3dData->backgroundColor.blue;

  InCombo->Items->BeginUpdate();
  try
  {
    InCombo->Clear();
    for (Int32 i=0; i<at3dData->inCount; i++)
    {
      InCombo->Items->Add(at3dData->inLabel[i]);
    }

    InCombo->ItemIndex = at3dData->inIndex;
    InCombo->Items->EndUpdate();
  }
  catch(...)
  {
    InCombo->Items->EndUpdate();
  }

  OutCombo->Items->BeginUpdate();
  try
  {
    OutCombo->Clear();
    for (Int32 i=0; i<at3dData->outCount; i++)
    {
      OutCombo->Items->Add(at3dData->outLabel[i]);
    }

    OutCombo->ItemIndex = at3dData->outIndex;
    OutCombo->Items->EndUpdate();
  }
  catch(...)
  {
    OutCombo->Items->EndUpdate();
  }

  RenderCombo->Items->BeginUpdate();
  try
  {
    RenderCombo->Clear();
    for (Int32 i=0; i<at3dData->oglRenderCount; i++)
    {
      RenderCombo->Items->Add(at3dData->oglRenderLabel[i]);
    }

    RenderCombo->ItemIndex = at3dData->oglRenderIndex;
    RenderCombo->Items->EndUpdate();
  }
  catch(...)
  {
    RenderCombo->Items->EndUpdate();
  }

  BackgroundCombo->Items->BeginUpdate();
  try
  {
    BackgroundCombo->Clear();
    for (Int32 i=0; i<at3dData->backgroundCount; i++)
    {
      BackgroundCombo->Items->Add(at3dData->backgroundLabel[i]);
    }

    BackgroundCombo->ItemIndex = at3dData->backgroundIndex;
    BackgroundCombo->Items->EndUpdate();
  }
  catch(...)
  {
    BackgroundCombo->Items->EndUpdate();
  }

  red = green = blue = 1.0f;
  SetEditRGB();

  SetEditTexture();

  TextureTypeCombo->Items->BeginUpdate();
  try
  {
    TextureTypeCombo->Clear();
    for (Int32 i=0; i<at3dData->textureTypeCount; i++)
    {
      TextureTypeCombo->Items->Add(at3dData->textureTypeLabel[i]);
    }

    TextureTypeCombo->ItemIndex = at3dData->textureTypeIndex;
    TextureTypeCombo->Items->EndUpdate();
  }
  catch(...)
  {
    TextureTypeCombo->Items->EndUpdate();
  }

  UseFilmstrip->Checked = (at3dData->useFilmstripFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_USE_AW_FILMSTRIP, at3dData->useFilmstripFlag);

  FilmstripTimer->Checked = (at3dData->filmstripTimerFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_AW_FILMSTRIP_TIMER, at3dData->filmstripTimerFlag);

  EditSnapKeepRelative->Checked = (at3dData->editSnapKeepRelativeFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_SNAP_KEEP_RELATIVE, at3dData->editSnapKeepRelativeFlag);

  GrabKeyboardFocus->Checked = (at3dData->grabKeyboardFocusFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_GRAB_KEYBOARD_FOCUS, at3dData->grabKeyboardFocusFlag);

  AlwaysShowCameraCursor->Checked = (at3dData->alwaysShowCameraCursorFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, at3dData->alwaysShowCameraCursorFlag);

  TouchScreenInput->Checked = (at3dData->touchScreenInputFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, at3dData->touchScreenInputFlag);

  ShowDebugStopMessages->Checked = (at3dData->debugStopMessagesFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_DEBUG_STOP_MESSAGES, at3dData->debugStopMessagesFlag);

  if (at3dData->demBitsPerPixel == 8)
  {
    DemBits8->Checked = 1;
  }
  else if (at3dData->demBitsPerPixel == 24)
  {
    DemBits24->Checked = 1;
  }
  else
  {
    DemBits32->Checked = 1;
  }

  EditMoveVertexStep->Text = at3dData->editMoveVertexStep;

  EditRotateXAngle->Text    = at3dData->editRotateXAngle;
  EditRotateYAngle->Text    = at3dData->editRotateYAngle;
  EditRotateZAngle->Text    = at3dData->editRotateZAngle;
  EditRotateStepAngle->Text = at3dData->editRotateStepAngle;

  EditScaleXFactor->Text    = at3dData->editScaleXFactor;
  EditScaleYFactor->Text    = at3dData->editScaleYFactor;
  EditScaleZFactor->Text    = at3dData->editScaleZFactor;
  EditScaleStepFactor->Text = at3dData->editScaleStepFactor;

  EditGridSize->Text = at3dData->editGridSize;

  EditLocalAxis->Checked = 1;

  Convert1_FlagStretchX->Checked = (at3dData->meshToLandscape.stretchXFlag) ? 1 : 0;
  Convert1_FlagStretchY->Checked = (at3dData->meshToLandscape.stretchYFlag) ? 1 : 0;

  if (!at3dData->meshToLandscape.convertFlag)
  {
    Convert1_KeepOriginal->Checked = 1;
  }
  else if (at3dData->meshToLandscape.convertFlag == 1)
  {
    Convert1_Landscape->Checked = 1;
  }
  else
  {
    Convert1_Mesh->Checked = 1;
  }

  Convert1_AddPointCloud->Checked = (at3dData->meshToLandscape.addCloudFlag) ? 1 : 0;
  Convert1_DropFaces->Checked = (at3dData->meshToLandscape.dropFacesFlag) ? 1 : 0;
  Convert1_AddBase->Checked = (at3dData->meshToLandscape.addBaseFlag) ? 1 : 0;
  Convert1_AddUV->Checked = (at3dData->meshToLandscape.addUVFlag) ? 1 : 0;
  Convert1_WriteXYZ->Checked = (at3dData->meshToLandscape.writeXYZFlag) ? 1 : 0;

  Convert1_XSpacing->Text = at3dData->meshToLandscape.xSpacing;
  Convert1_YSpacing->Text = at3dData->meshToLandscape.ySpacing;

  Convert1_DropElevation->Text = at3dData->meshToLandscape.dropElevation;
  Convert1_BaseThickness->Text = at3dData->meshToLandscape.baseThickness;

  Extrude2D_StepAngle->Text = at3dData->extrude2D_stepAngle;

  VrmlByChiefArchitect->Checked = (at3dData->vrmlByChiefArchitectFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_VRML_BY_CHIEF_ARCHITECT, at3dData->vrmlByChiefArchitectFlag);

  VrmlMiterJointExtrusion->Checked = (at3dData->vrmlMiterJointExtrusionFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_VRML_MITER_JOINT_EXTRUSION, at3dData->vrmlMiterJointExtrusionFlag);

  UpdateKerkytheaDisplay();

  XyzSkipDlg->Checked = (at3dData->xyzSkipDlgFlag) ? 1: 0;

  if (at3dData->xyzScanFlag == 2)
  {
    XyzDoubleZType->Checked = 1;
  }
  else if (at3dData->xyzScanFlag == 1)
  {
    XyzScanType->Checked = 1;
  }
  else
  {
    XyzMeshType->Checked = 1;
  }

  if (at3dData->xyzBinaryFileFlag)
  {
    XyzBinaryType->Checked = 1;
  }
  else
  {
    XyzAsciiType->Checked = 1;
  }

  if (at3dData->xyzMotorolaFlag)
  {
    XyzMotorola->Checked = 1;
  }
  else
  {
    XyzIntel->Checked = 1;
  }

  if (at3dData->xyzDoubleFlag)
  {
    XyzDoubleType->Checked = 1;
  }
  else
  {
    XyzFloatType->Checked = 1;
  }

  XyzRows->Text    = at3dData->xyzRows;
  XyzColumns->Text = at3dData->xyzColumns;

  if (at3dData->xyzByColumnFlag)
  {
    XyzByColumn->Checked = 1;
  }
  else
  {
    XyzByRow->Checked = 1;
  }

  XyzReadClip->Checked = (at3dData->xyzClipFlag) ? 1 : 0;

  XyzFromRow->Text = at3dData->xyzFromRow;
  XyzToRow->Text   = at3dData->xyzToRow;

  XyzFromColumn->Text = at3dData->xyzFromColumn;
  XyzToColumn->Text   = at3dData->xyzToColumn;

  if (!at3dData->xyzAxisFlag)
  {
    XyzAxis1->Checked = 1;
  }
  else if (at3dData->xyzAxisFlag == 1)
  {
    XyzAxis2->Checked = 1;
  }
  else
  {
    XyzAxis3->Checked = 1;
  }

  XyzScaleZ->Checked    = (at3dData->xyzScaleZFlag) ? 1 : 0;
  XyzScaleFactorZ->Text = at3dData->xyzScaleFactorZ;

  if (at3dData->xyzHasOptionsFlag == 1)
  {
    XyzOptions_ID->Checked = 1;
  }
  else if (at3dData->xyzHasOptionsFlag == 2)
  {
    XyzOptions_RGB->Checked = 1;
  }
  else if (at3dData->xyzHasOptionsFlag == 3)
  {
    XyzOptions_Skip->Checked = 1;
  }
  else
  {
    at3dData->xyzHasOptionsFlag = 0;
    XyzOptions_None->Checked = 1;
  }

  XyzUseId->Checked = (at3dData->xyzUseIdFlag) ? 1 : 0;

  XyzRgbIntegers->Checked = (at3dData->xyzRgbIntegersFlag) ? 1 : 0;

  XyzSkipCount->Text = at3dData->xyzSkipCount;

  XyzSaveAsDXF->Checked = (at3dData->xyzSaveDxfFlag) ? 1 : 0;

  XyzUseNoData->Checked = (at3dData->xyzNoDataFlag) ? 1 : 0;
  XyzNoDataValue->Text  = at3dData->xyzNoDataValue;

  if (!at3dData->xyzNoDataReplaceFlag)
  {
    XyzNoDataReplace1->Checked = 1;
  }
  else if (at3dData->xyzNoDataReplaceFlag == 1)
  {
    XyzNoDataReplace2->Checked = 1;
  }
  else
  {
    XyzNoDataReplace3->Checked = 1;
  }

  XyzNoDataReplaceValue->Text = at3dData->xyzNoDataReplaceValue;

  DisplayStage->Checked    = (at3dData->stageUseFlag) ? 1 : 0;
  StageUseOrigin->Checked  = (at3dData->stageOriginFlag) ? 1 : 0;
  StageUseSquares->Checked = (at3dData->stageSquaresFlag) ? 1 : 0;
  StageUseTexture->Checked = (at3dData->stageImageFlag) ? 1 : 0;

  StageGridSpacing->Text = at3dData->stageSpacing;
  StageWidth->Text       = at3dData->stageWidth;
  StageDepth->Text       = at3dData->stageDepth;
  StageOriginX->Text     = at3dData->stageOriginX;
  StageOriginY->Text     = at3dData->stageOriginY;

  ReducePolygonsStep->Text = at3dData->reducePolygonsStep;

  DisplayBackgroundImage->Checked = (at3dData->backgroundImageUseFlag) ? 1 : 0;

  BitmapTo3D_SkipDlg->Checked      = (at3dData->bitmapTo3D_SkipDlg_Flag) ? 1 : 0;
  BitmapTo3D_Decode0->Checked      = (at3dData->bitmapTo3D_Decode0_Flag) ? 1 : 0;
  BitmapTo3D_SaveOutlines->Checked = (at3dData->bitmapTo3D_SaveOutlines_Flag) ? 1 : 0;
  BitmapTo3D_JoinOutlines->Checked = (at3dData->bitmapTo3D_JoinOutlines_Flag) ? 1 : 0;
  BitmapTo3D_AddFaces->Checked     = (at3dData->bitmapTo3D_AddFaces_Flag) ? 1 : 0;
  BitmapTo3D_Extrude->Checked      = (at3dData->bitmapTo3D_Extrude_Flag) ? 1 : 0;

  BitmapTo3D_ExtrudeHeight->Text = at3dData->bitmapTo3D_ExtrudeHeight;

  BitmapToDem_SkipDlg->Checked  = (at3dData->bitmapToDem_SkipDlg_Flag) ? 1 : 0;
  BitmapToDem_SeaLevel->Checked = (at3dData->bitmapToDem_SeaLevel_Flag) ? 1 : 0;

  BitmapToDem_Multiplier1->Text = at3dData->bitmapToDem_BelowMultiplier;
  BitmapToDem_Multiplier2->Text = at3dData->bitmapToDem_AboveMultiplier;
  BitmapToDem_MinHeight->Text   = at3dData->bitmapToDem_MinHeight;
  BitmapToDem_Spacing->Text     = at3dData->bitmapToDem_XY_Spacing;

  ModoCameraSunlight->Checked = (at3dData->modoCameraSunlightFlag) ? 1 : 0;
  ModoHierarchy->Checked      = (at3dData->modoHierarchyFlag) ? 1 : 0;
  ModoWithJoints->Checked     = (at3dData->modoWithJointsFlag) ? 1 : 0;

  RevolveCameraX->Text = 0.0;
  RevolveCameraY->Text = 0.0;
  RevolveCameraZ->Text = 5.0;
  RevolveCameraSeconds->Text = 0.05;

  RotateCameraSimulateX->Text = 2;
  RotateCameraSimulateY->Text = 0;
  RotateCameraSimulateSeconds->Text = 0.025;

  at3dData->oglSelectedLight = 0;
  OglLight0->Checked = 1;


  KmlLookAtAltitudeMode->Items->BeginUpdate();
  try
  {
    KmlLookAtAltitudeMode->Clear();
    for (Int32 i=0; i<KML_ALTITUDE_MODE_COUNT; i++)
    {
      char* ptr = at3dData->kmlAltitudeModeLabels[i];
      if ((*ptr == '\0') || (ptr == 0))
      {
        break;
      }
      else
      {
        KmlLookAtAltitudeMode->Items->Add(ptr);
      }
    }

    KmlLookAtAltitudeMode->Items->EndUpdate();
  }
  catch(...)
  {
    KmlLookAtAltitudeMode->Items->EndUpdate();
  }


  KmlModelAltitudeMode->Items->BeginUpdate();
  try
  {
    KmlModelAltitudeMode->Clear();
    for (Int32 i=0; i<KML_ALTITUDE_MODE_COUNT; i++)
    {
      char* ptr = at3dData->kmlAltitudeModeLabels[i];
      if ((*ptr == '\0') || (ptr == 0))
      {
        break;
      }
      else
      {
        KmlModelAltitudeMode->Items->Add(ptr);
      }
    }

    KmlModelAltitudeMode->Items->EndUpdate();
  }
  catch(...)
  {
    KmlModelAltitudeMode->Items->EndUpdate();
  }

  if (at3dData->colladaOutputUpAxisFlag == COLLADA_UP_AXIS_X)
  {
    ColladaUpAxisX->Checked = 1;
  }
  else if (at3dData->colladaOutputUpAxisFlag == COLLADA_UP_AXIS_Y)
  {
    ColladaUpAxisY->Checked = 1;
  }
  else //if (at3dData->colladaOutputUpAxisFlag == COLLADA_UP_AXIS_Z)
  {
    ColladaUpAxisZ->Checked = 1;
  }

  CheckFixTransparencies->Checked = (at3dData->checkFixTransparenciesFlag) ? 1 : 0;

  KmzInputModelFactors->Checked = (at3dData->kmzInputModelFactorsFlag) ? 1 : 0;

  AlwaysWriteColladaTransparency->Checked = (at3dData->alwaysWriteColladaTransparencyFlag) ? 1 : 0;

  MayaVersion->Text = at3dData->mayaVersion;

  DirectxFramePrefix->Text    = at3dData->directxFramePrefix;
  DirectxMaterialPrefix->Text = at3dData->directxMaterialPrefix;

  UseDirectxFramePrefix->Checked    = (at3dData->useDirectxFramePrefixFlag) ? 1 : 0;
  UseDirectxMaterialPrefix->Checked = (at3dData->useDirectxMaterialPrefixFlag) ? 1 : 0;

  NewEmbeddedName->Clear();

  Update_KML_Page();

  if (at3dData->xamlOutputClassFlag)
  {
    XamlViewport3D->Checked = 1;
  }
  else
  {
    XamlPage->Checked = 1;
  }

  VColorDivide->Checked = (at3dData->saveCloudColorDivideFlag) ? 1 : 0;

  // Make DLL window visible
  //
  At3d_Show();

} // end At3d_Start()
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Stop_DLL(void)
{
  // free all memory used by the DLL
	//
  if ((at3dData) && (at3dData->dllOpen))
  {
    At3d_Stop();
    at3dData->dllOpen = 0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ShowWindowClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Show();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::HideWindowClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Hide();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MoveWindow1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Position(6, 6);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MoveWindow2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Position(10, 40);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SizeWindow1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Size(600, 500);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SizeWindow2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Size(350, 250);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseBBoxClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_UseWireframe(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseWiresClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_UseWireframe(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewSolidClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_PaintSolid();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DrawWireframeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_DrawWireframe();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ParallelClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ViewType(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::PerspectiveClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ViewType(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XYZOnClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_PlaneIndicator(1);
  At3d_WriteFlag(LIBFLAG_VIEW_AXES, 1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XYZOffClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_PlaneIndicator(0);
  At3d_WriteFlag(LIBFLAG_VIEW_AXES, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OnTopNoClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_FormStyle(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OnTopYesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_FormStyle(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::InComboChange(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->inIndex = InCombo->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RemoveFacesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->removeFacesFlag = (RemoveFaces->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReadClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  // when reading XYZ file and XYZ Type is Mesh
  //
  if ((at3dData->inType[at3dData->inIndex] == INTYPE_XYZ) ||
      (at3dData->inType[at3dData->inIndex] == INTYPE_XYZ_NO_MESH))
  {
    Get_XYZ_Data();
  }

  // when reading Bitmap to #d file
  //
  if ((at3dData->inType[at3dData->inIndex] == INTYPE_BMP) ||
      (at3dData->inType[at3dData->inIndex] == INTYPE_IFF))
  {
    Get_BitmapTo3D_Data();
  }

  // when reading Bitmap to Landscape file
  //
  if ((at3dData->inType[at3dData->inIndex] == INTYPE_BMP_DEM) ||
      (at3dData->inType[at3dData->inIndex] == INTYPE_IFF_DEM))
  {
    Get_BitmapToLandscape_Data();
  }

  at3dData->ReadFile();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BufferReadClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bufferReadFlag = 1;
  at3dData->ReadFile();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ShowSecurityClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ShowSecurity();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OutComboChange(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->outIndex = OutCombo->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OutputQuadsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->quadFlag = (OutputQuads->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OutputUVClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->uvFlag = (OutputUV->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OutputVNormalsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->vnormFlag = (OutputVNormals->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OutputVColorsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->vcolorFlag = (OutputVColors->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Save2DLinesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->save2DFlag = (Save2DLines->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MergeVerticesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->MergeVertices();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MergeObjectsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_MergeObjects(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MergeMaterialsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_MergeObjects(2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::WriteClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->WriteFile();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BufferWriteClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bufferWriteFlag = 1;
  at3dData->WriteFile();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ClearClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->Clear_All_Objects();

  PageControl1->ActivePage = ReadSheet;
  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DeleteLayerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->DeleteLayer();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewShadedClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_PaintShaded();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewFrontClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ViewFront();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewBackClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ViewBack();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewRightClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ViewRight();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewLeftClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ViewLeft();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewTopClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ViewTop();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewBottomClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ViewBottom();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferFromClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->TransferFromDll(0);

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::FastTransferFromClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->TransferFromDll(1);

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferClearClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->Clear_All_Objects();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferToClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->TransferToDll(0);

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::FastTransferToClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->TransferToDll(1);

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferFreeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->ClearTransferData();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MakeQuadsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  try
  {
    at3dData->triQuadNormalAngle = TriQuadNormalAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->triQuadNormalAngle = 0.0;
  }

  TriQuadNormalAngle->Text = at3dData->triQuadNormalAngle;

  try
  {
    at3dData->triQuadEdgeAngle = TriQuadEdgeAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->triQuadEdgeAngle = 130.0;
  }

  TriQuadEdgeAngle->Text = at3dData->triQuadEdgeAngle;

  At3d_SetTriQuadAngle(at3dData->triQuadNormalAngle);
  At3d_SetTriQuadEdgeAngle(at3dData->triQuadEdgeAngle);
  At3d_MakeQuads();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DeleteQuadsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  At3d_DeleteQuads();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ShowQuadsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  At3d_WriteFlag(LIBFLAG_SHOW_QUADFACE, 1);
  At3d_DrawWireframe();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::HideQuadsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  At3d_WriteFlag(LIBFLAG_SHOW_QUADFACE, 0);
  At3d_DrawWireframe();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::HiddenWireframeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  At3d_WriteFlag(LIBFLAG_HIDDEN_WIREFRAME, (HiddenWireframe->Checked) ? 1 : 0);
  At3d_DrawWireframe();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::AlignActivateClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->StartStopAlign();

  // display layer name in edit box
  //
  AlignName->Text = at3dData->alignName;

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::PresetNormalsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  At3d_AlignPresetNormals();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::AlignShowAllClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  At3d_AlignShowAll();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::AlignFlipOneClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  At3d_AlignFlipOne();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetAlignLayerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->SelectAlignLayer();
  AlignName->Text = at3dData->alignName;

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DoLayerFlipClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->FlipAlignLayer();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextSelectFontClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->SelectFont();
  TextFontName->Text = at3dData->textFaceName;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextFontDirectoryClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->GetFontDirectory();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextOneLayerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textLayersFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextMultipleLayersClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textLayersFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextCreateStencilClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textStencilFlag = (TextCreateStencil->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextPlusCoreClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textPlusCoreFlag = (TextPlusCore->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextPointsDownClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (at3dData->textPoints > 3)
  {
    at3dData->textPoints--;
    TextPoints->Text = at3dData->textPoints;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextPointsUpClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textPoints++;
  TextPoints->Text = at3dData->textPoints;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextOnCurveClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textOnCurveFlag = (TextOnCurve->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextOrigVertsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textOrigVertsFlag = (TextOrigVerts->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextAllCharsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textAllCharsFlag = (TextAllChars->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextReduceVerticesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textReduceFlag = (TextReduceVertices->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextJoinClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textJoinFlag = (TextJoin->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextFaceClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textFaceFlag = (TextFace->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextExtrudeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textExtrudeFlag = (TextExtrude->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextReverseOutlinesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textReverseOutlinesFlag = (TextReverseOutlines->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ConvertTextClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  DoConvertText(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextPasteClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!Clipboard()->HasFormat(CF_UNICODETEXT)) return;

  NativeUInt handle = Clipboard()->GetAsHandle(CF_UNICODETEXT);
  if (!handle) return;

  wchar_t* dataPtr = (wchar_t*)GlobalLock((void*)handle);
  if (!dataPtr)
  {
    GlobalUnlock((void*)handle);
    return;
  }

  wcscpy(at3dData->textUnicodeText, dataPtr);
  GlobalUnlock((void*)handle);

  TextUnicodeEdit->Text = at3dData->textUnicodeText;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ConvertUnicodeTextClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  DoConvertText(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DoConvertText(Int32 unicodeFlag)
{
  if (!unicodeFlag)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, TextEdit->Text);

    strncpy(at3dData->textText, tempName.c_str(), 255);
    at3dData->textText[255] = '\0';
  }

  at3dData->textHeight = TextHeight->Text.ToDouble();
  at3dData->textDepth  = TextDepth->Text.ToDouble();
  at3dData->textBorder = TextBorder->Text.ToDouble();

  at3dData->ConvertText(unicodeFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DepthBufferOnClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_FULL_WDW_DEPTH_BUFFER, 1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DepthBufferOffClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_FULL_WDW_DEPTH_BUFFER, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseOpenGLClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_USE_OPENGL,
                 at3dData->useOglFlag = (UseOpenGL->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------
void __fastcall TTest_At3dx_DllForm::OglLightsMaterialsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_OglLightsMaterials();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TwoSidedLightingClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING,
                 at3dData->twoSidedLightingFlag = (TwoSidedLighting->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::CullBackFacesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES,
                 at3dData->cullBackFacesFlag = (CullBackFaces->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DisplayTexturesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES,
                 at3dData->displayTexturesFlag = (DisplayTextures->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DefaultTextureClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_USE_DEFAULT_TEXTURE,
                 at3dData->defaultTextureFlag = (DefaultTexture->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OglReplaceClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_REPLACE,
                 at3dData->oglReplaceFlag = (OglReplace->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseWhiteBaseClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_WHITE_BASE,
                 at3dData->useWhiteBaseFlag = (UseWhiteBase->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RenderComboChange(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglRenderIndex = RenderCombo->ItemIndex;
  At3d_WriteFlag(LIBFLAG_OGL_RENDER, at3dData->oglRenderType[at3dData->oglRenderIndex]);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ParentWindowClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_SetParent(dllParentHwnd);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UnparentWindowClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_SetParent(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::FormPaint(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (UseOpenGL->Checked) At3d_RenderOpenGLScene();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MakeSmooth1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->MakeSmooth1();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MakeSmooth2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->MakeSmooth2();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EraseSmooth1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->EraseSmooth1();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EraseSmooth2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->EraseSmooth2();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraViewpointClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_GetCameraViewPoint(&at3dData->cameraXViewpoint,
                                         &at3dData->cameraYViewpoint,
                                         &at3dData->cameraZViewpoint);

  if (result)
  {
    CameraXViewpoint->Text = at3dData->cameraXViewpoint;
    CameraYViewpoint->Text = at3dData->cameraYViewpoint;
    CameraZViewpoint->Text = at3dData->cameraZViewpoint;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraViewpointClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->cameraXViewpoint = CameraXViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraXViewpoint = 0.0;
  }

  try
  {
    at3dData->cameraYViewpoint = CameraYViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraYViewpoint = 0.0;
  }

  try
  {
    at3dData->cameraZViewpoint = CameraZViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraZViewpoint = 0.0;
  }

  At3d_SetCameraViewPoint(at3dData->cameraXViewpoint,
                          at3dData->cameraYViewpoint,
                          at3dData->cameraZViewpoint);

  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraRotateSpeed(void)
{
  Int32 result = At3d_GetCameraRotateSpeed(&at3dData->cameraRotateSpeed);

  if (result)
  {
    if (at3dData->cameraRotateSpeed == CAMERA_ROTATE_SPEED_1)
    {
      CameraRotateSpeed->Text = "R1";
    }
    else
    {
      CameraRotateSpeed->Text = "R2";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraRotateSpeed1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_SetCameraRotateSpeed(CAMERA_ROTATE_SPEED_1);

  if (result) GetCameraRotateSpeed();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraRotateSpeed2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_SetCameraRotateSpeed(CAMERA_ROTATE_SPEED_2);

  if (result) GetCameraRotateSpeed();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraAnglesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_GetCameraAngles(&at3dData->cameraXAngle, &at3dData->cameraYAngle,
                       &at3dData->cameraZAngle);

  CameraXAngle->Text = at3dData->cameraXAngle;
  CameraYAngle->Text = at3dData->cameraYAngle;
  CameraZAngle->Text = at3dData->cameraZAngle;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraAnglesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->cameraXAngle = CameraXAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraXAngle = 0.0;
  }

  try
  {
    at3dData->cameraYAngle = CameraYAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraYAngle = 0.0;
  }

  try
  {
    at3dData->cameraZAngle = CameraZAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraZAngle = 0.0;
  }

  At3d_RotateCamera(0, at3dData->cameraXAngle, at3dData->cameraYAngle,
                    at3dData->cameraZAngle);

  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraStepAngle(void)
{
  try
  {
    at3dData->cameraStepAngle = CameraStepAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraStepAngle = 1.0;
  }
  CameraStepAngle->Text = at3dData->cameraStepAngle;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraXAngle1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepAngle();
  At3d_RotateCamera(1, -at3dData->cameraStepAngle, 0.0, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraXAngle2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepAngle();
  At3d_RotateCamera(1, at3dData->cameraStepAngle, 0.0, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraYAngle1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, -at3dData->cameraStepAngle, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraYAngle2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, at3dData->cameraStepAngle, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZAngle1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, 0.0, -at3dData->cameraStepAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZAngle2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, 0.0, at3dData->cameraStepAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraInitialDistanceClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_GetCameraInitialDistance(&at3dData->cameraInitialDistance);

  if (result)
  {
    CameraInitialDistance->Text = at3dData->cameraInitialDistance;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraInitialDistanceClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->cameraInitialDistance = CameraInitialDistance->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraInitialDistance = 1.0;
  }

  Int32 result = At3d_SetCameraInitialDistance(at3dData->cameraInitialDistance);

  if (result) GetCameraInitialDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraDistanceClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_GetCameraDistance(&at3dData->cameraDistance);

  if (result)
  {
    CameraDistance->Text = at3dData->cameraDistance;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraDistanceClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->cameraDistance = CameraDistance->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraDistance = 1.0;
  }

  Int32 result = At3d_SetCameraDistance(at3dData->cameraDistance);

  if (result) GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraZoomSpeedClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_GetCameraZoomSpeed(&at3dData->cameraZoomSpeed);

  if (result)
  {
    if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_1)
    {
      CameraZoomSpeed->Text = "Z1";
    }
    else if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_2)
    {
      CameraZoomSpeed->Text = "Z2";
    }
    else if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_3)
    {
      CameraZoomSpeed->Text = "Z3";
    }
    else if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_4)
    {
      CameraZoomSpeed->Text = "Z4";
    }
    else if (at3dData->cameraZoomSpeed == CAMERA_ZOOM_SPEED_5)
    {
      CameraZoomSpeed->Text = "Z5";
    }
    else
    {
      CameraZoomSpeed->Text = "Z6";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZoomSpeed1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_1);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZoomSpeed2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_2);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZoomSpeed3Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_3);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZoomSpeed4Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_4);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZoomSpeed5Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_5);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZoomSpeed6Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_6);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ZoomInClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ZoomIn();
  GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ZoomOutClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ZoomOut();
  GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ZoomResetClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_ZoomReset();

  GetCameraViewpointClick(Sender);
  GetCameraAnglesClick(Sender);
  GetCameraDistanceClick(Sender);
  GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraFieldOfViewClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_GetCameraFieldOfView(&at3dData->cameraFieldOfView);

  if (result)
  {
    CameraFieldOfView->Text = at3dData->cameraFieldOfView;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraFieldOfViewClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->cameraFieldOfView = CameraFieldOfView->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraFieldOfView = 30.0;
  }

  Int32 result = At3d_SetCameraFieldOfView(0, at3dData->cameraFieldOfView);

  if (result) GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraStepFOV(void)
{
  try
  {
    at3dData->cameraStepFOV = CameraStepFOV->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->cameraStepFOV = 1.0;
  }
  CameraStepFOV->Text = at3dData->cameraStepFOV;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraFOV1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepFOV();
  At3d_SetCameraFieldOfView(1, -at3dData->cameraStepFOV);
  GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraFOV2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepFOV();
  At3d_SetCameraFieldOfView(1, at3dData->cameraStepFOV);
  GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraPanClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->cameraXPan = CameraXPan->Text.ToInt();
  }
  catch(...)
  {
    at3dData->cameraXPan = 0;
  }

  try
  {
    at3dData->cameraZPan = CameraZPan->Text.ToInt();
  }
  catch(...)
  {
    at3dData->cameraZPan = 0;
  }

  At3d_PanCamera(0, at3dData->cameraXPan, at3dData->cameraZPan);

  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetCameraStepPan(void)
{
  try
  {
    at3dData->cameraStepPan = CameraStepPan->Text.ToInt();
  }
  catch(...)
  {
    at3dData->cameraStepPan = 1;
  }
  CameraStepPan->Text = at3dData->cameraStepPan;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraXPan1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepPan();
  At3d_PanCamera(1, -at3dData->cameraStepPan, 0);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraXPan2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepPan();
  At3d_PanCamera(1, at3dData->cameraStepPan, 0);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZPan1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepPan();
  At3d_PanCamera(1, 0, -at3dData->cameraStepPan);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCameraZPan2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetCameraStepPan();
  At3d_PanCamera(1, 0, at3dData->cameraStepPan);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewCenterXClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_CenterX();
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ViewCenterZClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_CenterZ();
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetMinMaxObjectClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_GetObjectMinMax(&at3dData->objectMinMax.minX);

  MinXObject->Text = at3dData->objectMinMax.minX;
  MinYObject->Text = at3dData->objectMinMax.minY;
  MinZObject->Text = at3dData->objectMinMax.minZ;

  MaxXObject->Text = at3dData->objectMinMax.maxX;
  MaxYObject->Text = at3dData->objectMinMax.maxY;
  MaxZObject->Text = at3dData->objectMinMax.maxZ;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::CopyXtoYZObjectClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  EditYObject->Text = EditXObject->Text;
  EditZObject->Text = EditXObject->Text;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::CalcCenterObjectClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_GetObjectCenter(&at3dData->objectCenter.x);

  EditXObject->Text = at3dData->objectCenter.x;
  EditYObject->Text = at3dData->objectCenter.y;
  EditZObject->Text = at3dData->objectCenter.z;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetEditXYZObject(void)
{
  try
  {
    at3dData->objectEdit.x = EditXObject->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->objectEdit.x = 0.0;
  }
  EditXObject->Text = at3dData->objectEdit.x;

  try
  {
    at3dData->objectEdit.y = EditYObject->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->objectEdit.y = 0.0;
  }
  EditYObject->Text = at3dData->objectEdit.y;

  try
  {
    at3dData->objectEdit.z = EditZObject->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->objectEdit.z = 0.0;
  }
  EditZObject->Text = at3dData->objectEdit.z;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetCenterObjectClick(TObject *Sender)
{
  GetEditXYZObject();
  At3d_SetObjectCenter(&at3dData->objectEdit.x);
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetMin1ObjectClick(TObject *Sender)
{
  At3d_SetObject_Min000();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetMin2ObjectClick(TObject *Sender)
{
  GetEditXYZObject();
  At3d_SetObject_MinXYZ(&at3dData->objectEdit.x);
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetMin3ObjectClick(TObject *Sender)
{
  At3d_SetObject_XYCenter_Z0();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetMin4ObjectClick(TObject *Sender)
{
  At3d_SetObject_XYNoChange_Z0();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetMin5ObjectClick(TObject *Sender)
{
  At3d_SetObject_XYZCenter();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetMin6ObjectClick(TObject *Sender)
{
  At3d_SetObject_XYCenter_ZNoChange();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DoScaleObjectClick(TObject *Sender)
{
  GetEditXYZObject();
  At3d_ScaleObject(&at3dData->objectEdit.x);
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ZeroAllZClick(TObject *Sender)
{
  At3d_ZeroAll_Z();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MirrorXObjectClick(TObject *Sender)
{
  At3d_MirrorObjectX();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MirrorYObjectClick(TObject *Sender)
{
  At3d_MirrorObjectY();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MirrorZObjectClick(TObject *Sender)
{
  At3d_MirrorObjectZ();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateX1ObjectClick(TObject *Sender)
{
  At3d_RotateObjectX_90Minus();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateX2ObjectClick(TObject *Sender)
{
  At3d_RotateObjectX_90Plus();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateX3ObjectClick(TObject *Sender)
{
  At3d_RotateObjectX_180();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateY1ObjectClick(TObject *Sender)
{
  At3d_RotateObjectY_90Minus();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateY2ObjectClick(TObject *Sender)
{
  At3d_RotateObjectY_90Plus();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateY3ObjectClick(TObject *Sender)
{
  At3d_RotateObjectY_180();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateZ1ObjectClick(TObject *Sender)
{
  At3d_RotateObjectZ_90Minus();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateZ2ObjectClick(TObject *Sender)
{
  At3d_RotateObjectZ_90Plus();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateZ3ObjectClick(TObject *Sender)
{
  At3d_RotateObjectZ_180();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateXYZObjectClick(TObject *Sender)
{
  At3d_RotateObjectXYZ();
  GetMinMaxObjectClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ShowReadMessagesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->showReadMessagesFlag = (ShowReadMessages->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ShowSaveMessagesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->showSaveMessagesFlag = (ShowSaveMessages->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetDefaultMaterialsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->GetDefaultMaterials();

  Ambient1->Text     = at3dData->defaultAmbient1;
  AmbientRed->Text   = at3dData->defaultAmbientRGB.red;
  AmbientGreen->Text = at3dData->defaultAmbientRGB.green;
  AmbientBlue->Text  = at3dData->defaultAmbientRGB.blue;

  ColorRed->Text   = at3dData->defaultColorRGB.red;
  ColorGreen->Text = at3dData->defaultColorRGB.green;
  ColorBlue->Text  = at3dData->defaultColorRGB.blue;

  Diffuse1->Text     = at3dData->defaultDiffuse1;
  DiffuseRed->Text   = at3dData->defaultDiffuseRGB.red;
  DiffuseGreen->Text = at3dData->defaultDiffuseRGB.green;
  DiffuseBlue->Text  = at3dData->defaultDiffuseRGB.blue;

  Specular1->Text     = at3dData->defaultSpecular1;
  SpecularRed->Text   = at3dData->defaultSpecularRGB.red;
  SpecularGreen->Text = at3dData->defaultSpecularRGB.green;
  SpecularBlue->Text  = at3dData->defaultSpecularRGB.blue;

  EmissiveRed->Text   = at3dData->defaultEmissiveRGB.red;
  EmissiveGreen->Text = at3dData->defaultEmissiveRGB.green;
  EmissiveBlue->Text  = at3dData->defaultEmissiveRGB.blue;

  Roughness->Text = at3dData->defaultRoughness;
  Shininess->Text = at3dData->defaultShininess;

  Transparency1->Text     = at3dData->defaultTransparency1;
  TransparencyRed->Text   = at3dData->defaultTransparencyRGB.red;
  TransparencyGreen->Text = at3dData->defaultTransparencyRGB.green;
  TransparencyBlue->Text  = at3dData->defaultTransparencyRGB.blue;

  IndexRefraction->Text = at3dData->defaultIndexOfRefraction;
  CreaseAngle->Text     = at3dData->defaultCreaseAngle;

  DxfColor->Text            = at3dData->defaultDxfColor;
  DoubleSided->Text         = at3dData->defaultDoubleSided;
  RwxGeometrySampling->Text = at3dData->defaultRwxGeometrySampling;
  RwxLightSampling->Text    = at3dData->defaultRwxLightSampling;
  RwxTextureModes->Text     = at3dData->defaultRwxTextureModes;
  RwxTag->Text              = at3dData->defaultRwxClumpTag;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ClearDefaultMaterialsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Ambient1->Clear();
  AmbientRed->Clear();
  AmbientGreen->Clear();
  AmbientBlue->Clear();

  ColorRed->Clear();
  ColorGreen->Clear();
  ColorBlue->Clear();

  Diffuse1->Clear();
  DiffuseRed->Clear();
  DiffuseGreen->Clear();
  DiffuseBlue->Clear();

  Specular1->Clear();
  SpecularRed->Clear();
  SpecularGreen->Clear();
  SpecularBlue->Clear();

  EmissiveRed->Clear();
  EmissiveGreen->Clear();
  EmissiveBlue->Clear();

  Roughness->Clear();
  Shininess->Clear();

  Transparency1->Clear();
  TransparencyRed->Clear();
  TransparencyGreen->Clear();
  TransparencyBlue->Clear();

  IndexRefraction->Clear();
  CreaseAngle->Clear();

  DxfColor->Clear();
  DoubleSided->Clear();
  RwxGeometrySampling->Clear();
  RwxLightSampling->Clear();
  RwxTextureModes->Clear();
  RwxTag->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BackgroundComboChange(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->backgroundIndex = BackgroundCombo->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BackgroundGetClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (at3dData->backgroundIndex == 4)
  {
    At3d_GetDemColorMap(at3dData->backgroundType[at3dData->backgroundIndex],
                        &at3dData->backgroundColor.red);
  }
  else
  {
    At3d_GetColorMap(at3dData->backgroundType[at3dData->backgroundIndex],
                     &at3dData->backgroundColor.red);
  }

  BackgroundRed->Text   = at3dData->backgroundColor.red;
  BackgroundGreen->Text = at3dData->backgroundColor.green;
  BackgroundBlue->Text  = at3dData->backgroundColor.blue;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BackgroundSetClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->backgroundColor.red = my_double_to_float(BackgroundRed->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->backgroundColor.red = 0.0f;
  }
  BackgroundRed->Text = at3dData->backgroundColor.red;

  try
  {
    at3dData->backgroundColor.green = my_double_to_float(BackgroundGreen->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->backgroundColor.green = 0.0f;
  }
  BackgroundGreen->Text = at3dData->backgroundColor.green;

  try
  {
    at3dData->backgroundColor.blue = my_double_to_float(BackgroundBlue->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->backgroundColor.blue = 0.0f;
  }
  BackgroundBlue->Text = at3dData->backgroundColor.blue;

  if (at3dData->backgroundIndex == 4)
  {
    At3d_SetDemColorMap(at3dData->backgroundType[at3dData->backgroundIndex],
                        &at3dData->backgroundColor.red);
  }
  else
  {
    At3d_SetColorMap(at3dData->backgroundType[at3dData->backgroundIndex],
                     &at3dData->backgroundColor.red);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SpecialTransferFromClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->SpecialTransferFromDll(0);

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SpecialFastTransferFromClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->SpecialTransferFromDll(1);

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SpecialTransferToClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->SpecialTransferToDll(0);

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SpecialFastTransferToClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->SpecialTransferToDll(1);

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetEditRGB(void)
{
  try
  {
    red = my_double_to_float(Red->Text.ToDouble());
  }
  catch(...)
  {
    red = 0.0f;
  }

  try
  {
    green = my_double_to_float(Green->Text.ToDouble());
  }
  catch(...)
  {
    green = 0.0f;
  }

  try
  {
    blue = my_double_to_float(Blue->Text.ToDouble());
  }
  catch(...)
  {
    blue = 0.0f;
  }

  // clamp RGB in range 0.0 to 1.0
  //
  if (red < 0.0f) red = 0.0f;
  if (red > 1.0f) red = 1.0f;

  if (green < 0.0f) green = 0.0f;
  if (green > 1.0f) green = 1.0f;

  if (blue < 0.0f) blue = 0.0f;
  if (blue > 1.0f) blue = 1.0f;

  SetEditRGB();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditRGB(void)
{
  Red->Text   = FormatFloat("0.0###;;0", red);
  Green->Text = FormatFloat("0.0###;;0", green);
  Blue->Text  = FormatFloat("0.0###;;0", blue);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ColorPickerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditRGB();

  TColorDialog* dlg;
  try
  {
    dlg = new TColorDialog(0);
  }
  catch(...)
  {
    dlg = NULL;
  }

  if (dlg)
  {
    UInt8 r = (UInt8)(red * 255.0);
    UInt8 g = (UInt8)(green * 255.0);
    UInt8 b = (UInt8)(blue * 255.0);

    dlg->Options = dlg->Options << cdFullOpen;
    dlg->Color = (TColor)((r + (g << 8) + (b << 16)));

    dlg->Execute();

    red   = ((float)(dlg->Color & 0xFF) / 255.0f);
    green = ((float)((dlg->Color & 0xFF00) >> 8) / 255.0f);
    blue  = ((float)((dlg->Color & 0xFF0000) >> 16) / 255.0f);

    SetEditRGB();
    GetDxfPenClick(Sender);

    delete dlg;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetDxfPenClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditRGB();

  // get DXF Pen that matches color
  //
  Int32 dxfPen = At3d_FindDxfPen(red, green, blue);
  DxfPen->Text = dxfPen;

  // get RGB color for DXF Pen
  //
  float dxfRGB[3];
  At3d_GetColorMap(dxfPen, dxfRGB);
  DxfRed->Text   = FormatFloat("0.0###;;0", dxfRGB[0]);
  DxfGreen->Text = FormatFloat("0.0###;;0", dxfRGB[1]);
  DxfBlue->Text  = FormatFloat("0.0###;;0", dxfRGB[2]);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextureSearchClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  // turn off this flag so texture search prompt message
  // is not displayed when textures are not found
  //
  at3dData->textureSearchFlag = (TextureSearch->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextureMatchClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textureNameMatchFlag = (TextureMatch->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_TEXTURENAME_MATCH, at3dData->textureNameMatchFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RelocateTexturesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->relocateTexturesFlag = (RelocateTextures->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SaveTexturePathClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->saveTexturePathFlag = (SaveTexturePath->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_SAVE_TEXTUREPATH, at3dData->saveTexturePathFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::PartialTexturePathClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->partialTexturePathFlag = (PartialTexturePath->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_PARTIAL_TEXTUREPATH, at3dData->partialTexturePathFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferRedKnightClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->TransferRedKnightToDll();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferGreenKnightClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->TransferGreenKnightToDll();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferBlueKnightClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->TransferBlueKnightToDll();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferGreenMonkClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->TransferGreenMonkToDll();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferBlueMonkClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->TransferBlueMonkToDll();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferMultiMatKnightClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->TransferMultiMatKnightToDll();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferTextureSquare1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->TransferTextureSquareToDll1();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferTextureSquare2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->TransferTextureSquareToDll2();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferTextureSquare3Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->TransferTextureSquareToDll3();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferFromTypeFlag0Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->transferFromTypeFlag = TRANSFERFROM_TYPE_BASIC;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferFromTypeFlag1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->transferFromTypeFlag = TRANSFERFROM_TYPE_ARRAYS;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferFromTypeFlag2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->transferFromTypeFlag = TRANSFERFROM_TYPE_XYZ_UV;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseVertexFloatTransferClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->vertexFloatTransferFlag = (UseVertexFloatTransfer->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SavePointCloudsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->savePointCloudFlag = (SavePointClouds->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetEditTexture(void)
{
  try
  {
    at3dData->textureCenter[0] = my_double_to_float(TextureCenter0->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->textureCenter[0] = 0.0f;
  }

  try
  {
    at3dData->textureCenter[1] = my_double_to_float(TextureCenter1->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->textureCenter[1] = 0.0f;
  }

  try
  {
    at3dData->textureRotation = my_double_to_float(TextureRotation->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->textureRotation = 0.0f;
  }

  try
  {
    at3dData->textureScale[0] = my_double_to_float(TextureScale0->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->textureScale[0] = 1.0f;
  }

  if (at3dData->textureScale[0] <= 0.0f) at3dData->textureScale[0] = 1.0f;

  try
  {
    at3dData->textureScale[1] = my_double_to_float(TextureScale1->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->textureScale[1] = 1.0f;
  }

  if (at3dData->textureScale[1] <= 0.0f) at3dData->textureScale[1] = 1.0f;

  try
  {
    at3dData->textureTranslation[0] = my_double_to_float(TextureTranslation0->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->textureTranslation[0] = 0.0f;
  }

  try
  {
    at3dData->textureTranslation[1] = my_double_to_float(TextureTranslation1->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->textureTranslation[1] = 0.0f;
  }

  SetEditTexture();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditTexture(void)
{
  TextureCenter0->Text = FormatFloat("0.0###;;0",
                                     at3dData->textureCenter[0]);
  TextureCenter1->Text = FormatFloat("0.0###;;0",
                                     at3dData->textureCenter[1]);

  TextureRotation->Text = FormatFloat("0.0###;;0",
                                      at3dData->textureRotation);

  TextureScale0->Text = FormatFloat("0.0###;;0",
                                    at3dData->textureScale[0]);
  TextureScale1->Text = FormatFloat("0.0###;;0",
                                    at3dData->textureScale[1]);

  TextureTranslation0->Text = FormatFloat("0.0###;;0",
                                          at3dData->textureTranslation[0]);
  TextureTranslation1->Text = FormatFloat("0.0###;;0",
                                          at3dData->textureTranslation[1]);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextureTypeComboChange(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textureTypeIndex = TextureTypeCombo->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::CreateUVClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditTexture();
  at3dData->Create_UV_Coordinates();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RemoveUVClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->Remove_UV_Coordinates();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UVDialogClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Create_UV_Dialog();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Mesh3dmfClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->outType3dmf = SAVE_3DMF_MESH;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TriMesh3dmfClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->outType3dmf = SAVE_3DMF_TRIMESH;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Triangle3dmfClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->outType3dmf = SAVE_3DMF_TRIANGLE;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Quad3dmfClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->outType3dmf = SAVE_3DMF_QUAD;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TdmfPixel16Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->pixelFlag3dmf = PIXEL_3DMF_16;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TdmfPixel24Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->pixelFlag3dmf = PIXEL_3DMF_24;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TdmfPixel32Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->pixelFlag3dmf = PIXEL_3DMF_32;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseFilmstripClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_USE_AW_FILMSTRIP,
                 at3dData->useFilmstripFlag = (UseFilmstrip->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::FilmstripTimerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_AW_FILMSTRIP_TIMER,
                 at3dData->filmstripTimerFlag = (FilmstripTimer->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::AutoDivideLayersClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->autoDivideLayersFlag = (AutoDivideLayers->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_DIVIDE_LAYER_SAVE, at3dData->autoDivideLayersFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseAmbient1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useAmbient1Flag = (UseAmbient1->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseDiffuse1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useDiffuse1Flag = (UseDiffuse1->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseReflection1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useReflection1Flag = (UseReflection1->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseSpecular1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useSpecular1Flag = (UseSpecular1->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseTransparency1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useTransparency1Flag = (UseTransparency1->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetDemInfoClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = 0;

  if (At3d_GetDemSize(&at3dData->demColumns, &at3dData->demRows) == 1)
  {
    if (At3d_GetDemSpacing(&at3dData->demXSpacing, &at3dData->demYSpacing) == 1)
    {
      if (At3d_GetDemMinMax(&at3dData->demMinHeight, &at3dData->demMaxHeight) == 1)
      {
        if (At3d_CreateDemBitmap(at3dData->demBitsPerPixel, DemPanel->Width, DemPanel->Height) == 1)
        {
          result = 1;
        }
      }
    }
  }

  if (result)
  {
    at3dData->demXOffset = 0;
    at3dData->demYOffset = 0;

    DemColumns->Text = at3dData->demColumns;
    DemRows->Text    = at3dData->demRows;

    DemXSpacing->Text = FormatFloat("0.0###;;0",
                                    at3dData->demXSpacing);
    DemYSpacing->Text = FormatFloat("0.0###;;0",
                                    at3dData->demYSpacing);

    DemMinHeight->Text = FormatFloat("0.0###;;0",
                                     at3dData->demMinHeight);

    DemMaxHeight->Text = FormatFloat("0.0###;;0",
                                      at3dData->demMaxHeight);

    DemScrollX->Max = at3dData->demColumns;
    DemScrollY->Max = at3dData->demRows;

    DemScrollX->Position = 0;
    DemScrollY->Position = 0;

    GetDemClip();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetDemClip(void)
{
  At3d_GetDemClip(DemPanel->Handle,
                  at3dData->demXOffset, at3dData->demYOffset);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DemScrollXChange(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->demXOffset = DemScrollX->Position;
  GetDemClip();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DemScrollYChange(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->demYOffset = DemScrollY->Position;
  GetDemClip();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DemPanelMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  unused_W8057_Flag = ((Sender) || (Shift.Contains(ssShift))) ? 1 : 0;

  if (!at3dData->fileLoaded)
  {
    DemEast->Clear();
    DemNorth->Clear();
    DemHeight->Clear();
    return;
  }

  Int32 demX = X + at3dData->demXOffset;

  Int32 demY = at3dData->demRows - 1 - Y - at3dData->demYOffset;

  DemEast->Text = FormatFloat("0.0#;;0",
                              demX * at3dData->demXSpacing);

  DemNorth->Text = FormatFloat("0.0#;;0",
                               demY * at3dData->demYSpacing);

  float height;

  if (At3d_GetDemHeight(demX, demY, &height) == 1)
  {
    DemHeight->Text = FormatFloat("0.0#;;0", height);
  }
  else
  {
    DemHeight->Text = "NoData";
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DemBits8Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->demBitsPerPixel = 8;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DemBits24Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->demBitsPerPixel = 24;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DemBits32Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->demBitsPerPixel = 32;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DemSeaLevel0Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->demSeaLevelFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DemSeaLevel1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->demSeaLevelFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DemSeaLevel2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->demSeaLevelFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::WriteDemContourMapClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->demBelowFactor = my_double_to_float(DemBelowFactor->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->demBelowFactor = 1.0f;
  }
  DemBelowFactor->Text = at3dData->demBelowFactor;

  try
  {
    at3dData->demAboveFactor = my_double_to_float(DemAboveFactor->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->demAboveFactor = 1.0f;
  }
  DemAboveFactor->Text = at3dData->demAboveFactor;

  at3dData->WriteDemContourMap();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OneLayerPerFileClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oneLayerPerFileFlag = (OneLayerPerFile->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MultiMaterialClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->multiMaterialFlag = (MultiMaterial->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OutputAvatarClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->outputAvatarFlag = (OutputAvatar->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RemoveVerticesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->removeVerticesFlag = (RemoveVertices->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EraseLines1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->EraseLines1();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EraseLines2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->EraseLines2();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EraseClouds1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->EraseClouds1();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EraseClouds2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->EraseClouds2();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::WatertightCheckClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Screen->Cursor = crHourGlass;

  at3dData->WatertightCheck();

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::NoIndentClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->noIndentFlag = (NoIndent->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_NOINDENT_OUT, at3dData->noIndentFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::StartEditVerticesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_Start_Edit_Vertices();

  if (result)
  {
    // first layer in list is selected first for edit vertices mode
    //
    at3dData->editNextLayer = At3d_GetNextLayer(0, at3dData->editName);
    EditName->Text = at3dData->editName;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::StopEditVerticesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Stop_Edit_Vertices();

  at3dData->editName[0]   = '\0';
	at3dData->editNextLayer = 0;
  EditName->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditExtrudeStartClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_Extrude_Mouse();

  if (result)
  {
    DisplayPickVertexMode();

    at3dData->extrudeModeFlag = 1;

    // first layer in list is selected first for edit vertices mode
    //
    at3dData->editNextLayer = At3d_GetNextLayer(0, at3dData->editName);
    EditName->Text = at3dData->editName;

    if (at3dData->bevelSetsFlag) At3d_Edit_Select_Bevel_Set(at3dData->editName);

    DisplayExtrudeBevelFlags();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DisplayExtrudeBevelFlags(void)
{
  Int32 flag = At3d_Edit_Get_Extrude_Flag(EDIT_FLAG_UPPER_FACES);
  if (flag > -1)
  {
    EditUpperFaces->Checked = (flag) ? 1 : 0;
  }

  flag = At3d_Edit_Get_Extrude_Flag(EDIT_FLAG_UPPER_BEVELS);
  if (flag > -1)
  {
    EditUpperBevels->Checked = (flag) ? 1 : 0;
  }

  flag = At3d_Edit_Get_Extrude_Flag(EDIT_FLAG_LOWER_FACES);
  if (flag > -1)
  {
    EditLowerFaces->Checked = (flag) ? 1 : 0;
  }

  flag = At3d_Edit_Get_Extrude_Flag(EDIT_FLAG_LOWER_BEVELS);
  if (flag > -1)
  {
    EditLowerBevels->Checked = (flag) ? 1 : 0;
  }

  flag = At3d_Edit_Get_Divide_Flag();
  if (flag > -1)
  {
    EditDivide->Checked = (flag) ? 1 : 0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditExtrudeNowClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Extrude_Now();

  EditDisplayGrid->Checked = 0;
  EditSnapToGrid->Checked  = 0;

  at3dData->extrudeModeFlag = 0;
  at3dData->editName[0]     = '\0';
	at3dData->editNextLayer   = 0;
  EditName->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditDragBoxClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Drag_Box();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickVertexClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Pick_Vertex();
  DisplayPickVertexMode();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickModeFirstClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Set_Pick_Mode(EDIT_PICK_VERTEX_FIRST);
  DisplayPickVertexMode();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickModeConnectedClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Set_Pick_Mode(EDIT_PICK_VERTEX_CONNECTED);
  DisplayPickVertexMode();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickModeAnywhereClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Set_Pick_Mode(EDIT_PICK_VERTEX_ANYWHERE);
  DisplayPickVertexMode();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickModeEraseClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Set_Pick_Mode(EDIT_PICK_VERTEX_ERASE);
  DisplayPickVertexMode();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickModeDeleteAllClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Set_Pick_Mode(EDIT_PICK_VERTEX_DELETE_ALL);
  DisplayPickVertexMode();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickModeSwitchFirstClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Set_Pick_Mode(EDIT_PICK_VERTEX_SWITCH_FIRST);
  DisplayPickVertexMode();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DisplayPickVertexMode(void)
{
  Int32 flag = At3d_Edit_Get_Pick_Mode();

  const char* ptr;
  switch(flag)
  {
    case EDIT_PICK_VERTEX_FIRST:
      ptr = "Pick Vertex: First";
      break;

    case EDIT_PICK_VERTEX_CONNECTED:
      ptr = "Pick Vertex: Connected";
      break;

    case EDIT_PICK_VERTEX_ANYWHERE:
      ptr = "Pick Vertex: Anywhere";
      break;

    case EDIT_PICK_VERTEX_ERASE:
      ptr = "Pick Vertex: Erase";
      break;

    case EDIT_PICK_VERTEX_DELETE_ALL:
      ptr = "Pick Vertex: Clear All";
      break;

    case EDIT_PICK_VERTEX_SWITCH_FIRST:
      ptr = "Pick Vertex: Switch First";
      break;

    default:
      ptr = "Pick Vertex: Unknown";
      break;
  }

  EditPickModeLabel->Caption = ptr;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickAllVerticesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Pick_All_Vertices();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditDragVertexClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Drag_Vertex();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickEdgeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Pick_Edge();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditNewEdgeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_New_Edge();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditDeleteVertexClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Delete_Vertex();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditDivideEdgeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Divide_Edge();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditFlipXClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Mirror_Image_Flip_X();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditFlipYClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Mirror_Image_Flip_Y();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditFlipZClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Mirror_Image_Flip_Z();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditMoveBevelZeroClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Move_Bevel_Zero();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditSwitchClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Switch_Bevels_Outlines();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditOpenOutlineClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Open_Outline();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditCloseOutlineClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Close_Outline();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickSolidClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Pick_Solid_Wire(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickWireClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Pick_Solid_Wire(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditOtherSolidClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Other_Solid_Wire(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditOtherWireClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Other_Solid_Wire(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditRotateGroupClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Rotate_Group();
  GetEditRotateAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetEditRotateAnglesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_GetRotateAngles(&at3dData->editRotateXAngle,
                            &at3dData->editRotateYAngle,
                            &at3dData->editRotateZAngle);

  EditRotateXAngle->Text = at3dData->editRotateXAngle;
  EditRotateYAngle->Text = at3dData->editRotateYAngle;
  EditRotateZAngle->Text = at3dData->editRotateZAngle;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditRotateAnglesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->editRotateXAngle = EditRotateXAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editRotateXAngle = 0.0;
  }
  EditRotateXAngle->Text = at3dData->editRotateXAngle;

  try
  {
    at3dData->editRotateYAngle = EditRotateYAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editRotateYAngle = 0.0;
  }
  EditRotateYAngle->Text = at3dData->editRotateYAngle;

  try
  {
    at3dData->editRotateZAngle = EditRotateZAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editRotateZAngle = 0.0;
  }
  EditRotateZAngle->Text = at3dData->editRotateZAngle;

  At3d_Edit_SetRotateAngles(0, at3dData->editRotateXAngle,
                               at3dData->editRotateYAngle,
                               at3dData->editRotateZAngle);

  GetEditRotateAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetEditRotateStepAngle(void)
{
  try
  {
    at3dData->editRotateStepAngle = EditRotateStepAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editRotateStepAngle = 1.0;
  }
  EditRotateStepAngle->Text = at3dData->editRotateStepAngle;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditRotateXAngle1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditRotateStepAngle();
  At3d_Edit_SetRotateAngles(1, -at3dData->editRotateStepAngle, 0.0, 0.0);
  GetEditRotateAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditRotateXAngle2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditRotateStepAngle();
  At3d_Edit_SetRotateAngles(1, at3dData->editRotateStepAngle, 0.0, 0.0);
  GetEditRotateAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditRotateYAngle1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditRotateStepAngle();
  At3d_Edit_SetRotateAngles(1, 0.0, -at3dData->editRotateStepAngle, 0.0);
  GetEditRotateAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditRotateYAngle2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditRotateStepAngle();
  At3d_Edit_SetRotateAngles(1, 0.0, at3dData->editRotateStepAngle, 0.0);
  GetEditRotateAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditRotateZAngle1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditRotateStepAngle();
  At3d_Edit_SetRotateAngles(1, 0.0, 0.0, -at3dData->editRotateStepAngle);
  GetEditRotateAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditRotateZAngle2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditRotateStepAngle();
  At3d_Edit_SetRotateAngles(1, 0.0, 0.0, at3dData->editRotateStepAngle);
  GetEditRotateAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditRotateOkayClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Rotate_Okay();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditRotateCancelClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Rotate_Cancel();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditScaleGroupClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Scale_Group();
  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetEditScaleFactorsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_GetScaleFactors(&at3dData->editScaleXFactor,
                            &at3dData->editScaleYFactor,
                            &at3dData->editScaleZFactor);

  EditScaleXFactor->Text = at3dData->editScaleXFactor;
  EditScaleYFactor->Text = at3dData->editScaleYFactor;
  EditScaleZFactor->Text = at3dData->editScaleZFactor;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditScaleFactorsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->editScaleXFactor = EditScaleXFactor->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editScaleXFactor = 1.0;
  }
  EditScaleXFactor->Text = at3dData->editScaleXFactor;

  try
  {
    at3dData->editScaleYFactor = EditScaleYFactor->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editScaleYFactor = 1.0;
  }
  EditScaleYFactor->Text = at3dData->editScaleYFactor;

  try
  {
    at3dData->editScaleZFactor = EditScaleZFactor->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editScaleZFactor = 1.0;
  }
  EditScaleZFactor->Text = at3dData->editScaleZFactor;

  At3d_Edit_SetScaleFactors(0, at3dData->editScaleXFactor,
                               at3dData->editScaleYFactor,
                               at3dData->editScaleZFactor);

  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetEditScaleStepFactor(void)
{
  try
  {
    at3dData->editScaleStepFactor = EditScaleStepFactor->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editScaleStepFactor = 0.1;
  }
  EditScaleStepFactor->Text = at3dData->editScaleStepFactor;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditScaleXFactor1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditScaleStepFactor();
  At3d_Edit_SetScaleFactors(1, -at3dData->editScaleStepFactor, 0.0, 0.0);
  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditScaleXFactor2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditScaleStepFactor();
  At3d_Edit_SetScaleFactors(1, at3dData->editScaleStepFactor, 0.0, 0.0);
  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditScaleYFactor1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditScaleStepFactor();
  At3d_Edit_SetScaleFactors(1, 0.0, -at3dData->editScaleStepFactor, 0.0);
  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditScaleYFactor2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditScaleStepFactor();
  At3d_Edit_SetScaleFactors(1, 0.0, at3dData->editScaleStepFactor, 0.0);
  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditScaleZFactor1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditScaleStepFactor();
  At3d_Edit_SetScaleFactors(1, 0.0, 0.0, -at3dData->editScaleStepFactor);
  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditScaleZFactor2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditScaleStepFactor();
  At3d_Edit_SetScaleFactors(1, 0.0, 0.0, at3dData->editScaleStepFactor);
  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditScaleXYZFactor1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditScaleStepFactor();
  At3d_Edit_SetScaleFactors(1, -at3dData->editScaleStepFactor,
                               -at3dData->editScaleStepFactor,
                               -at3dData->editScaleStepFactor);
  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditScaleXYZFactor2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditScaleStepFactor();
  At3d_Edit_SetScaleFactors(1, at3dData->editScaleStepFactor,
                               at3dData->editScaleStepFactor,
                               at3dData->editScaleStepFactor);
  GetEditScaleFactorsClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditScaleOkayClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Scale_Okay();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditScaleCancelClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Scale_Cancel();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditReduceVerticesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Reduce_Vertices();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditPickGhostClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Pick_Ghost_Outline();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditGhostOkayClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Pick_Ghost_Okay();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditGhostCancelClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Pick_Ghost_Cancel();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditGhostOnClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Use_Ghost_Outline(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditGhostOffClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Use_Ghost_Outline(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditLocalAxisClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Local_World_Axis_Switch(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditWorldAxisClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Local_World_Axis_Switch(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditVertexSnapStartClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Vertex_Snap();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditVertexSnapShowAllClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Snap_Show_Other();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditSnapKeepRelativeClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->editSnapKeepRelativeFlag = (EditSnapKeepRelative->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_SNAP_KEEP_RELATIVE, at3dData->editSnapKeepRelativeFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditVertexSnapOkayClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Snap_Okay();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditVertexSnapCancelClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Snap_Cancel();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditUpperFacesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 flag = (EditUpperFaces->Checked) ? 1 : 0;
  At3d_Edit_Set_Extrude_Flag(EDIT_FLAG_UPPER_FACES, flag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditUpperBevelsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 flag = (EditUpperBevels->Checked) ? 1 : 0;
  At3d_Edit_Set_Extrude_Flag(EDIT_FLAG_UPPER_BEVELS, flag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditLowerFacesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 flag = (EditLowerFaces->Checked) ? 1 : 0;
  At3d_Edit_Set_Extrude_Flag(EDIT_FLAG_LOWER_FACES, flag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditLowerBevelsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 flag = (EditLowerBevels->Checked) ? 1 : 0;
  At3d_Edit_Set_Extrude_Flag(EDIT_FLAG_LOWER_BEVELS, flag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditSwitchUpperClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Upper_Lower_Switch(EDIT_BEVELS_UPPER);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditSwitchLowerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Upper_Lower_Switch(EDIT_BEVELS_LOWER);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditDisplayGridClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 flag = (EditDisplayGrid->Checked) ? 1 : 0;

  try
  {
    at3dData->editGridSize = EditGridSize->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editGridSize = 0.01;
  }

  At3d_Edit_Display_Grid(flag, at3dData->editGridSize);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditSnapToGridClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 flag = (EditSnapToGrid->Checked) ? 1 : 0;

  At3d_Edit_Snap_To_Grid(flag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditUndoSelectedClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Undo_Vertices_Selected();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditUndoLayerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Undo_Vertices_Layer();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditUndoAllLayersClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Undo_Vertices_All_Layers();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditGetUpperClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->EditGetUpperBevels();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditClearUpperClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Clear_Bevels(EDIT_BEVELS_UPPER);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditSetUpperClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->EditSetUpperBevels();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditGetLowerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->EditGetLowerBevels();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditClearLowerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Clear_Bevels(EDIT_BEVELS_LOWER);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditSetLowerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->EditSetLowerBevels();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ApplyBevels1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->ApplyBevels1();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ApplyBevels2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->ApplyBevels2();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ApplyBevels3Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->ApplyBevels3();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextMakeBevelSetsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->Text_MakeBevelSets();

  // display layer names with first layer in list selected first
  //
  if (at3dData->bevelSetsFlag)
  {
    at3dData->textNextLayer = At3d_GetNextLayer(0, at3dData->textName);
    TextLayerName->Text = at3dData->textName;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextSelectLayerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  // get next layer in list
  //
  if (at3dData->bevelSetsFlag)
  {
    at3dData->textNextLayer = At3d_GetNextLayer(at3dData->textNextLayer, at3dData->textName);
    TextLayerName->Text = at3dData->textName;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditDivideClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 flag = (EditDivide->Checked) ? 1 : 0;
  At3d_Edit_Set_Divide_Flag(flag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_GetSizeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_Get_Mesh_To_Landscape_Size(&at3dData->meshToLandscape.minX);

  if (result)
  {
    Convert1_MinX->Text = at3dData->meshToLandscape.minX;
    Convert1_MinY->Text = at3dData->meshToLandscape.minY;
    Convert1_MinZ->Text = at3dData->meshToLandscape.minZ;

    Convert1_MaxX->Text = at3dData->meshToLandscape.maxX;
    Convert1_MaxY->Text = at3dData->meshToLandscape.maxY;
    Convert1_MaxZ->Text = at3dData->meshToLandscape.maxZ;

    Convert1_SizeX->Text = at3dData->meshToLandscape.sizeX;
    Convert1_SizeY->Text = at3dData->meshToLandscape.sizeY;
    Convert1_SizeZ->Text = at3dData->meshToLandscape.sizeZ;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_CalculateClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->meshToLandscape.xSpacing = Convert1_XSpacing->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->meshToLandscape.xSpacing = 10.0;
  }

  try
  {
    at3dData->meshToLandscape.ySpacing = Convert1_YSpacing->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->meshToLandscape.ySpacing = 10.0;
  }

  At3d_WriteFlag(LIBFLAG_MESH_TO_DEM_STRETCH_X,
                 at3dData->meshToLandscape.stretchXFlag);

  At3d_WriteFlag(LIBFLAG_MESH_TO_DEM_STRETCH_Y,
                 at3dData->meshToLandscape.stretchYFlag);

  Int32 result = At3d_Calculate_Mesh_To_Landscape(
                                            at3dData->meshToLandscape.xSpacing,
                                            at3dData->meshToLandscape.ySpacing,
                                            &at3dData->meshToLandscape.xCount,
                                            &at3dData->meshToLandscape.newSizeX);

  if (result)
  {
    Convert1_XCount->Text    = at3dData->meshToLandscape.xCount;
    Convert1_YCount->Text    = at3dData->meshToLandscape.yCount;
    Convert1_VertCount->Text = at3dData->meshToLandscape.vertCount;

    Convert1_MeshVertices->Text  = at3dData->meshToLandscape.vertCount;
    Convert1_MeshEdges->Text     = at3dData->meshToLandscape.edgeCount;
    Convert1_MeshTriangles->Text = at3dData->meshToLandscape.triangleCount;

    Convert1_NewSizeX->Text = at3dData->meshToLandscape.newSizeX;
    Convert1_NewSizeY->Text = at3dData->meshToLandscape.newSizeY;

    Convert1_EditStretchX->Text = at3dData->meshToLandscape.lastXSpacing;
    Convert1_EditStretchY->Text = at3dData->meshToLandscape.lastYSpacing;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_FlagStretchXClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.stretchXFlag = (Convert1_FlagStretchX->Checked) ? 1 : 0;
  Convert1_CalculateClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_FlagStretchYClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.stretchYFlag = (Convert1_FlagStretchY->Checked) ? 1 : 0;
  Convert1_CalculateClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_KeepOriginalClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.convertFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_LandscapeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.convertFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_MeshClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.convertFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_AddPointCloudClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.addCloudFlag = (Convert1_AddPointCloud->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_DropFacesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.dropFacesFlag = (Convert1_DropFaces->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_AddBaseClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.addBaseFlag = (Convert1_AddBase->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_AddUVClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.addUVFlag = (Convert1_AddUV->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_WriteXYZClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->meshToLandscape.writeXYZFlag = (Convert1_WriteXYZ->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert1_MeshToLandscapeClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  // To write an XYZ file during the conversion process
  // a full file path and name are required
  //
  char drive0[_MAX_DRIVE];
	char dir0[_MAX_DIR];
	char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];
  char xyzName[MAX_PATH];

  if (at3dData->inName[0] == '\0')
  {
    xyzName[0] = '\0';
  }
  else
  {
    _splitpath(at3dData->inName, drive0, dir0, name0, ext0);
    _makepath(xyzName, drive0, dir0, name0, ".xyz");
  }

  try
  {
    at3dData->meshToLandscape.dropElevation = Convert1_DropElevation->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->meshToLandscape.dropElevation = 10.0;
  }

  try
  {
    at3dData->meshToLandscape.baseThickness = Convert1_BaseThickness->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->meshToLandscape.baseThickness = 10.0;
  }

  At3d_WriteFlag(LIBFLAG_MESH_TO_DEM_CONVERT,
                 at3dData->meshToLandscape.convertFlag);

  At3d_WriteFlag(LIBFLAG_MESH_TO_DEM_ADD_CLOUD,
                 at3dData->meshToLandscape.addCloudFlag);

  At3d_WriteFlag(LIBFLAG_MESH_TO_DEM_DROP_FACES,
                 at3dData->meshToLandscape.dropFacesFlag);

  At3d_WriteFlag(LIBFLAG_MESH_TO_DEM_ADD_BASE,
                 at3dData->meshToLandscape.addBaseFlag);

  At3d_WriteFlag(LIBFLAG_MESH_TO_DEM_ADD_UV,
                 at3dData->meshToLandscape.addUVFlag);

  At3d_WriteFlag(LIBFLAG_MESH_TO_DEM_WRITE_XYZ,
                 at3dData->meshToLandscape.writeXYZFlag);

  if (at3dData->meshToLandscape.writeXYZFlag)
  {
    // set the necessary flags that control writing XYZ files
    //
    // write ASCII file
    //
    At3d_WriteFlag(LIBFLAG_OUT_XYZ_BINARY, 0);

    // use CAD right hand axis system
    //
    At3d_WriteFlag(LIBFLAG_OUT_XYZ_AXIS, 0);

    // duplicates are already removed
    //
    At3d_WriteFlag(LIBFLAG_OUT_XYZ_REMOVE_DUPLICATES, 0);

    // separate coordinates with spaces
    //
    At3d_WriteFlag(LIBFLAG_OUT_XYZ_TAB, 0);

    // do  not add X, Y or Z prefix to coordinates
    //
    At3d_WriteFlag(LIBFLAG_OUT_XYZ_PREFIX, 0);
  }

  At3d_Convert_Mesh_To_Landscape(xyzName, at3dData->meshToLandscape.dropElevation,
                                 at3dData->meshToLandscape.baseThickness);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_StartClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (at3dData->extrude2D_open) return;

  at3dData->extrude2D_open = At3d_ExtrudePseudo2D_Start();

  if (at3dData->extrude2D_open)
  {
    // start with first layer
    //
    at3dData->extrude2D_layerPtr = NULL;
    Extrude2D_GetNextLayerClick(Sender);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_GetNextLayerClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  // just select layers in order they are in list
  //
  at3dData->extrude2D_layerPtr = At3d_GetNextLayer(at3dData->extrude2D_layerPtr,
                                                   at3dData->extrude2D_layerName);

  if (at3dData->extrude2D_layerPtr)
  {
    Extrude2D_LayerName->Text = at3dData->extrude2D_layerName;

    if (At3d_ExtrudePseudo2D_SetLayer(at3dData->extrude2D_layerName)> 0)
    {
      Extrude2D_Display_Layer_Settings();
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_Display_Layer_Settings(void)
{
  if (!at3dData->extrude2D_open) return;

  Extrude2D_Display_Mesh_Size();
  Extrude2D_Display_Rotations();

  at3dData->extrude2D_typeFlag = At3d_ExtrudePseudo2D_GetTypeFlag();

  if (at3dData->extrude2D_typeFlag == EXTRUDE_PSEUDO2D_NORMAL)
  {
    Extrude2D_Normal->Checked = 1;
  }
  else if (at3dData->extrude2D_typeFlag == EXTRUDE_PSEUDO2D_FLAT_TOP)
  {
    Extrude2D_FlatTop->Checked = 1;
  }
  else
  {
    Extrude2D_FlatBottom->Checked = 1;
  }

  at3dData->extrude2D_thickness = At3d_ExtrudePseudo2D_GetThickness();
  at3dData->extrude2D_adjustLocation = At3d_ExtrudePseudo2D_GetAdjustLocation();

  Extrude2D_Thickness->Text      = at3dData->extrude2D_thickness;
  Extrude2D_AdjustLocation->Text = at3dData->extrude2D_adjustLocation;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_Display_Mesh_Size(void)
{
  if (!at3dData->extrude2D_open) return;

  At3d_ExtrudePseudo2D_Get_Mesh_Size(at3dData->extrude2D_minMax);

  Extrude2D_MinX->Text = at3dData->extrude2D_minMax[0];
  Extrude2D_MinY->Text = at3dData->extrude2D_minMax[1];
  Extrude2D_MinZ->Text = at3dData->extrude2D_minMax[2];

  Extrude2D_MaxX->Text = at3dData->extrude2D_minMax[3];
  Extrude2D_MaxY->Text = at3dData->extrude2D_minMax[4];
  Extrude2D_MaxZ->Text = at3dData->extrude2D_minMax[5];
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_Display_Rotations(void)
{
  if (!at3dData->extrude2D_open) return;

  At3d_ExtrudePseudo2D_GetRotateAngles(&at3dData->extrude2D_xRotation,
                                       &at3dData->extrude2D_yRotation,
                                       &at3dData->extrude2D_zRotation);

  Extrude2D_XRotation->Text = at3dData->extrude2D_xRotation;
  Extrude2D_YRotation->Text = at3dData->extrude2D_yRotation;
  Extrude2D_ZRotation->Text = at3dData->extrude2D_zRotation;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_NormalClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->extrude2D_typeFlag = EXTRUDE_PSEUDO2D_NORMAL;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_FlatTopClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->extrude2D_typeFlag = EXTRUDE_PSEUDO2D_FLAT_TOP;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_FlatBottomClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->extrude2D_typeFlag = EXTRUDE_PSEUDO2D_FLAT_BOTTOM;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_MarkRotationsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  At3d_ExtrudePseudo2D_Mark();
  Extrude2D_Display_Rotations();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_GetStepAngle(void)
{
  try
  {
    at3dData->extrude2D_stepAngle = Extrude2D_StepAngle->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->extrude2D_stepAngle = 1.0;
  }

  Extrude2D_StepAngle->Text = at3dData->extrude2D_stepAngle;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_XRotate1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  Extrude2D_GetStepAngle();
  At3d_ExtrudePseudo2D_SetRotateAngles(1, -at3dData->extrude2D_stepAngle, 0.0, 0.0);
  Extrude2D_Display_Rotations();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_XRotate2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  Extrude2D_GetStepAngle();
  At3d_ExtrudePseudo2D_SetRotateAngles(1, at3dData->extrude2D_stepAngle, 0.0, 0.0);
  Extrude2D_Display_Rotations();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_YRotate1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  Extrude2D_GetStepAngle();
  At3d_ExtrudePseudo2D_SetRotateAngles(1, 0.0, -at3dData->extrude2D_stepAngle, 0.0);
  Extrude2D_Display_Rotations();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_YRotate2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  Extrude2D_GetStepAngle();
  At3d_ExtrudePseudo2D_SetRotateAngles(1, 0.0, at3dData->extrude2D_stepAngle, 0.0);
  Extrude2D_Display_Rotations();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_ZRotate1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  Extrude2D_GetStepAngle();
  At3d_ExtrudePseudo2D_SetRotateAngles(1, 0.0, 0.0, -at3dData->extrude2D_stepAngle);
  Extrude2D_Display_Rotations();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_ZRotate2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  Extrude2D_GetStepAngle();
  At3d_ExtrudePseudo2D_SetRotateAngles(1, 0.0, 0.0, at3dData->extrude2D_stepAngle);
  Extrude2D_Display_Rotations();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_ClearRotationsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  At3d_ExtrudePseudo2D_Clear();
  Extrude2D_Display_Rotations();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_Get_Thickness_Adjust(void)
{
  try
  {
    at3dData->extrude2D_thickness = Extrude2D_Thickness->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->extrude2D_thickness = 1.0;
  }

  Extrude2D_Thickness->Text = at3dData->extrude2D_thickness;

  try
  {
    at3dData->extrude2D_adjustLocation = Extrude2D_AdjustLocation->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->extrude2D_adjustLocation = 1.0;
  }

  Extrude2D_AdjustLocation->Text = at3dData->extrude2D_adjustLocation;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_ExtrudeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  Extrude2D_Get_Thickness_Adjust();
  At3d_ExtrudePseudo2D_Extrude(at3dData->extrude2D_typeFlag,
                               at3dData->extrude2D_thickness,
                               at3dData->extrude2D_adjustLocation);
  Extrude2D_Display_Mesh_Size();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_RedoClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  Extrude2D_Get_Thickness_Adjust();
  At3d_ExtrudePseudo2D_Redo(at3dData->extrude2D_typeFlag,
                            at3dData->extrude2D_thickness,
                            at3dData->extrude2D_adjustLocation);
  Extrude2D_Display_Mesh_Size();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_UndoClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  At3d_ExtrudePseudo2D_Undo();
  Extrude2D_Display_Mesh_Size();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Extrude2D_StopClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!at3dData->extrude2D_open) return;

  if (At3d_ExtrudePseudo2D_Stop() > 0) at3dData->extrude2D_open = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateAboutOriginClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->originRotateFlag = (RotateAboutOrigin->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, at3dData->originRotateFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DrawObjectAxesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->drawObjectAxesFlag = (DrawObjectAxes->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_DRAW_OBJECT_AXES, at3dData->drawObjectAxesFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditSelectLayerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  // just select layers in order they are in list
  //
  at3dData->editNextLayer = At3d_GetNextLayer(at3dData->editNextLayer, at3dData->editName);
  void* result = At3d_Edit_Select_Layer(at3dData->editName);

  if (result)
  {
    EditName->Text = at3dData->editName;
    DisplayPickVertexMode();

    if ((at3dData->extrudeModeFlag) && (at3dData->bevelSetsFlag))
    {
      At3d_Edit_Select_Bevel_Set(at3dData->editName);

      DisplayExtrudeBevelFlags();
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::PageControl1Change(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (PageControl1->ActivePage == ViewSheet)
  {
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetEditMoveVertexClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Edit_Get_Vertex(&at3dData->editMoveVertexX,
                       &at3dData->editMoveVertexY,
                       &at3dData->editMoveVertexZ);

  EditMoveVertexX->Text = at3dData->editMoveVertexX;
  EditMoveVertexY->Text = at3dData->editMoveVertexY;
  EditMoveVertexZ->Text = at3dData->editMoveVertexZ;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetEditMoveVertexClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->editMoveVertexX = EditMoveVertexX->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editMoveVertexX = 0.0;
  }
  EditMoveVertexX->Text = at3dData->editMoveVertexX;

  try
  {
    at3dData->editMoveVertexY = EditMoveVertexY->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editMoveVertexY = 0.0;
  }
  EditMoveVertexY->Text = at3dData->editMoveVertexY;

  try
  {
    at3dData->editMoveVertexZ = EditMoveVertexZ->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editMoveVertexZ = 0.0;
  }
  EditMoveVertexZ->Text = at3dData->editMoveVertexZ;

  At3d_Edit_Set_Vertex(at3dData->editMoveVertexX,
                       at3dData->editMoveVertexY,
                       at3dData->editMoveVertexZ);

  GetEditMoveVertexClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetEditMoveVertexStep(void)
{
  try
  {
    at3dData->editMoveVertexStep = EditMoveVertexStep->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->editMoveVertexStep = 1.0;
  }

  EditMoveVertexStep->Text = at3dData->editMoveVertexStep;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditMoveVertexX1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditMoveVertexStep();
  At3d_Edit_Move_Vertex(-at3dData->editMoveVertexStep, 0.0, 0.0);
  GetEditMoveVertexClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditMoveVertexX2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditMoveVertexStep();
  At3d_Edit_Move_Vertex(at3dData->editMoveVertexStep, 0.0, 0.0);
  GetEditMoveVertexClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditMoveVertexY1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditMoveVertexStep();
  At3d_Edit_Move_Vertex(0.0, -at3dData->editMoveVertexStep, 0.0);
  GetEditMoveVertexClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditMoveVertexY2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditMoveVertexStep();
  At3d_Edit_Move_Vertex(0.0, at3dData->editMoveVertexStep, 0.0);
  GetEditMoveVertexClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditMoveVertexZ1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditMoveVertexStep();
  At3d_Edit_Move_Vertex(0.0, 0.0, -at3dData->editMoveVertexStep);
  GetEditMoveVertexClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditMoveVertexZ2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  GetEditMoveVertexStep();
  At3d_Edit_Move_Vertex(0.0, 0.0, at3dData->editMoveVertexStep);
  GetEditMoveVertexClick(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TTest_At3dx_DllForm::XyzSkipDlgClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzSkipDlgFlag = (XyzSkipDlg->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzMeshTypeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzScanFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzScanTypeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzScanFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzDoubleZTypeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzScanFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzAsciiTypeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzBinaryFileFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzBinaryTypeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzBinaryFileFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzIntelClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzMotorolaFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzMotorolaClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzMotorolaFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzFloatTypeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzDoubleFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzDoubleTypeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzDoubleFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Get_XYZ_Data(void)
{
  try
  {
    at3dData->xyzRows = XyzRows->Text.ToInt();
  }
  catch(...)
  {
    at3dData->xyzRows = 0;
  }

  try
  {
    at3dData->xyzColumns = XyzColumns->Text.ToInt();
  }
  catch(...)
  {
    at3dData->xyzColumns = 0;
  }

  try
  {
    at3dData->xyzFromRow = XyzFromRow->Text.ToInt();
  }
  catch(...)
  {
    at3dData->xyzFromRow = 0;
  }

  try
  {
    at3dData->xyzToRow = XyzToRow->Text.ToInt();
  }
  catch(...)
  {
    at3dData->xyzToRow = 0;
  }

  try
  {
    at3dData->xyzFromColumn = XyzFromColumn->Text.ToInt();
  }
  catch(...)
  {
    at3dData->xyzFromColumn = 0;
  }

  try
  {
    at3dData->xyzToColumn = XyzToColumn->Text.ToInt();
  }
  catch(...)
  {
    at3dData->xyzToColumn = 0;
  }

  try
  {
    at3dData->xyzScaleFactorZ = XyzScaleFactorZ->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->xyzScaleFactorZ = 1.0;
  }

  try
  {
    at3dData->xyzNoDataValue = XyzNoDataValue->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->xyzNoDataValue = -9999.0;
  }

  try
  {
    at3dData->xyzNoDataReplaceValue = XyzNoDataReplaceValue->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->xyzNoDataReplaceValue = 0.0;
  }

  try
  {
    at3dData->xyzSkipCount = abs(XyzSkipCount->Text.ToInt());
  }
  catch(...)
  {
    at3dData->xyzSkipCount = 0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzByRowClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzByColumnFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzByColumnClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzByColumnFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzReadClipClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzClipFlag = (XyzReadClip->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzAxis1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzAxisFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzAxis2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzAxisFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzAxis3Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzAxisFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzScaleZClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzScaleZFlag = (XyzScaleZ->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzSaveAsDXFClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzSaveDxfFlag = (XyzSaveAsDXF->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzUseNoDataClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzNoDataFlag = (XyzUseNoData->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzNoDataReplace1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzNoDataReplaceFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzNoDataReplace2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzNoDataReplaceFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzNoDataReplace3Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzNoDataReplaceFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaUseRadianceClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaUseRadianceFlag = (KerkytheaUseRadiance->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaRadianceAsEmitterClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaRadianceEmitterFlag = (KerkytheaRadianceAsEmitter->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaUsePngAlphaMasksClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaUsePngAlphaMasksFlag = (KerkytheaUsePngAlphaMasks->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalAsOmniClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalAsSpotFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalAsSunClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalAsSpotFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalAsSpotClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalAsSpotFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalAttenuation1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalAttenuationFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalAttenuation2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalAttenuationFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalAttenuation3Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalAttenuationFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointUpClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointUpFlag = (KerkytheaPointUp->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointAsSpotClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointAsSpotFlag = (KerkytheaPointAsSpot->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointAttenuation1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointAttenuationFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointAttenuation2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointAttenuationFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointAttenuation3Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointAttenuationFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSpotAttenuation1Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotAttenuationFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSpotAttenuation2Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotAttenuationFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSpotAttenuation3Click(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotAttenuationFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaUseHotFractionClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotUseHotFractionFlag = (KerkytheaUseHotFraction->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSpotReplaceClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotReplaceFlag = (KerkytheaSpotReplace->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaGetDataClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaUseRadianceFlag            = At3d_ReadFlag(LIBFLAG_KERKYTHEA_USE_RADIANCE);
  at3dData->kerkytheaRadianceEmitterFlag        = At3d_ReadFlag(LIBFLAG_KERKYTHEA_RADIANCE_EMITTER);
  at3dData->kerkytheaUsePngAlphaMasksFlag       = At3d_ReadFlag(LIBFLAG_KERKYTHEA_USE_PNG_ALPHA_MASKS);

  at3dData->kerkytheaDirectionalAsSpotFlag      = At3d_ReadFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_AS_SPOT);
  at3dData->kerkytheaPointAsSpotFlag            = At3d_ReadFlag(LIBFLAG_KERKYTHEA_POINT_AS_SPOT);
  at3dData->kerkytheaPointUpFlag                = At3d_ReadFlag(LIBFLAG_KERKYTHEA_POINT_UP);
  at3dData->kerkytheaDirectionalAttenuationFlag = At3d_ReadFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_ATTENUATION);
  at3dData->kerkytheaPointAttenuationFlag       = At3d_ReadFlag(LIBFLAG_KERKYTHEA_POINT_ATTENUATION);
  at3dData->kerkytheaSpotAttenuationFlag        = At3d_ReadFlag(LIBFLAG_KERKYTHEA_SPOT_ATTENUATION);
  at3dData->kerkytheaSpotReplaceFlag            = At3d_ReadFlag(LIBFLAG_KERKYTHEA_SPOT_REPLACE);
  at3dData->kerkytheaSpotUseHotFractionFlag     = At3d_ReadFlag(LIBFLAG_KERKYTHEA_SPOT_USE_HOT_FRACTION);

  at3dData->kerkytheaDirectionalShadowFlag         = At3d_ReadFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_SHADOW);
  at3dData->kerkytheaDirectionalSoftShadowFlag     = At3d_ReadFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_SOFT_SHADOW);
  at3dData->kerkytheaDirectionalGlobalPhotonsFlag  = At3d_ReadFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_GLOBAL_PHOTONS);
  at3dData->kerkytheaDirectionalCausticPhotonsFlag = At3d_ReadFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_CAUSTIC_PHOTONS);
  at3dData->kerkytheaDirectionalNegativeFlag       = At3d_ReadFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_NEGATIVE);

  at3dData->kerkytheaPointShadowFlag         = At3d_ReadFlag(LIBFLAG_KERKYTHEA_POINT_SHADOW);
  at3dData->kerkytheaPointSoftShadowFlag     = At3d_ReadFlag(LIBFLAG_KERKYTHEA_POINT_SOFT_SHADOW);
  at3dData->kerkytheaPointGlobalPhotonsFlag  = At3d_ReadFlag(LIBFLAG_KERKYTHEA_POINT_GLOBAL_PHOTONS);
  at3dData->kerkytheaPointCausticPhotonsFlag = At3d_ReadFlag(LIBFLAG_KERKYTHEA_POINT_CAUSTIC_PHOTONS);
  at3dData->kerkytheaPointNegativeFlag       = At3d_ReadFlag(LIBFLAG_KERKYTHEA_POINT_NEGATIVE);

  at3dData->kerkytheaSpotShadowFlag         = At3d_ReadFlag(LIBFLAG_KERKYTHEA_SPOT_SHADOW);
  at3dData->kerkytheaSpotSoftShadowFlag     = At3d_ReadFlag(LIBFLAG_KERKYTHEA_SPOT_SOFT_SHADOW);
  at3dData->kerkytheaSpotGlobalPhotonsFlag  = At3d_ReadFlag(LIBFLAG_KERKYTHEA_SPOT_GLOBAL_PHOTONS);
  at3dData->kerkytheaSpotCausticPhotonsFlag = At3d_ReadFlag(LIBFLAG_KERKYTHEA_SPOT_CAUSTIC_PHOTONS);
  at3dData->kerkytheaSpotNegativeFlag       = At3d_ReadFlag(LIBFLAG_KERKYTHEA_SPOT_NEGATIVE);

  At3d_Kerkythea_GetMultiplier(0, &at3dData->kerkytheaDirectionalMultiplier);
  At3d_Kerkythea_GetMultiplier(1, &at3dData->kerkytheaPointMultiplier);
  At3d_Kerkythea_GetMultiplier(2, &at3dData->kerkytheaSpotMultiplier);

  At3d_Kerkythea_GetSpotFallOff(0, &at3dData->kerkytheaDirectionalFallOff);
  At3d_Kerkythea_GetSpotFallOff(1, &at3dData->kerkytheaPointFallOff);
  At3d_Kerkythea_GetSpotFallOff(2, &at3dData->kerkytheaSpotFallOff);

  At3d_Kerkythea_GetSpotHotSpot(0, &at3dData->kerkytheaDirectionalHotSpot);
  At3d_Kerkythea_GetSpotHotSpot(1, &at3dData->kerkytheaPointHotSpot);
  At3d_Kerkythea_GetSpotHotSpot(2, &at3dData->kerkytheaSpotHotSpot);

  At3d_Kerkythea_GetSpotHotFraction(&at3dData->kerkytheaSpotHotFraction);

  At3d_Kerkythea_GetRadius(0, &at3dData->kerkytheaDirectionalRadius);
  At3d_Kerkythea_GetRadius(1, &at3dData->kerkytheaPointRadius);
  At3d_Kerkythea_GetRadius(2, &at3dData->kerkytheaSpotRadius);

  UpdateKerkytheaDisplay();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSetDataClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->kerkytheaDirectionalMultiplier = my_double_to_float(KerkytheaDirectionalMultiplier->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaDirectionalMultiplier = 1.0f;
  }

  try
  {
    at3dData->kerkytheaPointMultiplier = my_double_to_float(KerkytheaPointMultiplier->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaPointMultiplier = 1.0f;
  }

  try
  {
    at3dData->kerkytheaSpotMultiplier = my_double_to_float(KerkytheaSpotMultiplier->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaSpotMultiplier = 1.0f;
  }

  try
  {
    at3dData->kerkytheaDirectionalFallOff = my_double_to_float(KerkytheaDirectionalFallOff->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaDirectionalFallOff = 120.0f;
  }

  try
  {
    at3dData->kerkytheaPointFallOff = my_double_to_float(KerkytheaPointFallOff->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaPointFallOff = 120.0f;
  }

  try
  {
    at3dData->kerkytheaSpotFallOff = my_double_to_float(KerkytheaSpotFallOff->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaSpotFallOff = 120.0f;
  }

  try
  {
    at3dData->kerkytheaDirectionalHotSpot = my_double_to_float(KerkytheaDirectionalHotSpot->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaDirectionalHotSpot = 60.0f;
  }

  try
  {
    at3dData->kerkytheaPointHotSpot = my_double_to_float(KerkytheaPointHotSpot->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaPointHotSpot = 60.0f;
  }

  try
  {
    at3dData->kerkytheaSpotHotSpot = my_double_to_float(KerkytheaSpotHotSpot->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaSpotHotSpot = 60.0f;
  }

  try
  {
    at3dData->kerkytheaSpotHotFraction = my_double_to_float(KerkytheaSpotHotFraction->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaSpotHotFraction = 1.0f;
  }

  try
  {
    at3dData->kerkytheaDirectionalRadius = my_double_to_float(KerkytheaDirectionalRadius->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaDirectionalRadius = 0.2f;
  }

  try
  {
    at3dData->kerkytheaPointRadius = my_double_to_float(KerkytheaPointRadius->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaPointRadius = 0.2f;
  }

  try
  {
    at3dData->kerkytheaSpotRadius = my_double_to_float(KerkytheaSpotRadius->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kerkytheaSpotRadius = 0.2f;
  }


  At3d_WriteFlag(LIBFLAG_KERKYTHEA_USE_RADIANCE, at3dData->kerkytheaUseRadianceFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_RADIANCE_EMITTER, at3dData->kerkytheaRadianceEmitterFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_USE_PNG_ALPHA_MASKS, at3dData->kerkytheaUsePngAlphaMasksFlag);

  At3d_WriteFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_AS_SPOT, at3dData->kerkytheaDirectionalAsSpotFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_POINT_AS_SPOT, at3dData->kerkytheaPointAsSpotFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_POINT_UP, at3dData->kerkytheaPointUpFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_ATTENUATION, at3dData->kerkytheaDirectionalAttenuationFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_POINT_ATTENUATION, at3dData->kerkytheaPointAttenuationFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_SPOT_ATTENUATION, at3dData->kerkytheaSpotAttenuationFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_SPOT_REPLACE, at3dData->kerkytheaSpotReplaceFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_SPOT_USE_HOT_FRACTION, at3dData->kerkytheaSpotUseHotFractionFlag);

  At3d_WriteFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_SHADOW, at3dData->kerkytheaDirectionalShadowFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_SOFT_SHADOW, at3dData->kerkytheaDirectionalSoftShadowFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_GLOBAL_PHOTONS, at3dData->kerkytheaDirectionalGlobalPhotonsFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_CAUSTIC_PHOTONS, at3dData->kerkytheaDirectionalCausticPhotonsFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_DIRECTIONAL_NEGATIVE, at3dData->kerkytheaDirectionalNegativeFlag);

  At3d_WriteFlag(LIBFLAG_KERKYTHEA_POINT_SHADOW, at3dData->kerkytheaPointShadowFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_POINT_SOFT_SHADOW, at3dData->kerkytheaPointSoftShadowFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_POINT_GLOBAL_PHOTONS, at3dData->kerkytheaPointGlobalPhotonsFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_POINT_CAUSTIC_PHOTONS, at3dData->kerkytheaPointCausticPhotonsFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_POINT_NEGATIVE, at3dData->kerkytheaPointNegativeFlag);

  At3d_WriteFlag(LIBFLAG_KERKYTHEA_SPOT_SHADOW, at3dData->kerkytheaSpotShadowFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_SPOT_SOFT_SHADOW, at3dData->kerkytheaSpotSoftShadowFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_SPOT_GLOBAL_PHOTONS, at3dData->kerkytheaSpotGlobalPhotonsFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_SPOT_CAUSTIC_PHOTONS, at3dData->kerkytheaSpotCausticPhotonsFlag);
  At3d_WriteFlag(LIBFLAG_KERKYTHEA_SPOT_NEGATIVE, at3dData->kerkytheaSpotNegativeFlag);


  At3d_Kerkythea_SetMultiplier(0, at3dData->kerkytheaDirectionalMultiplier);
  At3d_Kerkythea_SetMultiplier(1, at3dData->kerkytheaPointMultiplier);
  At3d_Kerkythea_SetMultiplier(2, at3dData->kerkytheaSpotMultiplier);

  At3d_Kerkythea_SetSpotFallOff(0, at3dData->kerkytheaDirectionalFallOff);
  At3d_Kerkythea_SetSpotFallOff(1, at3dData->kerkytheaPointFallOff);
  At3d_Kerkythea_SetSpotFallOff(2, at3dData->kerkytheaSpotFallOff);

  At3d_Kerkythea_SetSpotHotSpot(0, at3dData->kerkytheaDirectionalHotSpot);
  At3d_Kerkythea_SetSpotHotSpot(1, at3dData->kerkytheaPointHotSpot);
  At3d_Kerkythea_SetSpotHotSpot(2, at3dData->kerkytheaSpotHotSpot);

  At3d_Kerkythea_SetSpotHotFraction(at3dData->kerkytheaSpotHotFraction);

  At3d_Kerkythea_SetRadius(0, at3dData->kerkytheaDirectionalRadius);
  At3d_Kerkythea_SetRadius(1, at3dData->kerkytheaPointRadius);
  At3d_Kerkythea_SetRadius(2, at3dData->kerkytheaSpotRadius);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UpdateKerkytheaDisplay(void)
{
  KerkytheaUseRadiance->Checked = (at3dData->kerkytheaUseRadianceFlag) ? 1 : 0;
  KerkytheaRadianceAsEmitter->Checked = (at3dData->kerkytheaRadianceEmitterFlag) ? 1 : 0;
  KerkytheaUsePngAlphaMasks->Checked = (at3dData->kerkytheaUsePngAlphaMasksFlag) ? 1 : 0;

  if (!at3dData->kerkytheaDirectionalAsSpotFlag)
  {
    KerkytheaDirectionalAsOmni->Checked = 1;
  }
  else if (at3dData->kerkytheaDirectionalAsSpotFlag == 1)
  {
    KerkytheaDirectionalAsSun->Checked = 1;
  }
  else
  {
    KerkytheaDirectionalAsSpot->Checked = 1;
  }

  if (!at3dData->kerkytheaDirectionalAttenuationFlag)
  {
    KerkytheaDirectionalAttenuation1->Checked = 1;
  }
  else if (at3dData->kerkytheaDirectionalAttenuationFlag == 1)
  {
    KerkytheaDirectionalAttenuation2->Checked = 1;
  }
  else
  {
    KerkytheaDirectionalAttenuation3->Checked = 1;
  }

  KerkytheaPointUp->Checked = (at3dData->kerkytheaPointUpFlag) ? 1 : 0;
  KerkytheaPointAsSpot->Checked = (at3dData->kerkytheaPointAsSpotFlag) ? 1 : 0;

  if (!at3dData->kerkytheaPointAttenuationFlag)
  {
    KerkytheaPointAttenuation1->Checked = 1;
  }
  else if (at3dData->kerkytheaPointAttenuationFlag == 1)
  {
    KerkytheaPointAttenuation2->Checked = 1;
  }
  else
  {
    KerkytheaPointAttenuation3->Checked = 1;
  }

  if (!at3dData->kerkytheaSpotAttenuationFlag)
  {
    KerkytheaSpotAttenuation1->Checked = 1;
  }
  else if (at3dData->kerkytheaSpotAttenuationFlag == 1)
  {
    KerkytheaSpotAttenuation2->Checked = 1;
  }
  else
  {
    KerkytheaSpotAttenuation3->Checked = 1;
  }

  KerkytheaUseHotFraction->Checked = (at3dData->kerkytheaSpotUseHotFractionFlag) ? 1 : 0;
  KerkytheaSpotReplace->Checked    = (at3dData->kerkytheaSpotReplaceFlag) ? 1 : 0;

  KerkytheaDirectionalShadow->Checked         = (at3dData->kerkytheaDirectionalShadowFlag) ? 1 : 0;
  KerkytheaDirectionalSoftShadow->Checked     = (at3dData->kerkytheaDirectionalSoftShadowFlag) ? 1 : 0;
  KerkytheaDirectionalGlobalPhotons->Checked  = (at3dData->kerkytheaDirectionalGlobalPhotonsFlag) ? 1 : 0;
  KerkytheaDirectionalCausticPhotons->Checked = (at3dData->kerkytheaDirectionalCausticPhotonsFlag) ? 1 : 0;
  KerkytheaDirectionalNegative->Checked       = (at3dData->kerkytheaDirectionalNegativeFlag) ? 1 : 0;

  KerkytheaPointShadow->Checked         = (at3dData->kerkytheaPointShadowFlag) ? 1 : 0;
  KerkytheaPointSoftShadow->Checked     = (at3dData->kerkytheaPointSoftShadowFlag) ? 1 : 0;
  KerkytheaPointGlobalPhotons->Checked  = (at3dData->kerkytheaPointGlobalPhotonsFlag) ? 1 : 0;
  KerkytheaPointCausticPhotons->Checked = (at3dData->kerkytheaPointCausticPhotonsFlag) ? 1 : 0;
  KerkytheaPointNegative->Checked       = (at3dData->kerkytheaPointNegativeFlag) ? 1 : 0;

  KerkytheaSpotShadow->Checked         = (at3dData->kerkytheaSpotShadowFlag) ? 1 : 0;
  KerkytheaSpotSoftShadow->Checked     = (at3dData->kerkytheaSpotSoftShadowFlag) ? 1 : 0;
  KerkytheaSpotGlobalPhotons->Checked  = (at3dData->kerkytheaSpotGlobalPhotonsFlag) ? 1 : 0;
  KerkytheaSpotCausticPhotons->Checked = (at3dData->kerkytheaSpotCausticPhotonsFlag) ? 1 : 0;
  KerkytheaSpotNegative->Checked       = (at3dData->kerkytheaSpotNegativeFlag) ? 1 : 0;

  KerkytheaDirectionalMultiplier->Text = at3dData->kerkytheaDirectionalMultiplier;
  KerkytheaPointMultiplier->Text       = at3dData->kerkytheaPointMultiplier;
  KerkytheaSpotMultiplier->Text        = at3dData->kerkytheaSpotMultiplier;

  KerkytheaDirectionalFallOff->Text = at3dData->kerkytheaDirectionalFallOff;
  KerkytheaPointFallOff->Text       = at3dData->kerkytheaPointFallOff;
  KerkytheaSpotFallOff->Text        = at3dData->kerkytheaSpotFallOff;

  KerkytheaDirectionalHotSpot->Text = at3dData->kerkytheaDirectionalHotSpot;
  KerkytheaPointHotSpot->Text       = at3dData->kerkytheaPointHotSpot;
  KerkytheaSpotHotSpot->Text        = at3dData->kerkytheaSpotHotSpot;

  KerkytheaSpotHotFraction->Text = at3dData->kerkytheaSpotHotFraction;

  KerkytheaDirectionalRadius->Text = at3dData->kerkytheaDirectionalRadius;
  KerkytheaPointRadius->Text       = at3dData->kerkytheaPointRadius;
  KerkytheaSpotRadius->Text        = at3dData->kerkytheaSpotRadius;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::VrmlByChiefArchitectClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->vrmlByChiefArchitectFlag = (VrmlByChiefArchitect->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::VrmlTextNoClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->vrmlTextFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::VrmlTextFlatClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->vrmlTextFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::VrmlTextExtrudedClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->vrmlTextFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DxfTextNoClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useDxfTextFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DxfTextFlatClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useDxfTextFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DxfTextExtrudedClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useDxfTextFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TextAsTextClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->textAsTextFlag = (TextAsText->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_WRITE_TEXT_AS_TEXT, at3dData->textAsTextFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalShadowClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalShadowFlag = (KerkytheaDirectionalShadow->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalSoftShadowClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalSoftShadowFlag = (KerkytheaDirectionalSoftShadow->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalNegativeClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalNegativeFlag = (KerkytheaDirectionalNegative->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalGlobalPhotonsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalGlobalPhotonsFlag = (KerkytheaDirectionalGlobalPhotons->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaDirectionalCausticPhotonsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaDirectionalCausticPhotonsFlag = (KerkytheaDirectionalCausticPhotons->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointShadowClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointShadowFlag = (KerkytheaPointShadow->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointSoftShadowClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointSoftShadowFlag = (KerkytheaPointSoftShadow->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointNegativeClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointNegativeFlag = (KerkytheaPointNegative->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointGlobalPhotonsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointGlobalPhotonsFlag = (KerkytheaPointGlobalPhotons->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaPointCausticPhotonsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaPointCausticPhotonsFlag = (KerkytheaPointCausticPhotons->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSpotShadowClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotShadowFlag = (KerkytheaSpotShadow->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSpotSoftShadowClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotSoftShadowFlag = (KerkytheaSpotSoftShadow->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSpotNegativeClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotNegativeFlag = (KerkytheaSpotNegative->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSpotGlobalPhotonsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotGlobalPhotonsFlag = (KerkytheaSpotGlobalPhotons->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KerkytheaSpotCausticPhotonsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kerkytheaSpotCausticPhotonsFlag = (KerkytheaSpotCausticPhotons->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DisplayStageClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->stageUseFlag = (DisplayStage->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_STAGE_USE, at3dData->stageUseFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::StageUseOriginClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->stageOriginFlag = (StageUseOrigin->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_STAGE_ORIGIN, at3dData->stageOriginFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::StageUseSquaresClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->stageSquaresFlag = (StageUseSquares->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_STAGE_TYPE, at3dData->stageSquaresFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::StageUseTextureClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->stageImageFlag = (StageUseTexture->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_STAGE_UV, at3dData->stageImageFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetStageImageClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->GetStageImage();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ChangeStageClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->stageSpacing = StageGridSpacing->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->stageSpacing = 1.0;
  }

  try
  {
    at3dData->stageWidth = StageWidth->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->stageWidth = 10.0;
  }

  try
  {
    at3dData->stageDepth = StageDepth->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->stageDepth = 10.0;
  }

  try
  {
    at3dData->stageOriginX = StageOriginX->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->stageOriginX = 0.0;
  }

  try
  {
    at3dData->stageOriginY = StageOriginY->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->stageOriginY = 0.0;
  }

  At3d_SetStageGrid(at3dData->stageSpacing, at3dData->stageWidth,
                    at3dData->stageDepth, at3dData->stageOriginX,
                    at3dData->stageOriginY);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsStartClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->StartReducePolygons();

  ReducePolygonsDisplayData();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsSelectLayerClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->SelectReducePolygonsLayer();

  ReducePolygonsDisplayData();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsDisplayData(void)
{
  // display layer name in edit box
  //
  ReducePolygonsName->Text = at3dData->reducePolygonsName;

  // display vertex and triangle counts
  //
  ReducePolygonsVertex0->Text   = at3dData->reducePolygonsVertices0;
  ReducePolygonsTriangle0->Text = at3dData->reducePolygonsTriangles0;

  ReducePolygonsVertex1->Text   = at3dData->reducePolygonsVertices1;
  ReducePolygonsTriangle1->Text = at3dData->reducePolygonsTriangles1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsGetVertices(void)
{
  try
  {
    at3dData->reducePolygonsVertices1 = ReducePolygonsVertex1->Text.ToInt();
  }
  catch(...)
  {
    at3dData->reducePolygonsVertices1 = 3;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsSetClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  ReducePolygonsGetVertices();

  at3dData->DoReducePolygons();

  ReducePolygonsDisplayData();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsGetStep(void)
{
  try
  {
    at3dData->reducePolygonsStep = ReducePolygonsStep->Text.ToInt();
  }
  catch(...)
  {
    at3dData->reducePolygonsStep = 1;
  }

  if (at3dData->reducePolygonsStep < 1) at3dData->reducePolygonsStep = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsDecreaseClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  ReducePolygonsGetVertices();
  ReducePolygonsGetStep();

  at3dData->reducePolygonsVertices1 -= at3dData->reducePolygonsStep;

  at3dData->DoReducePolygons();

  ReducePolygonsDisplayData();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsIncreaseClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  ReducePolygonsGetVertices();
  ReducePolygonsGetStep();

  at3dData->reducePolygonsVertices1 += at3dData->reducePolygonsStep;

  at3dData->DoReducePolygons();

  ReducePolygonsDisplayData();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsOkayClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_StopReducePolygons(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReducePolygonsCancelClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_StopReducePolygons(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DisplayBackgroundImageClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->backgroundImageUseFlag = (DisplayBackgroundImage->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_BACKGROUND_IMAGE_USE, at3dData->backgroundImageUseFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SelectBackgroundImageClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->GetBackgroundImage();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReadAndRotateClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->ReadAndRotateFile();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BitmapTo3D_SkipDlgClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bitmapTo3D_SkipDlg_Flag = (BitmapTo3D_SkipDlg->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BitmapTo3D_Decode0Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bitmapTo3D_Decode0_Flag = (BitmapTo3D_Decode0->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BitmapTo3D_SaveOutlinesClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bitmapTo3D_SaveOutlines_Flag = (BitmapTo3D_SaveOutlines->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BitmapTo3D_JoinOutlinesClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bitmapTo3D_JoinOutlines_Flag = (BitmapTo3D_JoinOutlines->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BitmapTo3D_AddFacesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bitmapTo3D_AddFaces_Flag = (BitmapTo3D_AddFaces->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BitmapTo3D_ExtrudeClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bitmapTo3D_Extrude_Flag = (BitmapTo3D_Extrude->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Get_BitmapTo3D_Data(void)
{
  try
  {
    at3dData->bitmapTo3D_ExtrudeHeight = BitmapTo3D_ExtrudeHeight->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->bitmapTo3D_ExtrudeHeight = 1.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BitmapToDem_SkipDlgClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bitmapToDem_SkipDlg_Flag = (BitmapToDem_SkipDlg->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::BitmapToDem_SeaLevelClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bitmapToDem_SeaLevel_Flag = (BitmapToDem_SeaLevel->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Get_BitmapToLandscape_Data(void)
{
  try
  {
    at3dData->bitmapToDem_BelowMultiplier = BitmapToDem_Multiplier1->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->bitmapToDem_BelowMultiplier = 1.0;
  }

  try
  {
    at3dData->bitmapToDem_AboveMultiplier = BitmapToDem_Multiplier2->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->bitmapToDem_AboveMultiplier = 10.0;
  }

  try
  {
    at3dData->bitmapToDem_MinHeight = BitmapToDem_MinHeight->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->bitmapToDem_MinHeight = 0.0;
  }

  try
  {
    at3dData->bitmapToDem_XY_Spacing = BitmapToDem_Spacing->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->bitmapToDem_XY_Spacing = 1.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::EditMakeBevelSetsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->bevelSetsFlag = At3d_Edit_Make_Bevel_Sets();

  // display layer names with first layer in list selected first
  //
  if (at3dData->bevelSetsFlag)
  {
    at3dData->editNextLayer = At3d_GetNextLayer(0, at3dData->editName);
    EditName->Text = at3dData->editName;

    At3d_Edit_Select_Bevel_Set(at3dData->editName);

    DisplayExtrudeBevelFlags();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ModoCameraSunlightClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->modoCameraSunlightFlag = (ModoCameraSunlight->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ModoHierarchyClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->modoHierarchyFlag = (ModoHierarchy->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ModoWithJointsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->modoWithJointsFlag = (ModoWithJoints->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetOglSnapshotClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!SnapshotPanel)
  {
    try
    {
      SnapshotPanel = new CMyImagePanel(at3dData, this);
    }
    catch(...)
    {
      SnapshotPanel = NULL;
      return;
    }

    SnapshotPanel->Visible = 1;

    SnapshotPanel->Left    = TempSnapshotPanel->Left;
    SnapshotPanel->Top     = TempSnapshotPanel->Top;
    SnapshotPanel->Width   = TempSnapshotPanel->Width;
    SnapshotPanel->Height  = TempSnapshotPanel->Height;

    SnapshotPanel->Parent  = TempSnapshotPanel->Parent;

    TempSnapshotPanel->Visible = 0;
  }

  if (SnapshotPanel)
  {
    at3dData->GetOglSnapshot(SnapshotPanel->Width, SnapshotPanel->Height,
                             SnapshotPanel->Handle);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RevolveCameraStartClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  double xAngle;
  double yAngle;
  double zAngle;
  double seconds;

  try
  {
    xAngle = RevolveCameraX->Text.ToDouble();
  }
  catch(...)
  {
    xAngle = 1.0;
  }

  try
  {
    yAngle = RevolveCameraY->Text.ToDouble();
  }
  catch(...)
  {
    yAngle = 0.0;
  }

  try
  {
    zAngle = RevolveCameraZ->Text.ToDouble();
  }
  catch(...)
  {
    zAngle = 0.0;
  }

  try
  {
    seconds = RevolveCameraSeconds->Text.ToDouble();
  }
  catch(...)
  {
    seconds = 0.0;
  }

  At3d_Revolve_Camera_Start(xAngle, yAngle, zAngle, seconds);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RevolveCameraStopClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Revolve_Camera_Stop();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DisplayOglDebugMsgsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglDebugMsgFlag = (DisplayOglDebugMsgs->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_DISPLAY_OGL_DEBUG_MSGS, at3dData->oglDebugMsgFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ShowStatusbarClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Show_Statusbar();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::HideStatusbarClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Hide_Statusbar();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReverseZoomDirectionClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->reverseZoomDirectionFlag = (ReverseZoomDirection->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_ZOOM_DIRECTION, at3dData->reverseZoomDirectionFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SelectFirstLayerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->layerMinMaxNextLayer = 0;
  SelectNextLayerClick(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SelectNextLayerClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  LAYER_MINMAX layerMinMax;

  // just select layers in order they are in list
  //
  at3dData->layerMinMaxNextLayer = At3d_GetNextLayer(at3dData->layerMinMaxNextLayer, at3dData->layerMinMaxName);
  Int32 result = At3d_Get_Layer_MinMax(at3dData->layerMinMaxName,
                                         &layerMinMax.cloudVertexCount,
                                         &layerMinMax.minX);

  if (result)
  {
    LayerMinMaxName->Text = at3dData->layerMinMaxName;

    LayerCloudVerts->Text = layerMinMax.cloudVertexCount;
    LayerLineVerts->Text  = layerMinMax.lineVertexCount;
    LayerMeshVerts->Text  = layerMinMax.meshVertexCount;

    LayerLines->Text     = layerMinMax.lineCount;
    LayerTriangles->Text = layerMinMax.triangleCount;

    LayerMinX->Text = layerMinMax.minX;
    LayerMinY->Text = layerMinMax.minY;
    LayerMinZ->Text = layerMinMax.minZ;

    LayerMaxX->Text = layerMinMax.maxX;
    LayerMaxY->Text = layerMinMax.maxY;
    LayerMaxZ->Text = layerMinMax.maxZ;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetOglGlobalAmbientClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 result = At3d_Get_OGL_Global_Ambient_Light(&at3dData->oglGlobalAmbient.red);

  if (result)
  {
    OglGlobalAmbientRed->Text   = at3dData->oglGlobalAmbient.red;
    OglGlobalAmbientGreen->Text = at3dData->oglGlobalAmbient.green;
    OglGlobalAmbientBlue->Text  = at3dData->oglGlobalAmbient.blue;
    OglGlobalAmbientAlpha->Text = at3dData->oglGlobalAmbient.alpha;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetOglGlobalAmbientClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  try
  {
    at3dData->oglGlobalAmbient.red = my_double_to_float(OglGlobalAmbientRed->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->oglGlobalAmbient.red = 0.0f;
  }

  try
  {
    at3dData->oglGlobalAmbient.green = my_double_to_float(OglGlobalAmbientGreen->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->oglGlobalAmbient.green = 0.0f;
  }

  try
  {
    at3dData->oglGlobalAmbient.blue = my_double_to_float(OglGlobalAmbientBlue->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->oglGlobalAmbient.blue = 0.0f;
  }

  // DLL always sets Global Ambient Alpha to 1.0

  Int32 result = At3d_Set_OGL_Global_Ambient_Light(&at3dData->oglGlobalAmbient.red);

  // to update OpenGL light settings immediately do a redraw
  //
  if (result) At3d_DrawWireframe();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OglLight0Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglSelectedLight = 0;
  GetOglLight();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OglLight1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglSelectedLight = 1;
  GetOglLight();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OglLight2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglSelectedLight = 2;
  GetOglLight();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OglLight3Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglSelectedLight = 3;
  GetOglLight();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OglLight4Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglSelectedLight = 4;
  GetOglLight();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OglLight5Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglSelectedLight = 5;
  GetOglLight();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OglLight6Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglSelectedLight = 6;
  GetOglLight();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OglLight7Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->oglSelectedLight = 7;
  GetOglLight();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GetOglLight(void)
{
  OGL_LIGHT* lightPtr = &at3dData->oglDirectionalLights[at3dData->oglSelectedLight];

  Int32 result = At3d_Get_OGL_Directional_Light(at3dData->oglSelectedLight,
                                                  &lightPtr->enabledFlag,
                                                  &lightPtr->ambient.red,
                                                  &lightPtr->diffuse.red,
                                                  &lightPtr->specular.red,
                                                  &lightPtr->position.x);

  if (result)
  {
    OglLightEnabled->Checked = (lightPtr->enabledFlag) ? 1 : 0;

    OglLightAmbientRed->Text   = lightPtr->ambient.red;
    OglLightAmbientGreen->Text = lightPtr->ambient.green;
    OglLightAmbientBlue->Text  = lightPtr->ambient.blue;
    OglLightAmbientAlpha->Text = lightPtr->ambient.alpha;

    OglLightDiffuseRed->Text   = lightPtr->diffuse.red;
    OglLightDiffuseGreen->Text = lightPtr->diffuse.green;
    OglLightDiffuseBlue->Text  = lightPtr->diffuse.blue;
    OglLightDiffuseAlpha->Text = lightPtr->diffuse.alpha;

    OglLightSpecularRed->Text   = lightPtr->specular.red;
    OglLightSpecularGreen->Text = lightPtr->specular.green;
    OglLightSpecularBlue->Text  = lightPtr->specular.blue;
    OglLightSpecularAlpha->Text = lightPtr->specular.alpha;

    OglLightPositionX->Text = lightPtr->position.x;
    OglLightPositionY->Text = lightPtr->position.y;
    OglLightPositionZ->Text = lightPtr->position.z;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetOglLightClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  OGL_LIGHT* lightPtr = &at3dData->oglDirectionalLights[at3dData->oglSelectedLight];

  lightPtr->enabledFlag = (OglLightEnabled->Checked) ? 1 : 0;

  try
  {
    lightPtr->ambient.red = my_double_to_float(OglLightAmbientRed->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->ambient.red = 0.0f;
  }

  try
  {
    lightPtr->ambient.green = my_double_to_float(OglLightAmbientGreen->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->ambient.green = 0.0f;
  }

  try
  {
    lightPtr->ambient.blue = my_double_to_float(OglLightAmbientBlue->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->ambient.blue = 0.0f;
  }

  // DLL always sets Light Ambient Alpha to 1.0

  try
  {
    lightPtr->diffuse.red = my_double_to_float(OglLightDiffuseRed->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->diffuse.red = 0.0f;
  }

  try
  {
    lightPtr->diffuse.green = my_double_to_float(OglLightDiffuseGreen->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->diffuse.green = 0.0f;
  }

  try
  {
    lightPtr->diffuse.blue = my_double_to_float(OglLightDiffuseBlue->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->diffuse.blue = 0.0f;
  }

  // DLL always sets Light Diffuse Alpha to 1.0

  try
  {
    lightPtr->specular.red = my_double_to_float(OglLightSpecularRed->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->specular.red = 0.0f;
  }

  try
  {
    lightPtr->specular.green = my_double_to_float(OglLightSpecularGreen->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->specular.green = 0.0f;
  }

  try
  {
    lightPtr->specular.blue = my_double_to_float(OglLightSpecularBlue->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->specular.blue = 0.0f;
  }

  // DLL always sets Light Specular Alpha to 1.0

  try
  {
    lightPtr->position.x = my_double_to_float(OglLightPositionX->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->position.x = 0.0f;
  }

  try
  {
    lightPtr->position.y = my_double_to_float(OglLightPositionY->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->position.y = 0.0f;
  }

  try
  {
    lightPtr->position.z = my_double_to_float(OglLightPositionZ->Text.ToDouble());
  }
  catch(...)
  {
    lightPtr->position.z = 0.0f;
  }

  Int32 result = At3d_Set_OGL_Directional_Light(at3dData->oglSelectedLight,
                                                 lightPtr->enabledFlag,
                                                 &lightPtr->ambient.red,
                                                 &lightPtr->diffuse.red,
                                                 &lightPtr->specular.red,
                                                 &lightPtr->position.x);

  // to update OpenGL light settings immediately do a redraw
  //
  if (result) At3d_DrawWireframe();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Convert3D_To_2DClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Convert_3D_To_2D();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Update_KML_Page(void)
{
  if (at3dData->kmlFolderName.IsEmpty())
  {
    KmlFolderName->Text = AnsiString();
  }
  else
  {
    KmlFolderName->Text = at3dData->kmlFolderName.c_str();
  }

  if (at3dData->kmlFolderDescription.IsEmpty())
  {
    KmlFolderDescription->Text = AnsiString();
  }
  else
  {
    KmlFolderDescription->Text = at3dData->kmlFolderDescription.c_str();
  }

  // --

  if (at3dData->kmlPlacemarkName.IsEmpty())
  {
    KmlPlacemarkName->Text = AnsiString();
  }
  else
  {
    KmlPlacemarkName->Text = at3dData->kmlPlacemarkName.c_str();
  }

  if (at3dData->kmlPlacemarkDescription.IsEmpty())
  {
    KmlPlacemarkDescription->Text = AnsiString();
  }
  else
  {
    KmlPlacemarkDescription->Text = at3dData->kmlPlacemarkDescription.c_str();
  }

  // --

  KmlUseModelsFolder->Checked = (at3dData->kmlUseModelsFolderFlag) ? 1 : 0;
  KmlUseImagesFolder->Checked = (at3dData->kmlUseImagesFolderFlag) ? 1 : 0;

  if (strlen(at3dData->kmlModelsFolderName))
  {
    KmlModelsFolderName->Text = at3dData->kmlModelsFolderName;
  }
  else
  {
    KmlModelsFolderName->Text = AnsiString();
  }

  if (strlen(at3dData->kmlImagesFolderName))
  {
    KmlImagesFolderName->Text = at3dData->kmlImagesFolderName;
  }
  else
  {
    KmlImagesFolderName->Text = AnsiString();
  }

  // --

  Display_LookAt_Latitude_Longitude();

  KmlLookAtAltitude->Text = at3dData->kmlLookAt_Altitude;
  KmlLookAtHeading->Text  = at3dData->kmlLookAt_Heading;
  KmlLookAtTilt->Text     = at3dData->kmlLookAt_Tilt;
  KmlLookAtRange->Text    = at3dData->kmlLookAt_Range;

  // --

  Display_Model_Latitude_Longitude();

  KmlModelAltitude->Text = at3dData->kmlModel_Altitude;
  KmlModelHeading->Text  = at3dData->kmlModel_Heading;
  KmlModelTilt->Text     = at3dData->kmlModel_Tilt;
  KmlModelRoll->Text     = at3dData->kmlModel_Roll;
  KmlModelScaleX->Text   = at3dData->kmlModel_Scale_X;
  KmlModelScaleY->Text   = at3dData->kmlModel_Scale_Y;
  KmlModelScaleZ->Text   = at3dData->kmlModel_Scale_Z;

  // --

  Display_Latitude_Longitude_Signs();

  // --

  KmlLookAtAltitudeMode->ItemIndex = at3dData->kmlLookAt_Altitude_Mode_Flag;

  KmlModelAltitudeMode->ItemIndex  = at3dData->kmlModel_Altitude_Mode_Flag;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Display_LookAt_Latitude_Longitude(void)
{
  KmlLookAtLatitudeDegrees->Text  = at3dData->kmlLookAt_Latitude_Degrees;
  KmlLookAtLatitudeMinutes->Text  = at3dData->kmlLookAt_Latitude_Minutes;
  KmlLookAtLatitudeSeconds->Text  = at3dData->kmlLookAt_Latitude_Seconds;

  KmlLookAtLongitudeDegrees->Text = at3dData->kmlLookAt_Longitude_Degrees;
  KmlLookAtLongitudeMinutes->Text = at3dData->kmlLookAt_Longitude_Minutes;
  KmlLookAtLongitudeSeconds->Text = at3dData->kmlLookAt_Longitude_Seconds;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Display_Model_Latitude_Longitude(void)
{
  KmlModelLatitudeDegrees->Text  = at3dData->kmlModel_Latitude_Degrees;
  KmlModelLatitudeMinutes->Text  = at3dData->kmlModel_Latitude_Minutes;
  KmlModelLatitudeSeconds->Text  = at3dData->kmlModel_Latitude_Seconds;

  KmlModelLongitudeDegrees->Text = at3dData->kmlModel_Longitude_Degrees;
  KmlModelLongitudeMinutes->Text = at3dData->kmlModel_Longitude_Minutes;
  KmlModelLongitudeSeconds->Text = at3dData->kmlModel_Longitude_Seconds;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Get_LookAt_Latitude_Longitude(void)
{
  try
  {
    at3dData->kmlLookAt_Latitude_Degrees = abs(KmlLookAtLatitudeDegrees->Text.ToInt());
  }
  catch(...)
  {
    at3dData->kmlLookAt_Latitude_Degrees = 0;
  }

  try
  {
    at3dData->kmlLookAt_Latitude_Minutes = abs(KmlLookAtLatitudeMinutes->Text.ToInt());
  }
  catch(...)
  {
    at3dData->kmlLookAt_Latitude_Minutes = 0;
  }

  try
  {
    at3dData->kmlLookAt_Latitude_Seconds = fabs(KmlLookAtLatitudeSeconds->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kmlLookAt_Latitude_Seconds = 0.0;
  }

  // --

  try
  {
    at3dData->kmlLookAt_Longitude_Degrees = abs(KmlLookAtLongitudeDegrees->Text.ToInt());
  }
  catch(...)
  {
    at3dData->kmlLookAt_Longitude_Degrees = 0;
  }

  try
  {
    at3dData->kmlLookAt_Longitude_Minutes = abs(KmlLookAtLongitudeMinutes->Text.ToInt());
  }
  catch(...)
  {
    at3dData->kmlLookAt_Longitude_Minutes = 0;
  }

  try
  {
    at3dData->kmlLookAt_Longitude_Seconds = fabs(KmlLookAtLongitudeSeconds->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kmlLookAt_Longitude_Seconds = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Get_Model_Latitude_Longitude(void)
{
  try
  {
    at3dData->kmlModel_Latitude_Degrees = abs(KmlModelLatitudeDegrees->Text.ToInt());
  }
  catch(...)
  {
    at3dData->kmlModel_Latitude_Degrees = 0;
  }

  try
  {
    at3dData->kmlModel_Latitude_Minutes = abs(KmlModelLatitudeMinutes->Text.ToInt());
  }
  catch(...)
  {
    at3dData->kmlModel_Latitude_Minutes = 0;
  }

  try
  {
    at3dData->kmlModel_Latitude_Seconds = fabs(KmlModelLatitudeSeconds->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kmlModel_Latitude_Seconds = 0.0;
  }

  // --

  try
  {
    at3dData->kmlModel_Longitude_Degrees = abs(KmlModelLongitudeDegrees->Text.ToInt());
  }
  catch(...)
  {
    at3dData->kmlModel_Longitude_Degrees = 0;
  }

  try
  {
    at3dData->kmlModel_Longitude_Minutes = abs(KmlModelLongitudeMinutes->Text.ToInt());
  }
  catch(...)
  {
    at3dData->kmlModel_Longitude_Minutes = 0;
  }

  try
  {
    at3dData->kmlModel_Longitude_Seconds = fabs(KmlModelLongitudeSeconds->Text.ToDouble());
  }
  catch(...)
  {
    at3dData->kmlModel_Longitude_Seconds = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::Display_Latitude_Longitude_Signs(void)
{
  KmlLookAtLatitudeSign->Checked = (at3dData->kmlLookAt_Latitude_Sign) ? 1 : 0;

  KmlLookAtLongitudeSign->Checked = (at3dData->kmlLookAt_Longitude_Sign) ? 1 : 0;

  // --

  KmlModelLatitudeSign->Checked = (at3dData->kmlModel_Latitude_Sign) ? 1 : 0;

  KmlModelLongitudeSign->Checked = (at3dData->kmlModel_Longitude_Sign) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlCopyLatitudeLongitudeClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Get_Model_Latitude_Longitude();

  at3dData->kmlLookAt_Latitude_Degrees = at3dData->kmlModel_Latitude_Degrees;
  at3dData->kmlLookAt_Latitude_Minutes = at3dData->kmlModel_Latitude_Minutes;
  at3dData->kmlLookAt_Latitude_Seconds = at3dData->kmlModel_Latitude_Seconds;
  at3dData->kmlLookAt_Latitude_Sign    = at3dData->kmlModel_Latitude_Sign;

  at3dData->kmlLookAt_Longitude_Degrees = at3dData->kmlModel_Longitude_Degrees;
  at3dData->kmlLookAt_Longitude_Minutes = at3dData->kmlModel_Longitude_Minutes;
  at3dData->kmlLookAt_Longitude_Seconds = at3dData->kmlModel_Longitude_Seconds;
  at3dData->kmlLookAt_Longitude_Sign    = at3dData->kmlModel_Longitude_Sign;

  Display_LookAt_Latitude_Longitude();
  Display_Latitude_Longitude_Signs();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlLookAtLatitudeSignClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kmlLookAt_Latitude_Sign = (KmlLookAtLatitudeSign->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlLookAtLongitudeSignClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kmlLookAt_Longitude_Sign = (KmlLookAtLongitudeSign->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlModelLatitudeSignClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kmlModel_Latitude_Sign = (KmlModelLatitudeSign->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlModelLongitudeSignClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kmlModel_Longitude_Sign = (KmlModelLongitudeSign->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlUseModelsFolderClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kmlUseModelsFolderFlag = (KmlUseModelsFolder->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlUseImagesFolderClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kmlUseImagesFolderFlag = (KmlUseImagesFolder->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlLookAtAltitudeModeChange(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kmlLookAt_Altitude_Mode_Flag = KmlLookAtAltitudeMode->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlModelAltitudeModeChange(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kmlModel_Altitude_Mode_Flag = KmlModelAltitudeMode->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlGetDataForDllClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (KmlFolderName->Text.IsEmpty())
  {
    at3dData->kmlFolderName = AnsiString();
  }
  else
  {
    at3dData->kmlFolderName = KmlFolderName->Text.c_str();
  }

  if (KmlFolderDescription->Text.IsEmpty())
  {
    at3dData->kmlFolderDescription = AnsiString();
  }
  else
  {
    at3dData->kmlFolderDescription = KmlFolderDescription->Text.c_str();
  }

  // --

  if (KmlPlacemarkName->Text.IsEmpty())
  {
    at3dData->kmlPlacemarkName = AnsiString();
  }
  else
  {
    at3dData->kmlPlacemarkName = KmlPlacemarkName->Text.c_str();
  }

  if (KmlPlacemarkDescription->Text.IsEmpty())
  {
    at3dData->kmlPlacemarkDescription = AnsiString();
  }
  else
  {
    at3dData->kmlPlacemarkDescription = KmlPlacemarkDescription->Text.c_str();
  }

  // --

  if (KmlModelsFolderName->Text.IsEmpty())
  {
    at3dData->kmlModelsFolderName[0] = '\0';
  }
  else
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, KmlModelsFolderName->Text);

    strncpy(at3dData->kmlModelsFolderName,
            tempName.c_str(), LEN_FILE_FNAME - 1);
    at3dData->kmlModelsFolderName[LEN_FILE_FNAME - 1] = '\0';
  }

  if (KmlImagesFolderName->Text.IsEmpty())
  {
    at3dData->kmlImagesFolderName[0] = '\0';
  }
  else
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, KmlImagesFolderName->Text);

    strncpy(at3dData->kmlImagesFolderName,
            tempName.c_str(), LEN_FILE_FNAME - 1);
    at3dData->kmlImagesFolderName[LEN_FILE_FNAME - 1] = '\0';
  }


  // --

  Get_LookAt_Latitude_Longitude();

  try
  {
    at3dData->kmlLookAt_Altitude = KmlLookAtAltitude->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlLookAt_Altitude = 0.0;
  }

  try
  {
    at3dData->kmlLookAt_Heading = KmlLookAtHeading->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlLookAt_Heading = 0.0;
  }

  try
  {
    at3dData->kmlLookAt_Tilt = KmlLookAtTilt->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlLookAt_Tilt = 0.0;
  }

  try
  {
    at3dData->kmlLookAt_Range = KmlLookAtRange->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlLookAt_Range = 0.0;
  }

  // --

  Get_Model_Latitude_Longitude();

  try
  {
    at3dData->kmlModel_Altitude = KmlModelAltitude->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlModel_Altitude = 0.0;
  }

  try
  {
    at3dData->kmlModel_Heading = KmlModelHeading->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlModel_Heading = 0.0;
  }

  try
  {
    at3dData->kmlModel_Tilt = KmlModelTilt->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlModel_Tilt = 0.0;
  }

  try
  {
    at3dData->kmlModel_Roll = KmlModelRoll->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlModel_Roll = 0.0;
  }

  try
  {
    at3dData->kmlModel_Scale_X = KmlModelScaleX->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlModel_Scale_X = 1.0;
  }

  try
  {
    at3dData->kmlModel_Scale_Y = KmlModelScaleY->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlModel_Scale_Y = 1.0;
  }

  try
  {
    at3dData->kmlModel_Scale_Z = KmlModelScaleZ->Text.ToDouble();
  }
  catch(...)
  {
    at3dData->kmlModel_Scale_Z = 1.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ReversePanDirectionClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->reversePanDirectionFlag = (ReversePanDirection->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_PAN_DIRECTION, at3dData->reversePanDirectionFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::GrabKeyboardFocusClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->grabKeyboardFocusFlag = (GrabKeyboardFocus->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_GRAB_KEYBOARD_FOCUS, at3dData->grabKeyboardFocusFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateCameraSimulateStartClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Int32 mouseX;
  Int32 mouseY;
  double seconds;

  try
  {
    mouseX = RotateCameraSimulateX->Text.ToInt();
  }
  catch(...)
  {
    mouseX = 1;
  }

  try
  {
    mouseY = RotateCameraSimulateY->Text.ToInt();
  }
  catch(...)
  {
    mouseY = 0;
  }


  try
  {
    seconds = RotateCameraSimulateSeconds->Text.ToDouble();
  }
  catch(...)
  {
    seconds = 0.0;
  }

  At3d_Rotate_Camera_Simulation_Start(mouseX, mouseY, seconds);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RotateCameraSimulateStopClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Rotate_Camera_Simulation_Stop();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::AlwaysShowCameraCursorClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->alwaysShowCameraCursorFlag = (AlwaysShowCameraCursor->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, at3dData->alwaysShowCameraCursorFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferOutlines1Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->Transfer_2D_Outlines(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferOutlines2Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->Transfer_2D_Outlines(2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TransferOutlines3Click(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->Transfer_2D_Outlines(3);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ShowDebugStopMessagesClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->debugStopMessagesFlag = (ShowDebugStopMessages->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_DEBUG_STOP_MESSAGES, at3dData->debugStopMessagesFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::OutputHierarchyClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->outputHierarchyFlag = (OutputHierarchy->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::PauseAtOglSnapshotMsgClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->pauseAtOglSnapshotMsgFlag = (PauseAtOglSnapshotMsg->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_DISPLAY_OGL_SNAPSHOT_DEBUG_MSGS, at3dData->pauseAtOglSnapshotMsgFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmlGetDataFromDllClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->Get_KML_Data_From_DLL();
  Update_KML_Page();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ColladaUpAxisXClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->colladaOutputUpAxisFlag = COLLADA_UP_AXIS_X;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ColladaUpAxisYClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->colladaOutputUpAxisFlag = COLLADA_UP_AXIS_Y;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::ColladaUpAxisZClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->colladaOutputUpAxisFlag = COLLADA_UP_AXIS_Z;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::InvertAllLayerTransparenciesClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Invert_All_Layer_Transparencies();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::MakeAllLayersOpaqueClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Make_All_Layers_Opaque();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetWhiteForUvLayersClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  At3d_Set_White_For_UV_Layers();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::CheckFixTransparenciesClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->checkFixTransparenciesFlag = (CheckFixTransparencies->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::KmzInputModelFactorsClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->kmzInputModelFactorsFlag = (KmzInputModelFactors->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::AlwaysWriteColladaTransparencyClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->alwaysWriteColladaTransparencyFlag = (AlwaysWriteColladaTransparency->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetMayaVersionClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (MayaVersion->Text.IsEmpty())
  {
    at3dData->mayaVersion = AnsiString();
  }
  else
  {
    at3dData->mayaVersion = MayaVersion->Text;
  }

  At3d_Set_Maya_Version(at3dData->mayaVersion.c_str());
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseDirectxFramePrefixClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useDirectxFramePrefixFlag = (UseDirectxFramePrefix->Checked) ? 1 : 0;

  SetDirectxFramePrefixClick(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetDirectxFramePrefixClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (DirectxFramePrefix->Text.IsEmpty())
  {
    at3dData->directxFramePrefix = AnsiString();
  }
  else
  {
    at3dData->directxFramePrefix = DirectxFramePrefix->Text;
  }

  At3d_Set_DirectX_Frame_Prefix(at3dData->useDirectxFramePrefixFlag,
                                at3dData->directxFramePrefix.c_str());
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::UseDirectxMaterialPrefixClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->useDirectxMaterialPrefixFlag = (UseDirectxMaterialPrefix->Checked) ? 1 : 0;

  SetDirectxMaterialPrefixClick(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::SetDirectxMaterialPrefixClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (DirectxMaterialPrefix->Text.IsEmpty())
  {
    at3dData->directxMaterialPrefix = AnsiString();
  }
  else
  {
    at3dData->directxMaterialPrefix = DirectxMaterialPrefix->Text;
  }

  At3d_Set_DirectX_Material_Prefix(at3dData->useDirectxMaterialPrefixFlag,
                                   at3dData->directxMaterialPrefix.c_str());
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::VrmlMiterJointExtrusionClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->vrmlMiterJointExtrusionFlag = (VrmlMiterJointExtrusion->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::RenameEmbeddedTexturesClick(
      TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  AnsiString newName;

  if (NewEmbeddedName->Text.IsEmpty())
  {
    newName = AnsiString();
  }
  else
  {
    newName = NewEmbeddedName->Text;
  }

  At3d_Rename_Embedded_Textures(newName.c_str());
}
//---------------------------------------------------------------------------


void __fastcall TTest_At3dx_DllForm::ReadDoStuffWriteClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->Read_DoStuff_Write_File();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XamlPageClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xamlOutputClassFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XamlViewport3DClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xamlOutputClassFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::VColorDivideClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->saveCloudColorDivideFlag = (VColorDivide->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzOptions_NoneClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzHasOptionsFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzOptions_IDClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzHasOptionsFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzOptions_RGBClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzHasOptionsFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzOptions_SkipClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzHasOptionsFlag = 3;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzUseIdClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzUseIdFlag = (XyzUseId->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::XyzRgbIntegersClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->xyzRgbIntegersFlag = (XyzRgbIntegers->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::DxfAllLayersOnClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->dxfAllLayersOnFlag = (DxfAllLayersOn->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_DllForm::TouchScreenInputClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  at3dData->touchScreenInputFlag = (TouchScreenInput->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, at3dData->touchScreenInputFlag);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------

